@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004, 2005 Andreas Huggel <ahuggel@gmx.net>
  * 
  * This program is part of the Exiv2 distribution.
  *
@@ -22,7 +22,7 @@
   @file    actions.hpp
   @brief   Implements base class Task, TaskFactory and the various supported
            actions (derived from Task).
-  @version Exiv2-0_5_040913 1.11
+  @version $Rev: 575 $
   @author  Andreas Huggel (ahu)
            <a href="mailto:ahuggel@gmx.net">ahuggel@gmx.net</a>
   @date    11-Dec-03, ahu: created
@@ -37,11 +37,17 @@
 #include <string>
 #include <map>
 
+#include "exiv2.hpp"
+#include "image.hpp"
+#include "exif.hpp"
+#include "iptc.hpp"
+
 // *****************************************************************************
 // class declarations
 
 namespace Exiv2 {
     class ExifData;
+    class Image;
 }
 
 // *****************************************************************************
@@ -52,7 +58,7 @@ namespace Exiv2 {
 namespace Action {
 
     //! Enumerates all tasks
-    enum TaskType { none, adjust, print, rename, erase, extract, insert };
+    enum TaskType { none, adjust, print, rename, erase, extract, insert, modify };
 
 // *****************************************************************************
 // class definitions
@@ -67,6 +73,8 @@ namespace Action {
     public:
         //! Shortcut for an auto pointer.
         typedef std::auto_ptr<Task> AutoPtr;
+        //! Virtual destructor.
+        virtual ~Task() {}
         //! Virtual copy construction.
         AutoPtr clone() const;
         /*!
@@ -149,6 +157,8 @@ namespace Action {
         typedef std::auto_ptr<Print> AutoPtr;
         AutoPtr clone() const;
 
+        //! Print the Jpeg comment
+        int printComment();
         //! Print uninterpreted Iptc information 
         int printIptc();
         //! Print Exif summary information
@@ -220,11 +230,19 @@ namespace Action {
         /*!
           @brief Delete the thumbnail image, incl IFD1 metadata from the file. 
          */
-        int eraseThumbnail(Exiv2::ExifData& exifData) const; 
+        int eraseThumbnail(Exiv2::Image* image) const; 
         /*!
           @brief Erase the complete Exif data block from the file.
          */
-        int eraseExifData(Exiv2::ExifData& exifData) const;
+        int eraseExifData(Exiv2::Image* image) const;
+        /*!
+          @brief Erase all Iptc data from the file.
+         */
+        int eraseIptcData(Exiv2::Image* image) const;
+        /*!
+          @brief Erase Jpeg comment from the file.
+         */
+        int eraseComment(Exiv2::Image* image) const;
 
     private:
         virtual Erase* clone_() const;
@@ -248,12 +266,7 @@ namespace Action {
                  "-thumb" and the appropriate suffix (".jpg" or ".tif"), depending
                  on the format of the Exif thumbnail image.
          */
-        int writeThumbnail(const Exiv2::ExifData& exifData) const; 
-        /*!
-          @brief Write the Exif data to a file. The filename is composed by
-                 replacing the suffix of the image filename with ".exf".
-         */
-        int writeExifData(Exiv2::ExifData& exifData) const;
+        int writeThumbnail() const; 
 
     private:
         virtual Extract* clone_() const;
@@ -271,11 +284,45 @@ namespace Action {
         typedef std::auto_ptr<Insert> AutoPtr;
         AutoPtr clone() const;
 
+        /*!
+          @brief Insert a Jpeg thumbnail image from a file into file \em path.
+                 The filename of the thumbanail is expected to be the image
+                 filename (\em path) minus its suffix plus "-thumb.jpg".
+         */
+        int insertThumbnail(const std::string& path) const;
+
     private:
         virtual Insert* clone_() const;
 
     }; // class Insert
 
+    /*!
+      @brief %Modify the Exif data according to the commands in the 
+             modification table.
+     */
+    class Modify : public Task {
+    public:
+        virtual ~Modify() {}
+        virtual int run(const std::string& path);
+        typedef std::auto_ptr<Modify> AutoPtr;
+        AutoPtr clone() const;
+        Modify() {} 
+
+    private:
+        virtual Modify* clone_() const;
+        //! Copy contructor needed because of AutoPtr memeber 
+        Modify(const Modify& src) {} 
+
+        //! Add a metadatum according to \em modifyCmd 
+        void addMetadatum(const ModifyCmd& modifyCmd);
+        //! Set a metadatum according to \em modifyCmd 
+        void setMetadatum(const ModifyCmd& modifyCmd);
+        //! Delete a metadatum according to \em modifyCmd 
+        void delMetadatum(const ModifyCmd& modifyCmd);
+
+        Exiv2::Image::AutoPtr image_;        //!< Image to modify 
+    }; // class Modify
+
 }                                       // namespace Action 
 
 #endif                                  // #ifndef ACTIONS_HPP_
