@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004, 2005 Andreas Huggel <ahuggel@gmx.net>
  * 
  * This program is part of the Exiv2 distribution.
  *
@@ -20,13 +20,13 @@
  */
 /*
   File:      datasets.cpp
-  Version:   Exiv2-0_5_040913 1.4
-  Author(s): Brad Schick (brad) <schick@robotbattle.com>
+  Version:   $Rev: 560 $
+  Author(s): Brad Schick (brad) <brad@robotbattle.com>
   History:   24-Jul-04, brad: created
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) Exiv2-0_5_040913 1.4 datasets.cpp,v");
+EXIV2_RCSID("@(#) $Id: datasets.cpp 560 2005-04-17 11:51:32Z ahuggel $");
 
 // *****************************************************************************
 // included header files
@@ -34,24 +34,26 @@ EXIV2_RCSID("@(#) Exiv2-0_5_040913 1.4 datasets.cpp,v");
 #include "error.hpp"
 #include "types.hpp"
 #include "value.hpp"
+#include "metadatum.hpp"
 
 #include <iostream>
 #include <iomanip>
+#include <sstream>
 
 // *****************************************************************************
 // class member definitions
 namespace Exiv2 {
 
     DataSet::DataSet(
-        uint16 number, 
+        uint16_t number, 
         const char* name,
         const char* desc,
         bool mandatory,
         bool repeatable,
-        uint32 minbytes, 
-        uint32 maxbytes,
+        uint32_t minbytes, 
+        uint32_t maxbytes,
         TypeId type,
-        uint16 recordId,
+        uint16_t recordId,
         const char* photoshop
     )
         : number_(number), name_(name), desc_(desc), mandatory_(mandatory), 
@@ -61,7 +63,7 @@ namespace Exiv2 {
     }
 
     RecordInfo::RecordInfo(
-        uint16 recordId,
+        uint16_t recordId,
         const char* name,
         const char* desc
     )
@@ -154,6 +156,8 @@ namespace Exiv2 {
         DataSet(0xffff, "(Invalid)", "(Invalid)", false, false, 0, 0, Exiv2::unsignedShort, IptcDataSets::application2, "")
     };
 
+    static const DataSet unknownDataSet(0xffff, "Unknown dataset", "Unknown dataset", false, true, 0, 0xffffffff, Exiv2::string, IptcDataSets::invalidRecord, "Unknown dataset");
+
     // Dataset lookup lists.This is an array with pointers to one list per IIM4 Record. 
     // The record id is used as the index into the array.
     const DataSet* IptcDataSets::records_[] = {
@@ -162,9 +166,7 @@ namespace Exiv2 {
         0
     };
 
-    const char* IptcDataSets::familyName_ = "Iptc";
-
-    int IptcDataSets::dataSetIdx(uint16 number, uint16 recordId)
+    int IptcDataSets::dataSetIdx(uint16_t number, uint16_t recordId)
     {
         if( recordId != envelope && recordId != application2 ) return -1;
         const DataSet* dataSet = records_[recordId];
@@ -176,7 +178,7 @@ namespace Exiv2 {
         return idx;
     }
 
-    int IptcDataSets::dataSetIdx(const std::string& dataSetName, uint16 recordId)
+    int IptcDataSets::dataSetIdx(const std::string& dataSetName, uint16_t recordId)
     {
         if( recordId != envelope && recordId != application2 ) return -1;
         const DataSet* dataSet = records_[recordId];
@@ -188,106 +190,96 @@ namespace Exiv2 {
         return idx;
     }
 
-    TypeId IptcDataSets::dataSetType(uint16 number, uint16 recordId)
+    TypeId IptcDataSets::dataSetType(uint16_t number, uint16_t recordId)
     {
         int idx = dataSetIdx(number, recordId);
-        if (idx == -1) throw Error("No dataSet for record Id");
+        if (idx == -1) return unknownDataSet.type_;
         return records_[recordId][idx].type_;
     }
 
-    const char* IptcDataSets::dataSetName(uint16 number, uint16 recordId)
+    std::string IptcDataSets::dataSetName(uint16_t number, uint16_t recordId)
     {
         int idx = dataSetIdx(number, recordId);
-        if (idx == -1) throw Error("No dataSet for record Id");
-        return records_[recordId][idx].name_;
+        if (idx != -1) return records_[recordId][idx].name_;
+
+        std::ostringstream os;
+        os << "0x" << std::setw(4) << std::setfill('0') << std::right
+           << std::hex << number;
+        return os.str();
     }
 
-    const char* IptcDataSets::dataSetDesc(uint16 number, uint16 recordId)
+    const char* IptcDataSets::dataSetDesc(uint16_t number, uint16_t recordId)
     {
         int idx = dataSetIdx(number, recordId);
-        if (idx == -1) throw Error("No dataSet for record Id");
+        if (idx == -1) return unknownDataSet.desc_;
         return records_[recordId][idx].desc_;
     }
 
-    const char* IptcDataSets::dataSetPsName(uint16 number, uint16 recordId)
+    const char* IptcDataSets::dataSetPsName(uint16_t number, uint16_t recordId)
     {
         int idx = dataSetIdx(number, recordId);
-        if (idx == -1) throw Error("No dataSet for record Id");
+        if (idx == -1) return unknownDataSet.photoshop_;
         return records_[recordId][idx].photoshop_;
     }
 
-    bool IptcDataSets::dataSetRepeatable(uint16 number, uint16 recordId)
+    bool IptcDataSets::dataSetRepeatable(uint16_t number, uint16_t recordId)
     {
         int idx = dataSetIdx(number, recordId);
-        if (idx == -1) throw Error("No dataSet for record Id");
+        if (idx == -1) return unknownDataSet.repeatable_;
         return records_[recordId][idx].repeatable_;
     }
 
-    const char* IptcDataSets::recordName(uint16 recordId)
+    uint16_t IptcDataSets::dataSet(const std::string& dataSetName, 
+                                   uint16_t recordId)
     {
-        if( recordId != envelope && recordId != application2 ) {
-            throw Error("Unknown record");
+        uint16_t dataSet;
+        int idx = dataSetIdx(dataSetName, recordId);
+        if (idx != -1) {
+            // dataSetIdx checks the range of recordId
+            dataSet = records_[recordId][idx].number_;
         }
-        return recordInfo_[recordId].name_;
+        else {
+            if (!isHex(dataSetName, 4, "0x")) throw Error(4, dataSetName);
+            std::istringstream is(dataSetName);
+            is >> std::hex >> dataSet;
+        }
+        return dataSet;
     }
 
-    const char* IptcDataSets::recordDesc(uint16 recordId)
+    std::string IptcDataSets::recordName(uint16_t recordId)
     {
-        if( recordId != envelope && recordId != application2 ) {
-            throw Error("Unknown record");
+        if (recordId == envelope || recordId == application2) {
+            return recordInfo_[recordId].name_;            
+        }
+
+        std::ostringstream os;
+        os << "0x" << std::setw(4) << std::setfill('0') << std::right
+           << std::hex << recordId;
+        return os.str();
+    }
+
+    const char* IptcDataSets::recordDesc(uint16_t recordId)
+    {
+        if (recordId != envelope && recordId != application2) {
+            return unknownDataSet.desc_;
         }
         return recordInfo_[recordId].desc_;
     }
 
-    uint16 IptcDataSets::recordId(const std::string& recordName)
+    uint16_t IptcDataSets::recordId(const std::string& recordName)
     {
-        uint16 i;
+        uint16_t i;
         for (i = application2; i > 0; --i) {
             if (recordInfo_[i].name_ == recordName) break;
         }
+        if (i == 0) {
+            if (!isHex(recordName, 4, "0x")) throw Error(5, recordName);
+            std::istringstream is(recordName);
+            is >> std::hex >> i;
+        }
         return i;
     }
 
-    std::string IptcDataSets::makeKey(const DataSet& dataSet)
-    {
-        return std::string(familyName())
-            + "." + std::string(recordName(dataSet.recordId_)) 
-            + "." + dataSet.name_;
-    }
-
-    std::string IptcDataSets::makeKey(uint16 number, uint16 recordId)
-    {
-        return std::string(familyName())
-            + "." + std::string(recordName(recordId)) 
-            + "." + std::string(dataSetName(number, recordId));
-    }
-
-    // This 'database lookup' function returns a match if it exists
-    std::pair<uint16, uint16> IptcDataSets::decomposeKey(const std::string& key)
-    {
-        // Get the type, record name and dataSet name parts of the key
-        std::string::size_type pos1 = key.find('.');
-        if (pos1 == std::string::npos) throw Error("Invalid key");
-        std::string type = key.substr(0, pos1);
-        if (type != "Iptc") throw Error("Invalid key");
-        std::string::size_type pos0 = pos1 + 1;
-        pos1 = key.find('.', pos0);
-        if (pos1 == std::string::npos) throw Error("Invalid key");
-        std::string recordName = key.substr(pos0, pos1 - pos0);
-        if (recordName == "") throw Error("Invalid key");
-        std::string dataSetName = key.substr(pos1 + 1);
-        if (dataSetName == "") throw Error("Invalid key");
-
-        // Use the parts of the key to find dataSet and recordInfo
-        uint16 recId = recordId(recordName);
-        if (recId == invalidRecord) return std::make_pair((uint16)0xffff, invalidRecord);
-
-        int idx = dataSetIdx(dataSetName, recId);
-        if (idx == -1 ) return std::make_pair((uint16)0xffff, invalidRecord);
-
-        return std::make_pair(records_[recId][idx].number_, recId);
-    } // IptcDataSets::decomposeKey
-
     void IptcDataSets::dataSetList(std::ostream& os)
     {
         const int count = sizeof(records_)/sizeof(records_[0]);
@@ -299,11 +291,88 @@ namespace Exiv2 {
         }
     } // IptcDataSets::dataSetList
     
+    const char* IptcKey::familyName_ = "Iptc";
+
+    IptcKey::IptcKey(const std::string& key)
+        : key_(key)
+    {
+        decomposeKey();
+    }
+
+    IptcKey::IptcKey(uint16_t tag, uint16_t record)
+        : tag_(tag), record_(record)
+    {
+        makeKey();
+    }
+
+    IptcKey::IptcKey(const IptcKey& rhs)
+        : tag_(rhs.tag_), record_(rhs.record_), key_(rhs.key_)
+    {
+    }
+
+    IptcKey& IptcKey::operator=(const IptcKey& rhs)
+    {
+        if (this == &rhs) return *this;
+        Key::operator=(rhs);
+        tag_ = rhs.tag_;
+        record_ = rhs.record_;
+        key_ = rhs.key_;
+        return *this;
+    }
+
+    IptcKey::AutoPtr IptcKey::clone() const
+    {
+        return AutoPtr(clone_());
+    }
+
+    IptcKey* IptcKey::clone_() const
+    {
+        return new IptcKey(*this);
+    }
+
+    void IptcKey::decomposeKey()
+    {
+        // Get the family name, record name and dataSet name parts of the key
+        std::string::size_type pos1 = key_.find('.');
+        if (pos1 == std::string::npos) throw Error(6, key_);
+        std::string familyName = key_.substr(0, pos1);
+        if (familyName != std::string(familyName_)) {
+            throw Error(6, key_);
+        }
+        std::string::size_type pos0 = pos1 + 1;
+        pos1 = key_.find('.', pos0);
+        if (pos1 == std::string::npos) throw Error(6, key_);
+        std::string recordName = key_.substr(pos0, pos1 - pos0);
+        if (recordName == "") throw Error(6, key_);
+        std::string dataSetName = key_.substr(pos1 + 1);
+        if (dataSetName == "") throw Error(6, key_);
+
+        // Use the parts of the key to find dataSet and recordId
+        uint16_t recId = IptcDataSets::recordId(recordName);
+        uint16_t dataSet = IptcDataSets::dataSet(dataSetName, recId);
+
+        // Possibly translate hex name parts (0xabcd) to real names 
+        recordName = IptcDataSets::recordName(recId);
+        dataSetName = IptcDataSets::dataSetName(dataSet, recId);
+
+        tag_ = dataSet;
+        record_ = recId;
+        key_ = familyName + "." + recordName + "." + dataSetName;
+    } // IptcKey::decomposeKey
+
+    void IptcKey::makeKey()
+    {
+        key_ = std::string(familyName_)
+            + "." + IptcDataSets::recordName(record_)
+            + "." + IptcDataSets::dataSetName(tag_, record_);
+    }
+
     // *************************************************************************
     // free functions
 
     std::ostream& operator<<(std::ostream& os, const DataSet& dataSet) 
     {
+        IptcKey iptcKey(dataSet.number_, dataSet.recordId_);
         return os << dataSet.name_ << ", "
                   << std::dec << dataSet.number_ << ", "
                   << "0x" << std::setw(4) << std::setfill('0') 
@@ -313,7 +382,10 @@ namespace Exiv2 {
                   << dataSet.repeatable_ << ", "
                   << std::dec << dataSet.minbytes_ << ", "
                   << dataSet.maxbytes_ << ", "
-                  << IptcDataSets::makeKey(dataSet) << ", "
+                  << iptcKey.key() << ", "
+                  << TypeInfo::typeName(
+                      IptcDataSets::dataSetType(dataSet.number_, 
+                                                dataSet.recordId_)) << ", "
                   << dataSet.desc_;
     }
 
