@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004, 2005 Andreas Huggel <ahuggel@gmx.net>
  * 
  * This program is part of the Exiv2 distribution.
  *
@@ -21,7 +21,7 @@
 /*!
   @file    exiv2.hpp
   @brief   Defines class Params, used for the command line handling of exiv2
-  @version Exiv2-0_5_040913 1.4
+  @version $Rev: 567 $
   @author  Andreas Huggel (ahu)
            <a href="mailto:ahuggel@gmx.net">ahuggel@gmx.net</a>
   @date    08-Dec-03, ahu: created
@@ -32,6 +32,7 @@
 // *****************************************************************************
 // included header files
 #include "utils.hpp"
+#include "types.hpp"
 
 // + standard includes
 #include <string>
@@ -40,6 +41,33 @@
 
 // *****************************************************************************
 // class definitions
+
+//! Command identifiers
+enum CmdId { invalidCmdId, add, set, del };
+//! Metadata identifiers
+enum MetadataId { invalidMetadataId, iptc, exif };
+//! Structure for one parsed modification command 
+struct ModifyCmd {
+    //! C'tor
+    ModifyCmd() :
+        cmdId_(invalidCmdId), metadataId_(invalidMetadataId), 
+        typeId_(Exiv2::invalidTypeId), explicitType_(false) {}
+    CmdId cmdId_;                               //!< Command identifier
+    std::string key_;                           //!< Exiv2 key string
+    MetadataId metadataId_;                     //!< Metadata identifier 
+    Exiv2::TypeId typeId_;                      //!< Exiv2 type identifier
+    //! Flag to indicate if the type was explicitely specified (true)
+    bool explicitType_;
+    std::string value_;                         //!< Data 
+};
+//! Container for modification commands
+typedef std::vector<ModifyCmd> ModifyCmds;
+//! Structure to link command identifiers to strings
+struct CmdIdAndString {
+    CmdId cmdId_;                               //!< Commands identifier
+    std::string cmdString_;                     //!< Command string
+};
+
 /*!
   @brief Implements the command line handling for the program. 
 
@@ -79,6 +107,13 @@ private:
     std::string optstring_;
 
 public:
+    //! Container for command files
+    typedef std::vector<std::string> CmdFiles;
+    //! Container for commands from the command line
+    typedef std::vector<std::string> CmdLines;
+    //! Container to store filenames.
+    typedef std::vector<std::string> Files;
+
     /*!
       @brief Controls all access to the global Params instance.
       @return Reference to the global Params instance.
@@ -86,46 +121,47 @@ public:
     static Params& instance();
 
     //! Enumerates print modes
-    enum PrintMode { summary, interpreted, values, hexdump, iptc };
-    //! Enumerates delete targets
-    enum DelTarget { delExif, delThumb };
-    //! Enumerates extract targets
-    enum ExtractTarget { extExif, extThumb };
+    enum PrintMode { pmSummary, pmInterpreted, pmValues, pmHexdump, pmIptc, 
+                     pmComment };
+    //! Enumerates common targets, bitmap
+    enum CommonTarget { ctExif = 1, ctIptc = 2, ctComment = 4, ctThumb = 8 };
+    //! Enumerates the policies to handle existing files in rename action
+    enum FileExistsPolicy { overwritePolicy, renamePolicy, askPolicy };
 
     bool help_;                         //!< Help option flag.
     bool version_;                      //!< Version option flag.
     bool verbose_;                      //!< Verbose (talkative) option flag.
     bool force_;                        //!< Force overwrites flag. 
+    FileExistsPolicy fileExistsPolicy_; //!< What to do if file to rename exists.
     bool adjust_;                       //!< Adjustment flag.
     PrintMode printMode_;               //!< Print mode. 
-    DelTarget delTarget_;               //!< What to delete.
-    ExtractTarget extractTarget_;       //!< What to extract.
     //! %Action (integer rather than TaskType to avoid dependency).
     int  action_;
+    int  target_;                       //!< What common target to process.
 
     long adjustment_;                   //!< Adjustment in seconds.
     std::string format_;                //!< Filename format (-r option arg).
-
-    //! Container to store filenames.
-    typedef std::vector<std::string> Files;
-
+    CmdFiles cmdFiles_;                 //!< Names of the modification command files
+    CmdLines cmdLines_;                 //!< Commands from the command line
+    ModifyCmds modifyCmds_;             //!< Parsed modification commands
+    std::string directory_;             //!< Location for files to extract/insert
     Files files_;                       //!< List of non-option arguments.
 
 private:
     /*!
       @brief Default constructor. Note that optstring_ is initialized here.
-             Private to force instantiation through instance().
+             The c'tor is private to force instantiation through instance().
      */
-    Params() : optstring_(":hVvfa:r:p:d:e:"),
+    Params() : optstring_(":hVvfFa:r:p:d:e:i:m:M:l:"),
                help_(false), 
                version_(false),
                verbose_(false), 
                force_(false), 
+               fileExistsPolicy_(askPolicy),
                adjust_(false),
-               printMode_(summary),
-               delTarget_(delExif),
-               extractTarget_(extExif),
+               printMode_(pmSummary),
                action_(0),
+               target_(ctExif|ctIptc|ctComment),
                adjustment_(0),
                format_("%Y%m%d_%H%M%S"),
                first_(true) {}
@@ -133,6 +169,17 @@ private:
     //! Prevent copy-construction: not implemented.
     Params(const Params& rhs);
 
+    //! @name Helpers
+    //@{
+    int evalRename(const std::string& optarg);
+    int evalAdjust(const std::string& optarg);
+    int evalPrint(const std::string& optarg);
+    int evalDelete(const std::string& optarg);
+    int evalExtract(const std::string& optarg);
+    int evalInsert(const std::string& optarg);
+    int evalModify(int opt, const std::string& optarg);
+    //@}
+
     //! Pointer to the global Params object.
     static Params* instance_;
 
