@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004, 2005 Andreas Huggel <ahuggel@gmx.net>
  * 
  * This program is part of the Exiv2 distribution.
  *
@@ -20,7 +20,7 @@
  */
 /*
   File:      fujimn.cpp
-  Version:   Exiv2-0_5_040913 1.10
+  Version:   $Rev: 569 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
   History:   18-Feb-04, ahu: created
              07-Mar-04, ahu: isolated as a separate component
@@ -31,7 +31,7 @@
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) Exiv2-0_5_040913 1.10 fujimn.cpp,v");
+EXIV2_RCSID("@(#) $Id: fujimn.cpp 569 2005-05-28 05:48:43Z ahuggel $");
 
 // *****************************************************************************
 // included header files
@@ -53,35 +53,57 @@ EXIV2_RCSID("@(#) Exiv2-0_5_040913 1.10 fujimn.cpp,v");
 // class member definitions
 namespace Exiv2 {
 
-    const FujiMakerNote::RegisterMakerNote FujiMakerNote::register_;
+    //! @cond IGNORE
+    FujiMakerNote::RegisterMn::RegisterMn()
+    {
+        MakerNoteFactory::registerMakerNote("FUJIFILM", "*", createFujiMakerNote);
+        MakerNoteFactory::registerMakerNote(
+            fujiIfdId, MakerNote::AutoPtr(new FujiMakerNote));
+
+        ExifTags::registerMakerTagInfo(fujiIfdId, tagInfo_);
+    }
+    //! @endcond
 
     // Fujifilm MakerNote Tag Info
-    static const MakerNote::MnTagInfo fujiMnTagInfo[] = {
-        MakerNote::MnTagInfo(0x0000, "Version", "Fujifilm Makernote version"),
-        MakerNote::MnTagInfo(0x1000, "Quality", "Image quality setting"),
-        MakerNote::MnTagInfo(0x1001, "Sharpness", "Sharpness setting"),
-        MakerNote::MnTagInfo(0x1002, "WhiteBalance", "White balance setting"),
-        MakerNote::MnTagInfo(0x1003, "Color", "Chroma saturation setting"),
-        MakerNote::MnTagInfo(0x1004, "Tone", "Contrast setting"),
-        MakerNote::MnTagInfo(0x1010, "FlashMode", "Flash firing mode setting"),
-        MakerNote::MnTagInfo(0x1011, "FlashStrength", "Flash firing strength compensation setting"),
-        MakerNote::MnTagInfo(0x1020, "Macro", "Macro mode setting"),
-        MakerNote::MnTagInfo(0x1021, "FocusMode", "Focusing mode setting"),
-        MakerNote::MnTagInfo(0x1030, "SlowSync", "Slow synchro mode setting"),
-        MakerNote::MnTagInfo(0x1031, "PictureMode", "Picture mode setting"),
-        MakerNote::MnTagInfo(0x1100, "Continuous", "Continuous shooting or auto bracketing setting"),
-        MakerNote::MnTagInfo(0x1300, "BlurWarning", "Blur warning status"),
-        MakerNote::MnTagInfo(0x1301, "FocusWarning", "Auto Focus warning status"),
-        MakerNote::MnTagInfo(0x1302, "AeWarning", "Auto Exposure warning status"),
+    const TagInfo FujiMakerNote::tagInfo_[] = {
+        TagInfo(0x0000, "Version", "Fujifilm Makernote version", fujiIfdId, makerTags, undefined, printValue),
+        TagInfo(0x1000, "Quality", "Image quality setting", fujiIfdId, makerTags, asciiString, printValue),
+        TagInfo(0x1001, "Sharpness", "Sharpness setting", fujiIfdId, makerTags, unsignedShort, print0x1001),
+        TagInfo(0x1002, "WhiteBalance", "White balance setting", fujiIfdId, makerTags, unsignedShort, print0x1002),
+        TagInfo(0x1003, "Color", "Chroma saturation setting", fujiIfdId, makerTags, unsignedShort, print0x1003),
+        TagInfo(0x1004, "Tone", "Contrast setting", fujiIfdId, makerTags, unsignedShort, print0x1004),
+        TagInfo(0x1010, "FlashMode", "Flash firing mode setting", fujiIfdId, makerTags, unsignedShort, print0x1010),
+        TagInfo(0x1011, "FlashStrength", "Flash firing strength compensation setting", fujiIfdId, makerTags, signedRational, printValue),
+        TagInfo(0x1020, "Macro", "Macro mode setting", fujiIfdId, makerTags, unsignedShort, printOffOn),
+        TagInfo(0x1021, "FocusMode", "Focusing mode setting", fujiIfdId, makerTags, unsignedShort, print0x1021),
+        TagInfo(0x1022, "0x1022", "Unknown", fujiIfdId, makerTags, unsignedShort, printValue),
+        TagInfo(0x1030, "SlowSync", "Slow synchro mode setting", fujiIfdId, makerTags, unsignedShort, printOffOn),
+        TagInfo(0x1031, "PictureMode", "Picture mode setting", fujiIfdId, makerTags, unsignedShort, print0x1031),
+        TagInfo(0x1032, "0x1032", "Unknown", fujiIfdId, makerTags, unsignedShort, printValue),
+        TagInfo(0x1100, "Continuous", "Continuous shooting or auto bracketing setting", fujiIfdId, makerTags, unsignedShort, printOffOn),
+        TagInfo(0x1101, "0x1101", "Unknown", fujiIfdId, makerTags, unsignedShort, printValue),
+        TagInfo(0x1200, "0x1200", "Unknown", fujiIfdId, makerTags, unsignedShort, printValue),
+        TagInfo(0x1300, "BlurWarning", "Blur warning status", fujiIfdId, makerTags, unsignedShort, printOffOn),
+        TagInfo(0x1301, "FocusWarning", "Auto Focus warning status", fujiIfdId, makerTags, unsignedShort, printOffOn),
+        TagInfo(0x1302, "AeWarning", "Auto Exposure warning status", fujiIfdId, makerTags, unsignedShort, printOffOn),
         // End of list marker
-        MakerNote::MnTagInfo(0xffff, "(UnknownFujiMakerNoteTag)", "Unknown FujiMakerNote tag")
+        TagInfo(0xffff, "(UnknownFujiMakerNoteTag)", "Unknown FujiMakerNote tag", fujiIfdId, makerTags, invalidTypeId, printValue)
     };
 
     FujiMakerNote::FujiMakerNote(bool alloc)
-        : IfdMakerNote(fujiMnTagInfo, alloc), ifdItem_("Fujifilm")
+        : IfdMakerNote(fujiIfdId, alloc)
     {
         byteOrder_ = littleEndian;
         absOffset_ = false;
+        byte buf[] = {
+            'F', 'U', 'J', 'I', 'F', 'I', 'L', 'M', 0x0c, 0x00, 0x00, 0x00
+        };
+        readHeader(buf, 12, byteOrder_);
+    }
+
+    FujiMakerNote::FujiMakerNote(const FujiMakerNote& rhs)
+        : IfdMakerNote(rhs)
+    {
     }
 
     int FujiMakerNote::readHeader(const byte* buf,
@@ -110,38 +132,27 @@ namespace Exiv2 {
         return rc;
     }
 
-    FujiMakerNote* FujiMakerNote::clone(bool alloc) const 
+    FujiMakerNote::AutoPtr FujiMakerNote::create(bool alloc) const
     {
-        FujiMakerNote* pMakerNote = new FujiMakerNote(alloc);
-        assert(pMakerNote);
-        pMakerNote->readHeader(header_.pData_, header_.size_, byteOrder_);
-        return pMakerNote;
+        return AutoPtr(create_(alloc));
     }
 
-    std::ostream& FujiMakerNote::printTag(std::ostream& os, 
-                                          uint16 tag, 
-                                          const Value& value) const
+    FujiMakerNote* FujiMakerNote::create_(bool alloc) const 
     {
-        switch (tag) {
-        case 0x1020: // fallthrough
-        case 0x1030: // fallthrough
-        case 0x1100: // fallthrough
-        case 0x1300: // fallthrough
-        case 0x1301: // fallthrough
-        case 0x1302: printOffOn(os, value); break;
-        case 0x1001: print0x1001(os, value); break;
-        case 0x1002: print0x1002(os, value); break;
-        case 0x1003: print0x1003(os, value); break;
-        case 0x1004: print0x1004(os, value); break;
-        case 0x1010: print0x1010(os, value); break;
-        case 0x1021: print0x1021(os, value); break;
-        case 0x1031: print0x1031(os, value); break;
-        default:
-            // All other tags (known or unknown) go here
-            os << value;
-            break;
-        }
-        return os;
+        AutoPtr makerNote = AutoPtr(new FujiMakerNote(alloc));
+        assert(makerNote.get() != 0);
+        makerNote->readHeader(header_.pData_, header_.size_, byteOrder_);
+        return makerNote.release();
+    }
+
+    FujiMakerNote::AutoPtr FujiMakerNote::clone() const
+    {
+        return AutoPtr(clone_());
+    }
+
+    FujiMakerNote* FujiMakerNote::clone_() const 
+    {
+        return new FujiMakerNote(*this);
     }
 
     std::ostream& FujiMakerNote::printOffOn(std::ostream& os,
@@ -255,13 +266,13 @@ namespace Exiv2 {
 // *****************************************************************************
 // free functions
 
-    MakerNote* createFujiMakerNote(bool alloc,
-                                   const byte* buf, 
-                                   long len, 
-                                   ByteOrder byteOrder, 
-                                   long offset)
+    MakerNote::AutoPtr createFujiMakerNote(bool alloc,
+                                           const byte* buf, 
+                                           long len, 
+                                           ByteOrder byteOrder, 
+                                           long offset)
     {
-        return new FujiMakerNote(alloc);         
+        return MakerNote::AutoPtr(new FujiMakerNote(alloc));
     }
 
 }                                       // namespace Exiv2
