@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004, 2005 Andreas Huggel <ahuggel@gmx.net>
  * 
  * This program is part of the Exiv2 distribution.
  *
@@ -24,7 +24,7 @@
            in Appendix 4: Makernote of Fujifilm of the document 
            <a href="http://park2.wakwak.com/%7Etsuruzoh/Computer/Digicams/exif-e.html">
            Exif file format</a> by TsuruZoh Tachibanaya
-  @version Exiv2-0_5_040913 1.7
+  @version $Rev: 569 $
   @author  Andreas Huggel (ahu)
            <a href="mailto:ahuggel@gmx.net">ahuggel@gmx.net</a>
   @date    11-Feb-04, ahu: created
@@ -36,10 +36,12 @@
 // included header files
 #include "types.hpp"
 #include "makernote.hpp"
+#include "tags.hpp"
 
 // + standard includes
 #include <string>
 #include <iosfwd>
+#include <memory>
 
 // *****************************************************************************
 // namespace extensions
@@ -53,9 +55,10 @@ namespace Exiv2 {
 // free functions
 
     /*!
-      @brief Return a pointer to a newly created empty MakerNote initialized to
-             operate in the memory management model indicated.  The caller owns
-             this copy and is responsible to delete it!
+      @brief Return an auto-pointer to a newly created empty MakerNote
+             initialized to operate in the memory management model indicated.
+             The caller owns this copy and the auto-pointer ensures that it
+             will be deleted.
       
       @param alloc Memory management model for the new MakerNote. Determines if
              memory required to store data should be allocated and deallocated
@@ -69,14 +72,15 @@ namespace Exiv2 {
       @param offset Offset from the start of the TIFF header of the makernote
              buffer (not used).
       
-      @return A pointer to a newly created empty MakerNote. The caller owns
-             this copy and is responsible to delete it!
+      @return An auto-pointer to a newly created empty MakerNote. The caller
+             owns this copy and the auto-pointer ensures that it will be
+             deleted.
      */
-    MakerNote* createFujiMakerNote(bool alloc,
-                                   const byte* buf, 
-                                   long len, 
-                                   ByteOrder byteOrder, 
-                                   long offset);
+    MakerNote::AutoPtr createFujiMakerNote(bool alloc,
+                                           const byte* buf, 
+                                           long len, 
+                                           ByteOrder byteOrder, 
+                                           long offset);
 
 // *****************************************************************************
 // class definitions
@@ -84,6 +88,9 @@ namespace Exiv2 {
     //! MakerNote for Fujifilm cameras
     class FujiMakerNote : public IfdMakerNote {
     public:
+        //! Shortcut for a %FujiMakerNote auto pointer.
+        typedef std::auto_ptr<FujiMakerNote> AutoPtr;
+
         //! @name Creators
         //@{
         /*!
@@ -91,6 +98,8 @@ namespace Exiv2 {
                  is required for the makernote entries.
          */
         FujiMakerNote(bool alloc =true);
+        //! Copy constructor
+        FujiMakerNote(const FujiMakerNote& rhs);
         //! Virtual destructor
         virtual ~FujiMakerNote() {}
         //@}
@@ -105,12 +114,8 @@ namespace Exiv2 {
         //! @name Accessors
         //@{
         int checkHeader() const;
-        FujiMakerNote* clone(bool alloc =true) const;
-        //! Return the name of the makernote item ("Fujifilm")
-        std::string ifdItem() const { return ifdItem_; }
-        std::ostream& printTag(std::ostream& os,
-                               uint16 tag, 
-                               const Value& value) const;
+        AutoPtr create(bool alloc =true) const;
+        AutoPtr clone() const;
         //@}
 
         //! @name Print functions for Fujifilm %MakerNote tags 
@@ -133,35 +138,25 @@ namespace Exiv2 {
         static std::ostream& print0x1031(std::ostream& os, const Value& value);
         //@}
 
-    private:
-        //! Structure used to auto-register the MakerNote.
-        struct RegisterMakerNote {
-            //! Default constructor
-            RegisterMakerNote() 
-            {
-                MakerNoteFactory& mnf = MakerNoteFactory::instance();
-                mnf.registerMakerNote("FUJIFILM", "*", createFujiMakerNote); 
-            }
+        //! @cond IGNORE
+        // Public only so that we can create a static instance
+        struct RegisterMn {
+            RegisterMn();
         };
-        /*!
-          The static member variable is initialized before main (see note) and
-          will in the process register the MakerNote class. (Remember the
-          definition of the variable in the implementation file!)
+        //! @endcond
 
-          @note The standard says that, if no function is explicitly called ever
-                in a module, then that module's static data might be never
-                initialized. This clause was introduced to allow dynamic link
-                libraries. The idea is, with this clause the loader is not
-                forced to eagerly load all modules, but load them only on
-                demand.
-         */
-        static const RegisterMakerNote register_; 
+    private:
+        //! Internal virtual create function.
+        FujiMakerNote* create_(bool alloc =true) const;
+        //! Internal virtual copy constructor.
+        FujiMakerNote* clone_() const;
 
-        //! The item name (second part of the key) used for makernote tags
-        std::string ifdItem_;
+        //! Tag information
+        static const TagInfo tagInfo_[];
 
     }; // class FujiMakerNote
 
+    static FujiMakerNote::RegisterMn registerFujiMakerNote;
 }                                       // namespace Exiv2
 
 #endif                                  // #ifndef FUJIMN_HPP_
