@@ -28,18 +28,16 @@
  #pragma alloca
 #endif
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
+#ifdef _MSC_VER
+# include "exv_msvc.h"
 #else
-# ifdef _MSC_VER
-#  include <config_win32.h>
-# endif
+# include "exv_conf.h"
 #endif
 
 #ifdef __GNUC__
 #define alloca __builtin_alloca
 #else /* not __GNUC__ */
-#if defined (HAVE_ALLOCA_H) || (defined(sparc) && (defined(sun) || (!defined(USG) && !defined(SVR4) && !defined(__svr4__))))
+#if defined (EXV_HAVE_ALLOCA_H) || (defined(sparc) && (defined(sun) || (!defined(USG) && !defined(SVR4) && !defined(__svr4__))))
 #include <alloca.h>
 #else
 #ifndef _AIX
