@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004, 2005 Andreas Huggel <ahuggel@gmx.net>
  * 
  * This program is part of the Exiv2 distribution.
  *
@@ -20,14 +20,14 @@
  */
 /*
   File:      ifd.cpp
-  Version:   Exiv2-0_5_040913 1.25
+  Version:   $Rev: 562 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
   History:   26-Jan-04, ahu: created
              11-Feb-04, ahu: isolated as a component
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) Exiv2-0_5_040913 1.25 ifd.cpp,v");
+EXIV2_RCSID("@(#) $Id: ifd.cpp 562 2005-04-20 18:16:29Z ahuggel $");
 
 // *****************************************************************************
 // included header files
@@ -50,30 +50,39 @@ EXIV2_RCSID("@(#) Exiv2-0_5_040913 1.25 ifd.cpp,v");
 namespace Exiv2 {
 
     Entry::Entry(bool alloc)
-        : alloc_(alloc), ifdId_(ifdIdNotSet), idx_(0), pMakerNote_(0), 
-          tag_(0), type_(0), count_(0), offset_(0), size_(0), pData_(0)
+        : alloc_(alloc), ifdId_(ifdIdNotSet), idx_(0), 
+          tag_(0), type_(0), count_(0), offset_(0), size_(0), pData_(0),
+          sizeDataArea_(0), pDataArea_(0)
     {
     }
 
     Entry::~Entry()
     {
-        if (alloc_) delete[] pData_;
-        // do *not* delete the MakerNote
+        if (alloc_) {
+            delete[] pData_;
+            delete[] pDataArea_;
+        }
     }
 
     Entry::Entry(const Entry& rhs)
         : alloc_(rhs.alloc_), ifdId_(rhs.ifdId_), idx_(rhs.idx_),
-          pMakerNote_(rhs.pMakerNote_), tag_(rhs.tag_), type_(rhs.type_), 
-          count_(rhs.count_), offset_(rhs.offset_), size_(rhs.size_), pData_(0)
+          tag_(rhs.tag_), type_(rhs.type_), 
+          count_(rhs.count_), offset_(rhs.offset_), size_(rhs.size_), pData_(0),
+          sizeDataArea_(rhs.sizeDataArea_), pDataArea_(0)
     {
         if (alloc_) {
             if (rhs.pData_) {
                 pData_ = new byte[rhs.size()];
                 memcpy(pData_, rhs.pData_, rhs.size());
             }
+            if (rhs.pDataArea_) {
+                pDataArea_ = new byte[rhs.sizeDataArea()];
+                memcpy(pDataArea_, rhs.pDataArea_, rhs.sizeDataArea());
+            }
         }
         else {
             pData_ = rhs.pData_;
+            pDataArea_ = rhs.pDataArea_;
         }
     }
 
@@ -83,12 +92,12 @@ namespace Exiv2 {
         alloc_ = rhs.alloc_;
         ifdId_ = rhs.ifdId_;
         idx_ = rhs.idx_;
-        pMakerNote_ = rhs.pMakerNote_;
         tag_ = rhs.tag_;
         type_ = rhs.type_;
         count_ = rhs.count_;
         offset_ = rhs.offset_;
         size_ = rhs.size_;
+        sizeDataArea_ = rhs.sizeDataArea_;
         if (alloc_) {
             delete[] pData_;
             pData_ = 0;
@@ -96,14 +105,21 @@ namespace Exiv2 {
                 pData_ = new byte[rhs.size()];
                 memcpy(pData_, rhs.pData_, rhs.size());
             }
+            delete[] pDataArea_;
+            pDataArea_ = 0;
+            if (rhs.pDataArea_) {
+                pDataArea_ = new byte[rhs.sizeDataArea()];
+                memcpy(pDataArea_, rhs.pDataArea_, rhs.sizeDataArea());
+            }
         }
         else {
             pData_ = rhs.pData_;
+            pDataArea_ = rhs.pDataArea_;
         }
         return *this;
     } // Entry::operator=
 
-    void Entry::setValue(uint32 data, ByteOrder byteOrder)
+    void Entry::setValue(uint32_t data, ByteOrder byteOrder)
     {
         if (pData_ == 0 || size_ < 4) {
             assert(alloc_);
@@ -117,13 +133,11 @@ namespace Exiv2 {
         count_ = 1;
     }
 
-    void Entry::setValue(uint16 type, uint32 count, const byte* buf, long len)
+    void Entry::setValue(uint16_t type, uint32_t count, const byte* buf, long len)
     {
         long dataSize = count * TypeInfo::typeSize(TypeId(type));
         // No minimum size requirement, but make sure the buffer can hold the data
-        if (len < dataSize) {
-            throw Error("Size too small");
-        }
+        if (len < dataSize) throw Error(24, tag(), dataSize, len);
         if (alloc_) {
             delete[] pData_;
             pData_ = new byte[len];
@@ -139,7 +153,7 @@ namespace Exiv2 {
             }
             else {
                 // Overwrite existing data if it fits into the buffer
-                if (dataSize > size_) throw Error("Value too large");
+                if (size_ < dataSize) throw Error(24, tag(), dataSize, size_);
                 memset(pData_, 0x0, size_);
                 memcpy(pData_, buf, dataSize);
                 // do not change size_
@@ -149,33 +163,116 @@ namespace Exiv2 {
         count_ = count;
     } // Entry::setValue
 
-    const byte* Entry::component(uint32 n) const
+    void Entry::setDataArea(const byte* buf, long len)
+    {
+        if (alloc_) {
+            delete[] pDataArea_;
+            pDataArea_ = new byte[len];
+            memcpy(pDataArea_, buf, len);
+            sizeDataArea_ = len;
+        }
+        else {
+            if (sizeDataArea_ == 0) {
+                // Set the data area pointer of a virgin entry
+                pDataArea_ = const_cast<byte*>(buf);
+                sizeDataArea_ = len;
+            }
+            else {
+                // Overwrite existing data if it fits into the buffer
+                if (sizeDataArea_ < len) {
+                    throw Error(25, tag(), sizeDataArea_, len);
+                }
+                memset(pDataArea_, 0x0, sizeDataArea_);
+                memcpy(pDataArea_, buf, len);
+                // do not change sizeDataArea_
+            }
+        }
+    } // Entry::setDataArea
+
+    void Entry::setDataAreaOffsets(uint32_t offset, ByteOrder byteOrder)
+    {
+        for (uint32_t i = 0; i < count(); ++i) {
+            byte* buf = pData_ + i * typeSize();
+            switch(TypeId(type())) {
+            case unsignedShort: {
+                uint16_t d = getUShort(buf, byteOrder);
+                if (d + offset > 0xffff) throw Error(26);
+                us2Data(buf, d + static_cast<uint16_t>(offset), byteOrder);
+                break;
+            }
+            case unsignedLong: {
+                ul2Data(buf, getULong(buf, byteOrder) + offset, byteOrder); 
+                break;
+            }
+            case unsignedRational: {
+                URational d = getURational(buf, byteOrder);
+                d.first = d.first + offset * d.second;
+                ur2Data(buf, d, byteOrder);
+                break;
+            }
+            case signedShort: {
+                int16_t d = getShort(buf, byteOrder);
+                if (d + static_cast<int32_t>(offset) > 0xffff) throw Error(26);
+                s2Data(buf, d + static_cast<int16_t>(offset), byteOrder);
+                break;
+            }
+            case signedLong: {
+                int32_t d = getLong(buf, byteOrder);
+                l2Data(buf, d + static_cast<int32_t>(offset), byteOrder);
+                break;
+            }
+            case signedRational: {
+                Rational d = getRational(buf, byteOrder);
+                d.first = d.first + static_cast<int32_t>(offset) * d.second;
+                r2Data(buf, d, byteOrder);
+                break;
+            }
+            default:
+                throw Error(27);
+                break;
+            }
+        }
+    } // Entry::setDataAreaOffsets
+
+    void Entry::updateBase(byte* pOldBase, byte* pNewBase)
+    {
+        if (!alloc_) {
+            if (pDataArea_) {
+                pDataArea_ = pDataArea_ - pOldBase + pNewBase;
+            }
+            if (pData_) {
+                pData_ = pData_ - pOldBase + pNewBase;
+            }
+        }
+    } // Entry::updateBase
+
+    const byte* Entry::component(uint32_t n) const
     {
         if (n >= count()) return 0;
         return data() + n * typeSize();
     } // Entry::component
 
     Ifd::Ifd(IfdId ifdId)
-        : alloc_(true), ifdId_(ifdId), offset_(0), dataOffset_(0),
-          pNext_(0), next_(0)
+        : alloc_(true), ifdId_(ifdId), pBase_(0), offset_(0), 
+          dataOffset_(0), hasNext_(true), pNext_(0), next_(0)
     {
         pNext_ = new byte[4];
         memset(pNext_, 0x0, 4);
     }
 
-    Ifd::Ifd(IfdId ifdId, uint32 offset)
-        : alloc_(true), ifdId_(ifdId), offset_(offset), dataOffset_(0),
-          pNext_(0), next_(0)
+    Ifd::Ifd(IfdId ifdId, long offset)
+        : alloc_(true), ifdId_(ifdId), pBase_(0), offset_(offset), 
+          dataOffset_(0), hasNext_(true), pNext_(0), next_(0)
     {
         pNext_ = new byte[4];
         memset(pNext_, 0x0, 4);
     }
 
-    Ifd::Ifd(IfdId ifdId, uint32 offset, bool alloc)
-        : alloc_(alloc), ifdId_(ifdId), offset_(offset), dataOffset_(0),
-          pNext_(0), next_(0)
+    Ifd::Ifd(IfdId ifdId, long offset, bool alloc, bool hasNext)
+        : alloc_(alloc), ifdId_(ifdId), pBase_(0), offset_(offset), 
+          dataOffset_(0), hasNext_(hasNext), pNext_(0), next_(0)
     {
-        if (alloc_) {
+        if (alloc_ && hasNext_) {
             pNext_ = new byte[4];
             memset(pNext_, 0x0, 4);
         }
@@ -183,22 +280,27 @@ namespace Exiv2 {
 
     Ifd::~Ifd()
     {
-        if (alloc_) delete[] pNext_;
+        // do not delete pBase_
+        if (alloc_ && hasNext_) delete[] pNext_;
     }
 
     Ifd::Ifd(const Ifd& rhs)
         : alloc_(rhs.alloc_), entries_(rhs.entries_), ifdId_(rhs.ifdId_),
-          offset_(rhs.offset_), dataOffset_(rhs.dataOffset_),
-          pNext_(rhs.pNext_), next_(rhs.next_)
+          pBase_(rhs.pBase_), offset_(rhs.offset_), dataOffset_(rhs.dataOffset_), 
+          hasNext_(rhs.hasNext_), pNext_(rhs.pNext_), next_(rhs.next_)
     {
-        if (alloc_ && rhs.pNext_) {
+        if (alloc_ && hasNext_) {
             pNext_ = new byte[4];
-            memcpy(pNext_, rhs.pNext_, 4); 
+            memset(pNext_, 0x0, 4);
+            if (rhs.pNext_) memcpy(pNext_, rhs.pNext_, 4); 
         }
     }
 
     int Ifd::read(const byte* buf, long len, ByteOrder byteOrder, long offset)
     {
+        // Todo: This is a hack to work around bug #424 - fix it properly!
+        if (ifdId_ == olympusIfdId) len = 65535;
+
         int rc = 0;
         long o = 0;
         Ifd::PreEntries preEntries;
@@ -224,12 +326,12 @@ namespace Exiv2 {
                 pe.count_ = getULong(buf + o + 4, byteOrder);
                 pe.size_ = pe.count_ * TypeInfo::typeSize(TypeId(pe.type_));
                 pe.offsetLoc_ = o + 8;
-                pe.offset_ = pe.size_ > 4 ? getULong(buf + o + 8, byteOrder) : 0;
+                pe.offset_ = pe.size_ > 4 ? getLong(buf + o + 8, byteOrder) : 0;
                 preEntries.push_back(pe);
                 o += 12;
             }
         }
-        if (rc == 0) {
+        if (rc == 0 && hasNext_) {
             if (len < o + 4) {
                 // Todo: How to handle debug output like this
                 std::cerr << "Error: " << ExifTags::ifdName(ifdId_) 
@@ -261,18 +363,21 @@ namespace Exiv2 {
             if (i->size_ > 4) {
                 if (offset_ == 0) {
                     // Set the 'guessed' IFD offset
-                    offset_ = i->offset_ - size();
+                    offset_ = i->offset_ 
+		      - (2 + 12 * static_cast<long>(preEntries.size()) 
+                         + (hasNext_ ? 4 : 0));
                 }
                 // Set the offset of the first data entry outside of the IFD
-                if (static_cast<unsigned long>(len) < i->offset_ - offset_) {
+                if (i->offset_ - offset_ >= len) {
                     // Todo: How to handle debug output like this
                     std::cerr << "Error: Offset of the 1st data entry of " 
                               << ExifTags::ifdName(ifdId_) 
                               << " is out of bounds:\n"
-                              << " Offset = " << i->offset_ - offset_
+                              << " Offset = 0x" << std::setw(8) 
+                              << std::setfill('0') << std::hex 
+                              << i->offset_ - offset_
                               << ", exceeds buffer size by "
-                              << i->offset_ - offset_ 
-                                 - static_cast<unsigned long>(len)
+                              << std::dec << i->offset_ - len
                               << " Bytes\n";
                     rc = 6;
                 }
@@ -294,19 +399,20 @@ namespace Exiv2 {
                 e.setIfdId(ifdId_);
                 e.setIdx(++idx);
                 e.setTag(i->tag_);
-                uint32 tmpOffset = 
+                long tmpOffset = 
                     i->size_ > 4 ? i->offset_ - offset_ : i->offsetLoc_;
-                if (static_cast<unsigned long>(len) < tmpOffset + i->size_) {
+                if (tmpOffset + i->size_ > len) {
                     // Todo: How to handle debug output like this
                     std::cerr << "Warning: Upper boundary of data for " 
                               << ExifTags::ifdName(ifdId_) 
                               << " entry " << static_cast<int>(i - begin) 
                               << " is out of bounds:\n"
-                              << " Offset = " << tmpOffset
-                              << ", size = " << i->size_ 
+                              << " Offset = 0x" << std::setw(8) 
+                              << std::setfill('0') << std::hex 
+                              << tmpOffset
+                              << ", size = " << std::dec << i->size_ 
                               << ", exceeds buffer size by "
-                              << tmpOffset + i->size_ 
-                                 - static_cast<unsigned long>(len)
+                              << tmpOffset + i->size_ - len
                               << " Bytes; Truncating the data.\n";
                     // Truncate the entry
                     i->size_ = 0;
@@ -321,6 +427,7 @@ namespace Exiv2 {
                 this->add(e);
             }
         }
+        if (!alloc_) pBase_ = const_cast<byte*>(buf) - offset_;
         if (rc) this->clear();
 
         return rc;
@@ -338,13 +445,13 @@ namespace Exiv2 {
                             FindEntryByIdx(idx));
     }
 
-    Ifd::const_iterator Ifd::findTag(uint16 tag) const 
+    Ifd::const_iterator Ifd::findTag(uint16_t tag) const 
     {
         return std::find_if(entries_.begin(), entries_.end(),
                             FindEntryByTag(tag));
     }
 
-    Ifd::iterator Ifd::findTag(uint16 tag)
+    Ifd::iterator Ifd::findTag(uint16_t tag)
     {
         return std::find_if(entries_.begin(), entries_.end(),
                             FindEntryByTag(tag));
@@ -356,7 +463,7 @@ namespace Exiv2 {
     }
 
     int Ifd::readSubIfd(
-        Ifd& dest, const byte* buf, long len, ByteOrder byteOrder, uint16 tag
+        Ifd& dest, const byte* buf, long len, ByteOrder byteOrder, uint16_t tag
     ) const
     {
         int rc = 0;
@@ -375,25 +482,38 @@ namespace Exiv2 {
 
     long Ifd::copy(byte* buf, ByteOrder byteOrder, long offset)
     {
+        if (entries_.size() == 0 && next_ == 0) return 0;
         if (offset != 0) offset_ = offset;
 
         // Add the number of entries to the data buffer
-        us2Data(buf, static_cast<uint16>(entries_.size()), byteOrder);
+        us2Data(buf, static_cast<uint16_t>(entries_.size()), byteOrder);
         long o = 2;
 
         // Add all directory entries to the data buffer
         long dataSize = 0;
+        long dataAreaSize = 0;
+        long totalDataSize = 0;
         const iterator b = entries_.begin();
         const iterator e = entries_.end();
-        iterator i = b;
-        for (; i != e; ++i) {
+        iterator i;
+        for (i = b; i != e; ++i) {
+            if (i->size() > 4) {
+                totalDataSize += i->size();
+            }
+        }
+        for (i = b; i != e; ++i) {
             us2Data(buf + o, i->tag(), byteOrder);
             us2Data(buf + o + 2, i->type(), byteOrder);
             ul2Data(buf + o + 4, i->count(), byteOrder);
+            if (i->sizeDataArea() > 0) { 
+                long dataAreaOffset = offset_+size()+totalDataSize+dataAreaSize;
+                i->setDataAreaOffsets(dataAreaOffset, byteOrder);
+                dataAreaSize += i->sizeDataArea();
+            }
             if (i->size() > 4) {
                 // Set the offset of the entry, data immediately follows the IFD
                 i->setOffset(size() + dataSize);
-                ul2Data(buf + o + 8, offset_ + i->offset(), byteOrder);
+                l2Data(buf + o + 8, offset_ + i->offset(), byteOrder);
                 dataSize += i->size();
             }
             else {
@@ -404,14 +524,16 @@ namespace Exiv2 {
             o += 12;
         }
 
-        // Add the offset to the next IFD to the data buffer
-        if (pNext_) {
-            memcpy(buf + o, pNext_, 4);
+        if (hasNext_) {
+            // Add the offset to the next IFD to the data buffer
+            if (pNext_) {
+                memcpy(buf + o, pNext_, 4);
+            }
+            else {
+                memset(buf + o, 0x0, 4);
+            }
+            o += 4;
         }
-        else {
-            memset(buf + o, 0x0, 4);
-        }
-        o += 4;
 
         // Add the data of all IFD entries to the data buffer
         for (i = b; i != e; ++i) {
@@ -421,28 +543,41 @@ namespace Exiv2 {
             }
         }
 
+        // Add all data areas to the data buffer
+        for (i = b; i != e; ++i) {
+            if (i->sizeDataArea() > 0) {
+                memcpy(buf + o, i->dataArea(), i->sizeDataArea());
+                o += i->sizeDataArea();
+            }
+        }
+
         return o;
     } // Ifd::copy
 
     void Ifd::clear()
     {
         entries_.clear();
-        if (alloc_) {
-            memset(pNext_, 0x0, 4);
-        }
-        else {
-            pNext_ = 0;
-        }
-        next_ = 0;
         offset_ = 0;
         dataOffset_ = 0;
+        if (hasNext_) {
+            if (alloc_) {
+                memset(pNext_, 0x0, 4);
+            }
+            else {
+                pBase_ = 0;
+                pNext_ = 0;
+            }
+            next_ = 0;
+        }
     } // Ifd::clear
 
-    void Ifd::setNext(uint32 next, ByteOrder byteOrder)
+    void Ifd::setNext(uint32_t next, ByteOrder byteOrder)
     {
-        assert(pNext_);
-        ul2Data(pNext_, next, byteOrder);
-        next_ = next;
+        if (hasNext_) {
+            assert(pNext_);
+            ul2Data(pNext_, next, byteOrder);
+            next_ = next;
+        }
     }
 
     void Ifd::add(const Entry& entry)
@@ -453,7 +588,7 @@ namespace Exiv2 {
         entries_.push_back(entry);
     }
 
-    int Ifd::erase(uint16 tag)
+    int Ifd::erase(uint16_t tag)
     {
         int idx = 0;
         iterator pos = findTag(tag);
@@ -469,10 +604,27 @@ namespace Exiv2 {
         return entries_.erase(pos);
     }
 
+    byte* Ifd::updateBase(byte* pNewBase)
+    {
+        byte *pOld = 0;
+        if (!alloc_) {
+            iterator end = this->end();
+            for (iterator pos = begin(); pos != end; ++pos) {
+                pos->updateBase(pBase_, pNewBase);
+            }
+            if (hasNext_) {
+                pNext_ = pNext_ - pBase_ + pNewBase;
+            }
+            pOld = pBase_;
+            pBase_ = pNewBase;
+        }
+        return pOld;
+    }
+
     long Ifd::size() const
     {
-        if (entries_.size() == 0) return 0;
-        return static_cast<long>(2 + 12 * entries_.size() + 4); 
+        if (entries_.size() == 0 && next_ == 0) return 0;
+        return static_cast<long>(2 + 12 * entries_.size() + (hasNext_ ? 4 : 0)); 
     }
 
     long Ifd::dataSize() const
@@ -481,6 +633,7 @@ namespace Exiv2 {
         const_iterator end = this->end();
         for (const_iterator i = begin(); i != end; ++i) {
             if (i->size() > 4) dataSize += i->size();
+            dataSize += i->sizeDataArea();
         }
         return dataSize;
     }
@@ -504,7 +657,7 @@ namespace Exiv2 {
         for (; i != e; ++i) {
             std::ostringstream offset;
             if (i->size() > 4) {
-                offset << " 0x" << std::setw(8) << std::setfill('0') 
+                offset << " 0x" << std::setw(8) << std::setfill('0')
                        << std::hex << std::right << i->offset();
             }
             else {
@@ -526,9 +679,11 @@ namespace Exiv2 {
                << "  " << offset.str()
                << "\n";
         }
-        os << prefix << "Next IFD: 0x" 
-           << std::setw(8) << std::setfill('0') << std::hex
-           << std::right << next() << "\n";
+        if (hasNext_) {
+            os << prefix << "Next IFD: 0x" 
+               << std::setw(8) << std::setfill('0') << std::hex
+               << std::right << next() << "\n";
+        }
         // Print data of IFD entries 
         for (i = b; i != e; ++i) {
             if (i->size() > 4) {
