@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004, 2005 Andreas Huggel <ahuggel@gmx.net>
  * 
  * This program is part of the Exiv2 distribution.
  *
@@ -21,7 +21,7 @@
 /*!
   @file    ifd.hpp
   @brief   Encoding and decoding of IFD (%Image File Directory) data
-  @version Exiv2-0_5_040913 1.20
+  @version $Rev: 562 $
   @author  Andreas Huggel (ahu)
            <a href="mailto:ahuggel@gmx.net">ahuggel@gmx.net</a>
   @date    09-Jan-04, ahu: created<BR>
@@ -45,7 +45,7 @@ namespace Exiv2 {
 
 // *****************************************************************************
 // class declarations
-    class MakerNote;
+    class Ifd;
 
 // *****************************************************************************
 // class definitions
@@ -64,7 +64,7 @@ namespace Exiv2 {
           data if alloc is true (the default), otherwise it remembers
           just the pointers into a read and writeable data buffer which
           it doesn't allocate or delete.
-        */ 
+         */ 
         explicit Entry(bool alloc =true);
         //! Destructor
         ~Entry();
@@ -77,15 +77,13 @@ namespace Exiv2 {
         //! Assignment operator
         Entry& operator=(const Entry& rhs);
         //! Set the tag
-        void setTag(uint16 tag) { tag_ = tag; }
+        void setTag(uint16_t tag) { tag_ = tag; }
         //! Set the IFD id
         void setIfdId(IfdId ifdId) { ifdId_ = ifdId; }
         //! Set the index (unique id of an entry within one IFD)
         void setIdx(int idx) { idx_ = idx; }
-        //! Set the pointer to the MakerNote
-        void setMakerNote(MakerNote* makerNote) { pMakerNote_ = makerNote; }
         //! Set the offset. The offset is relative to the start of the IFD.
-        void setOffset(uint32 offset) { offset_ = offset; }
+        void setOffset(long offset) { offset_ = offset; }
         /*!
           @brief Set the value of the entry to a single unsigned long component,
                  i.e., set the type of the entry to unsigned long, number of
@@ -98,8 +96,11 @@ namespace Exiv2 {
           contains a pointer to the Exif IFD). 
           <BR>This method cannot be used to set the value of a newly created
           %Entry in non-alloc mode.
-        */
-        void setValue(uint32 data, ByteOrder byteOrder);
+
+          @note This method is now deprecated, use data area related methods
+                instead.
+         */
+        void setValue(uint32_t data, ByteOrder byteOrder);
         /*!
           @brief Set type, count, the data buffer and its size.
 
@@ -115,26 +116,73 @@ namespace Exiv2 {
           will be as indicated in the size argument. I.e., it is possible to
           allocate (set) a data buffer larger than required to hold count
           components of the given type.
-          
+
           @param type The type of the data.
           @param count Number of components in the buffer.
           @param data Pointer to the data buffer.
           @param size Size of the desired data buffer in bytes.
-          @throw Error ("Value too large") if no memory allocation is allowed 
+          @throw Error if no memory allocation is allowed 
                  and the size of the data buffer is larger than the existing 
-                 data buffer of the entry.<BR>
-          @throw Error ("Size too small") if size is not large enough to hold
+                 data buffer of the entry or if size is not large enough to hold
                  count components of the given type.
          */
-        void setValue(uint16 type, uint32 count, const byte* data, long size);
+        void setValue(uint16_t type, uint32_t count, const byte* data, long size);
+        /*!
+          @brief Set the data area. Memory management as for 
+          setValue(uint16_t, uint32_t, const byte*, long)
+
+          For certain tags the regular value of an IFD entry is an offset to a
+          data area outside of the IFD. Examples are Exif tag 0x8769 in IFD0
+          (Exif.Image.ExifTag) or tag 0x0201 in IFD1
+          (Exif.Thumbnail.JPEGInterchangeFormat). The offset of ExifTag points
+          to a data area containing the Exif IFD. That of JPEGInterchangeFormat
+          contains the JPEG thumbnail image.  
+          This method sets the data area of a tag in accordance with the memory
+          allocation mode.
+
+          @param buf Pointer to the data area.
+          @param len Size of the data area.
+          
+          @throw Error in non-alloc mode, if there already is a dataarea but the 
+                 size of the existing dataarea is not large enough for the 
+                 new buffer.
+         */
+        void setDataArea(const byte* buf, long len);
+        /*!
+          @brief Set the offset(s) to the data area of an entry. 
+
+          Add @em offset to each data component of the entry. This is used by
+          Ifd::copy to convert the data components of an entry containing
+          offsets relative to the data area to become offsets from the start of
+          the TIFF header.  Usually, entries with a data area have exactly one 
+          unsigned long data component, which is 0.
+
+          @param offset Offset 
+          @param byteOrder Byte order
+
+          @throw Error if the offset is out of range for the data type of the 
+                 tag or the data type is not supported.
+         */
+        void setDataAreaOffsets(uint32_t offset, ByteOrder byteOrder);
+        /*!
+          @brief Update the base pointer of the Entry from \em pOldBase 
+                 to \em pNewBase.
+
+          Allows to re-locate the underlying data buffer to a new location
+          \em pNewBase. This method only has an effect in non-alloc mode.
+
+          @param pOldBase Base pointer of the old data buffer
+          @param pNewBase Base pointer of the new data buffer
+         */
+        void updateBase(byte* pOldBase, byte* pNewBase);
         //@}
 
         //! @name Accessors
         //@{
         //! Return the tag
-        uint16 tag() const { return tag_; }
+        uint16_t tag() const { return tag_; }
         //! Return the type id.
-        uint16 type() const { return type_; }
+        uint16_t type() const { return type_; }
         //! Return the name of the type
         const char* typeName() const 
             { return TypeInfo::typeName(TypeId(type_)); }
@@ -145,10 +193,8 @@ namespace Exiv2 {
         IfdId ifdId() const { return ifdId_; }
         //! Return the index (unique id >0 of an entry within an IFD, 0 if not set)
         int idx() const { return idx_; }
-        //! Return the pointer to the associated MakerNote
-        MakerNote* makerNote() const { return pMakerNote_; }
         //! Return the number of components in the value
-        uint32 count() const { return count_; }
+        uint32_t count() const { return count_; }
         /*!
           @brief Return the size of the data buffer in bytes.
           @note  There is no minimum size for the data buffer, except that it
@@ -156,19 +202,37 @@ namespace Exiv2 {
          */
         long size() const { return size_; }
         //! Return the offset from the start of the IFD to the data of the entry
-        uint32 offset() const { return offset_; }
+        long offset() const { return offset_; }
         /*!
-          @brief Return a pointer to the data area. Do not attempt to write
-          to this pointer.
-        */
+          @brief Return a pointer to the data buffer. Do not attempt to write
+                 to this pointer.
+         */
         const byte* data() const { return pData_; }
         /*!
           @brief Return a pointer to the n-th component, 0 if there is no 
                  n-th component. Do not attempt to write to this pointer.
          */
-        const byte* component(uint32 n) const;
+        const byte* component(uint32_t n) const;
         //! Get the memory allocation mode
         bool alloc() const { return alloc_; }
+        //! Return the size of the data area.
+        long sizeDataArea() const { return sizeDataArea_; }
+        /*!
+          @brief Return a pointer to the data area. Do not attempt to write to
+                 this pointer.
+
+          For certain tags the regular value of an IFD entry is an offset to a
+          data area outside of the IFD. Examples are Exif tag 0x8769 in IFD0
+          (Exif.Image.ExifTag) or tag 0x0201 in IFD1
+          (Exif.Thumbnail.JPEGInterchangeFormat). The offset of ExifTag points
+          to a data area containing the Exif IFD. That of JPEGInterchangeFormat
+          contains the JPEG thumbnail image.
+          Use this method to access (read-only) the data area of a tag. Use 
+          setDataArea() to write to the data area.
+
+          @return Return a pointer to the data area.
+         */
+        const byte* dataArea() const { return pDataArea_; }
         //@}
 
     private:
@@ -176,22 +240,20 @@ namespace Exiv2 {
         /*!
           True:  Requires memory allocation and deallocation,<BR>
           False: No memory management needed.
-        */
+         */
         bool alloc_;
         //! Redundant IFD id (it is also at the IFD)
         IfdId ifdId_;
         //! Unique id of an entry within an IFD (0 if not set)
         int idx_;
-        //! Pointer to the associated MakerNote
-        MakerNote* pMakerNote_;
         //! Tag
-        uint16 tag_;
+        uint16_t tag_;
         //! Type
-        uint16 type_;
+        uint16_t type_;
         //! Number of components
-        uint32 count_;
+        uint32_t count_;
         //! Offset from the start of the IFD to the data
-        uint32 offset_;
+        long offset_;
         /*!
           Size of the data buffer holding the value in bytes, there is 
           no minimum size.
@@ -199,7 +261,11 @@ namespace Exiv2 {
         long size_;
         //! Pointer to the data buffer
         byte* pData_;
-                               
+        //! Size of the data area
+        long sizeDataArea_;
+        //! Pointer to the data area
+        byte* pDataArea_;
+
     }; // class Entry
 
     //! Container type to hold all IFD directory entries
@@ -226,7 +292,7 @@ namespace Exiv2 {
     class FindEntryByTag {
     public:
         //! Constructor, initializes the object with the tag to look for
-        FindEntryByTag(uint16 tag) : tag_(tag) {}
+        FindEntryByTag(uint16_t tag) : tag_(tag) {}
         /*!
           @brief Returns true if the tag of the argument entry is equal
                  to that of the object.
@@ -235,7 +301,7 @@ namespace Exiv2 {
             { return tag_ == entry.tag(); }
 
     private:
-        uint16 tag_;
+        uint16_t tag_;
         
     }; // class FindEntryByTag
 
@@ -264,7 +330,7 @@ namespace Exiv2 {
             assignment behaviours, with the first resulting in entirely separate
             classes and the second mode resulting in multiple classes using one
             and the same data buffer.
-    */
+     */
     class Ifd {
         //! @name Not implemented
         //@{
@@ -273,6 +339,11 @@ namespace Exiv2 {
         //@}
 
     public:
+        //! %Entries const iterator type
+        typedef Entries::const_iterator const_iterator;
+        //! %Entries iterator type
+        typedef Entries::iterator iterator;
+
         //! @name Creators
         //@{
         /*!
@@ -285,24 +356,20 @@ namespace Exiv2 {
                  the IFD from the start of TIFF header. Memory management is
                  enabled.
          */
-        Ifd(IfdId ifdId, uint32 offset);
+        Ifd(IfdId ifdId, long offset);
         /*!
           @brief Constructor. Allows to set the IFD identifier, offset of the
-                 IFD from the start of TIFF header and choose whether or not
-                 memory management is required for the Entries.
+                 IFD from the start of TIFF header, choose whether or not
+                 memory management is required for the Entries, and decide
+                 whether this IFD has a next pointer.
          */
-        Ifd(IfdId ifdId, uint32 offset, bool alloc);
+        Ifd(IfdId ifdId, long offset, bool alloc, bool hasNext =true);
         //! Copy constructor
         Ifd(const Ifd& rhs);
         //! Destructor
         ~Ifd();
         //@}
 
-        //! Entries const iterator type
-        typedef Entries::const_iterator const_iterator;
-        //! Entries iterator type
-        typedef Entries::iterator iterator;
-
         //! @name Manipulators
         //@{
         /*!
@@ -324,28 +391,6 @@ namespace Exiv2 {
                     case.
          */
         int read(const byte* buf, long len, ByteOrder byteOrder, long offset =0);
-        /*!
-          @brief Read a sub-IFD from the location pointed to by the directory entry 
-                 with the given tag.
-
-          @param dest References the destination IFD.
-          @param buf The data buffer to read from. The buffer must contain all Exif 
-                     data starting from the TIFF header (unlike the read() method).
-          @param len Number of bytes in the data buffer 
-          @param byteOrder Applicable byte order (little or big endian).
-          @param tag Tag to look for.
-
-          @return 0 if successful;<BR>
-                  6 if reading the sub-IFD failed (see read() above) or
-                    the location pointed to by the directory entry with the 
-                    given tag is outside of the data buffer.
-
-          @note It is not considered an error if the tag cannot be found in the 
-                IFD. 0 is returned and no action is taken in this case.
-        */
-        int readSubIfd(
-            Ifd& dest, const byte* buf, long len, ByteOrder byteOrder, uint16 tag
-        ) const;
         /*!
           @brief Copy the IFD to a data array, update the offsets of the IFD and
                  all its entries, return the number of bytes written.
@@ -380,9 +425,10 @@ namespace Exiv2 {
         void clear();
         /*!
           @brief Set the offset of the next IFD. Byte order is needed to update
-                 the underlying data buffer in non-alloc mode.
+                 the underlying data buffer in non-alloc mode. This method only
+                 has an effect if the IFD was instantiated with hasNext = true.
          */
-        void setNext(uint32 next, ByteOrder byteOrder);
+        void setNext(uint32_t next, ByteOrder byteOrder);
         /*!
           @brief Add the entry to the IFD. No duplicate-check is performed,
                  i.e., it is possible to add multiple entries with the same tag.
@@ -395,7 +441,7 @@ namespace Exiv2 {
           @brief Delete the directory entry with the given tag. Return the index 
                  of the deleted entry or 0 if no entry with tag was found.
          */
-        int erase(uint16 tag);
+        int erase(uint16_t tag);
         /*!
           @brief Delete the directory entry at iterator position pos, return the
                  position of the next entry. Note that iterators into the
@@ -412,11 +458,44 @@ namespace Exiv2 {
         //! Find an IFD entry by idx, return an iterator into the entries list
         iterator findIdx(int idx);
         //! Find an IFD entry by tag, return an iterator into the entries list
-        iterator findTag(uint16 tag);
+        iterator findTag(uint16_t tag);
+        /*!
+          @brief Update the base pointer of the Ifd and all entries to \em pNewBase.
+
+          Allows to re-locate the underlying data buffer to a new location
+          \em pNewBase. This method only has an effect in non-alloc mode.
+
+          @param pNewBase Pointer to the new data buffer
+
+          @return Old base pointer or 0 if called in alloc mode
+         */
+        byte* updateBase(byte* pNewBase);
         //@}
 
         //! @name Accessors
         //@{
+        /*!
+          @brief Read a sub-IFD from the location pointed to by the directory entry 
+                 with the given tag.
+
+          @param dest References the destination IFD.
+          @param buf The data buffer to read from. The buffer must contain all Exif 
+                     data starting from the TIFF header (unlike the read() method).
+          @param len Number of bytes in the data buffer 
+          @param byteOrder Applicable byte order (little or big endian).
+          @param tag Tag to look for.
+
+          @return 0 if successful;<BR>
+                  6 if reading the sub-IFD failed (see read() above) or
+                    the location pointed to by the directory entry with the 
+                    given tag is outside of the data buffer.
+
+          @note It is not considered an error if the tag cannot be found in the 
+                IFD. 0 is returned and no action is taken in this case.
+        */
+        int readSubIfd(
+            Ifd& dest, const byte* buf, long len, ByteOrder byteOrder, uint16_t tag
+        ) const;
         //! Get the memory allocation mode, see the Ifd class description for details
         bool alloc() const { return alloc_; }
         //! The first entry
@@ -426,28 +505,28 @@ namespace Exiv2 {
         //! Find an IFD entry by idx, return a const iterator into the entries list
         const_iterator findIdx(int idx) const;
         //! Find an IFD entry by tag, return a const iterator into the entries list
-        const_iterator findTag(uint16 tag) const;
+        const_iterator findTag(uint16_t tag) const;
         //! Get the IfdId of the IFD
         IfdId ifdId() const { return ifdId_; }
         //! Get the offset of the IFD from the start of the TIFF header
         long offset() const { return offset_; }
         /*!
-          @brief Get the offset of the first data entry outside of the IFD, 
-                 return 0 if there is none. The data offset is determined when
-                 the IFD is read.
+          @brief Get the offset of the first data entry outside of the IFD from
+                 the start of the TIFF header, return 0 if there is none. The 
+                 data offset is determined when the IFD is read.
          */
         long dataOffset() const { return dataOffset_; }
         //! Get the offset to the next IFD from the start of the TIFF header
-        uint32 next() const { return next_; }
+        uint32_t next() const { return next_; }
         //! Get the number of directory entries in the IFD
         long count() const { return static_cast<long>(entries_.size()); }
         //! Get the size of this IFD in bytes (IFD only, without data)
         long size() const;
         /*!
-          @brief Return the total size of the data of this IFD in bytes,
-                 sums the size of all directory entries where size is greater
-                 than four (i.e., only data that requires memory outside the 
-                 IFD directory entries is counted).
+          @brief Return the total size of the data of this IFD in bytes; sums
+                 the size of all directory entries where size is greater than
+                 four plus the size of all data areas, i.e., all data that
+                 requires memory outside the IFD directory entries is counted.
          */
         long dataSize() const;
         /*!
@@ -460,12 +539,12 @@ namespace Exiv2 {
     private:
         //! Helper structure to build IFD entries
         struct PreEntry {
-            uint16 tag_;
-            uint16 type_; 
-            uint32 count_;
+            uint16_t tag_;
+            uint16_t type_; 
+            uint32_t count_;
             long size_;
             long offsetLoc_;
-            uint32 offset_;
+            long offset_;
         };
 
         //! cmpPreEntriesByOffset needs to know about PreEntry, that's all.
@@ -484,14 +563,18 @@ namespace Exiv2 {
         Entries entries_;
         //! IFD Id
         IfdId ifdId_;
-        //! Offset of the IFD from the start of TIFF header
+        //! Pointer to IFD from the start of the TIFF header
+        byte* pBase_;
+        //! Offset of the IFD from the start of the TIFF header
         long offset_;
         //! Offset of the first data entry outside of the IFD directory
         long dataOffset_;
+        //! Indicates whether the IFD has a next pointer
+        bool hasNext_;
         //! Pointer to the offset of next IFD from the start of the TIFF header
         byte* pNext_;
         //! The offset of the next IFD as data value (always in sync with *pNext_)
-        uint32 next_;
+        uint32_t next_;
 
     }; // class Ifd
 
