@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004, 2005 Andreas Huggel <ahuggel@gmx.net>
  * 
  * This program is part of the Exiv2 distribution.
  *
@@ -21,9 +21,9 @@
 /*!
   @file    iptc.hpp
   @brief   Encoding and decoding of Iptc data
-  @version Exiv2-0_5_040913 1.4
+  @version $Rev: 560 $
   @author  Brad Schick (brad) 
-           <a href="mailto:schick@robotbattle.com">schick@robotbattle.com</a>
+           <a href="mailto:brad@robotbattle.com">brad@robotbattle.com</a>
   @date    31-Jul-04, brad: created
  */
 #ifndef IPTC_HPP_
@@ -48,84 +48,9 @@ namespace Exiv2 {
 // *****************************************************************************
 // class definitions
 
-    //! Concrete keys for Iptc metadata.
-    class IptcKey : public Key {
-    public:
-        //! @name Creators
-        //@{
-        /*!
-          @brief Constructor to create an Iptc key from a key string. 
-
-          @param key The key string.
-          @throw Error ("Invalid key") if the first part of the key is not 
-                 'Iptc' or the remaining parts of the key cannot be parsed and
-                 converted to a record name and a dataset name.
-        */
-        explicit IptcKey(const std::string& key);
-        /*!
-          @brief Constructor to create an Iptc key from dataset and record ids.
-          @param tag Dataset id
-          @param record Record id
-         */
-        IptcKey(uint16 tag, uint16 record);
-        //! Copy constructor
-        IptcKey(const IptcKey& rhs);
-        //@}
-
-        //! @name Manipulators
-        //@{
-        /*!
-          @brief Assignment operator.
-         */
-        IptcKey& operator=(const IptcKey& rhs);
-        //@}
-
-        //! @name Accessors
-        //@{
-        virtual std::string key() const { return key_; }
-        virtual const char* familyName() const 
-            { return IptcDataSets::familyName(); }
-        /*!
-          @brief Return the name of the group (the second part of the key).
-                 For Iptc keys, the group name is the record name.
-        */
-        virtual std::string groupName() const { return recordName(); }
-        virtual std::string tagName() const
-            { return IptcDataSets::dataSetName(tag_, record_); }
-        virtual uint16 tag() const { return tag_; }
-        virtual IptcKey* clone() const;
-
-        //! Return the name of the record
-        const char* recordName() const
-            { return IptcDataSets::recordName(record_); }
-        //! Return the record id
-        uint16 record() const { return record_; }
-        //@}
-
-    protected:
-        //! @name Manipulators
-        //@{
-        /*!
-          @brief Parse and convert the key string into dataset and record id. 
-                 Updates tag_ and record_ if the string can be decomposed,
-                 or throws Error ("Invalid key").
-
-          @throw Error ("Invalid key") if the key cannot be parsed into
-                 family name, group name and tag name parts.
-         */
-        void decomposeKey();
-        //@}
-
-    private:
-        // DATA
-        uint16 tag_;                   //!< Tag value
-        uint16 record_;                //!< Record value 
-        std::string key_;              //!< Key
-
-    }; // class IptcKey
-
     /*!
-      @brief Information related to one Iptc dataset.
+      @brief Information related to one Iptc dataset. An Iptc metadatum consists 
+             of an IptcKey and a Value and provides methods to manipulate these.
      */
     class Iptcdatum : public Metadatum {
     public:
@@ -139,12 +64,12 @@ namespace Exiv2 {
                  set the value using setValue().
 
           @param key The key of the %Iptcdatum.
-          @param value Pointer to a %Iptcdatum value.
-          @throw Error ("Invalid key") if the key cannot be parsed and converted
+          @param pValue Pointer to a %Iptcdatum value.
+          @throw Error if the key cannot be parsed and converted
                  to a tag number and record id.
          */
         explicit Iptcdatum(const IptcKey& key, 
-                           const Value* value =0);
+                           const Value* pValue =0);
         //! Copy constructor
         Iptcdatum(const Iptcdatum& rhs);
         //! Destructor
@@ -156,16 +81,34 @@ namespace Exiv2 {
         //! Assignment operator
         Iptcdatum& operator=(const Iptcdatum& rhs);
         /*!
-          @brief Set the value. This method copies (clones) the value pointed
-                 to by pValue.
+          @brief Assign \em value to the %Iptcdatum. The type of the new Value
+                 is set to UShortValue.
+         */
+        Iptcdatum& operator=(const uint16_t& value);
+        /*!
+          @brief Assign \em value to the %Iptcdatum. 
+                 Calls setValue(const std::string&).
+         */
+        Iptcdatum& operator=(const std::string& value);
+        /*!
+          @brief Assign \em value to the %Iptcdatum.
+                 Calls setValue(const Value*).
+         */
+        Iptcdatum& operator=(const Value& value);
+        /*!
+          @brief Set the Value. This method copies (clones) the %Value pointed
+                 to by \em pValue.
          */
         void setValue(const Value* pValue);
         /*!
-          @brief Set the value to the string buf. 
-                 Uses Value::read(const std::string& buf). If the Iptcdatum does
-                 not have a value yet, then an StringValue is created.
+          @brief Set the value to the string \em value, using 
+                 Value::read(const std::string&). 
+                 If the %Iptcdatum does not have a Value yet, then a %Value of
+                 the correct type for this %Iptcdatum is created. If that
+                 fails (because of an unknown dataset), a StringValue is 
+                 created.
          */
-        void setValue(const std::string& buf);
+        void setValue(const std::string& value);
         //@}
 
         //! @name Accessors
@@ -182,53 +125,49 @@ namespace Exiv2 {
           @return Number of characters written.
         */
         long copy(byte* buf, ByteOrder byteOrder) const 
-            { return pValue_ == 0 ? 0 : pValue_->copy(buf, byteOrder); }
+            { return value_.get() == 0 ? 0 : value_->copy(buf, byteOrder); }
         /*!
           @brief Return the key of the Iptcdatum. The key is of the form
                  '<b>Iptc</b>.recordName.datasetName'. Note however that the key
                  is not necessarily unique, i.e., an IptcData may contain
                  multiple metadata with the same key.
          */
-        std::string key() const { return pKey_ == 0 ? "" : pKey_->key(); }
+        std::string key() const { return key_.get() == 0 ? "" : key_->key(); }
         /*!
            @brief Return the name of the record
            @return record name
-           @throw Error("Unknown record");
          */
-        const char* recordName() const
-            { return pKey_ == 0 ? "" : pKey_->recordName(); }
+        std::string recordName() const
+            { return key_.get() == 0 ? "" : key_->recordName(); }
         /*!
            @brief Return the record id 
            @return record id
          */
-        uint16 record() const 
-            { return pKey_ == 0 ? 0 : pKey_->record(); }
+        uint16_t record() const 
+            { return key_.get() == 0 ? 0 : key_->record(); }
         /*!
            @brief Return the name of the tag (aka dataset)
            @return tag name
-           @throw Error("No dataSet for record Id") if tag is unknown
          */
         std::string tagName() const
-            { return pKey_ == 0 ? "" : pKey_->tagName(); }
+            { return key_.get() == 0 ? "" : key_->tagName(); }
         //! Return the tag (aka dataset) number
-        uint16 tag() const
-            { return pKey_ == 0 ? 0 : pKey_->tag(); }
+        uint16_t tag() const
+            { return key_.get() == 0 ? 0 : key_->tag(); }
         //! Return the type id of the value
         TypeId typeId() const 
-            { return pValue_ == 0 ? invalidTypeId : pValue_->typeId(); }
+            { return value_.get() == 0 ? invalidTypeId : value_->typeId(); }
         //! Return the name of the type
         const char* typeName() const { return TypeInfo::typeName(typeId()); }
         //! Return the size in bytes of one component of this type
         long typeSize() const { return TypeInfo::typeSize(typeId()); }
         //! Return the number of components in the value
-        long count() const { return pValue_ == 0 ? 0 : pValue_->count(); }
+        long count() const { return value_.get() == 0 ? 0 : value_->count(); }
         //! Return the size of the value in bytes
-        long size() const { return pValue_ == 0 ? 0 : pValue_->size(); }
-        //! Return true if value was modified, otherwise false
-        bool modified() const { return modified_; }
+        long size() const { return value_.get() == 0 ? 0 : value_->size(); }
         //! Return the value as a string.
         std::string toString() const 
-            { return pValue_ == 0 ? "" : pValue_->toString(); }
+            { return value_.get() == 0 ? "" : value_->toString(); }
         /*!
           @brief Return the n-th component of the value converted to long. The
                  return value is -1 if the value of the Iptcdatum is not set and
@@ -236,7 +175,7 @@ namespace Exiv2 {
                  component.
          */
         long toLong(long n =0) const 
-            { return pValue_ == 0 ? -1 : pValue_->toLong(n); }
+            { return value_.get() == 0 ? -1 : value_->toLong(n); }
         /*!
           @brief Return the n-th component of the value converted to float.  The
                  return value is -1 if the value of the Iptcdatum is not set and
@@ -244,7 +183,7 @@ namespace Exiv2 {
                  component.
          */
         float toFloat(long n =0) const 
-            { return pValue_ == 0 ? -1 : pValue_->toFloat(n); }
+            { return value_.get() == 0 ? -1 : value_->toFloat(n); }
         /*!
           @brief Return the n-th component of the value converted to
                  Rational. The return value is -1/1 if the value of the
@@ -252,20 +191,22 @@ namespace Exiv2 {
                  undefined if there is no n-th component.
          */
         Rational toRational(long n =0) const 
-            { return pValue_ == 0 ? Rational(-1, 1) : pValue_->toRational(n); }
+            { return value_.get() == 0 ? Rational(-1, 1) : value_->toRational(n); }
         /*!
-          @brief Return a pointer to a copy (clone) of the value. The caller
-                 is responsible to delete this copy when it's no longer needed.
+          @brief Return an auto-pointer to a copy (clone) of the value. The
+                 caller owns this copy and the auto-pointer ensures that it will
+                 be deleted.
 
           This method is provided for users who need full control over the 
           value. A caller may, e.g., downcast the pointer to the appropriate
           subclass of Value to make use of the interface of the subclass to set
           or modify its contents.
           
-          @return A pointer to a copy (clone) of the value, 0 if the value is 
-                  not set.
+          @return An auto-pointer to a copy (clone) of the value, 0 if the value
+                  is not set.
          */
-        Value* getValue() const { return pValue_ == 0 ? 0 : pValue_->clone(); }
+        Value::AutoPtr getValue() const 
+            { return value_.get() == 0 ? Value::AutoPtr(0) : value_->clone(); }
         /*!
           @brief Return a constant reference to the value. 
 
@@ -283,23 +224,16 @@ namespace Exiv2 {
           @endcode
 
           @return A constant reference to the value.
-          @throw Error ("Value not set") if the value is not set.
+          @throw Error If the value is not set.
          */
-        const Value& value() const 
-            { if (pValue_) return *pValue_; throw Error("Value not set"); }
+        const Value& value() const
+            { if (value_.get() != 0) return *value_; throw Error(8); }
         //@}
 
-        /*! 
-          @brief Not really meant for public use, but public so IptcData doesn't
-                 have to be made a friend
-         */
-        void clearModified() { modified_ = false; }
-
     private:
         // DATA
-        IptcKey* pKey_;                //!< Key
-        Value* pValue_;                //!< Pointer to the value
-        bool modified_;                //!< Change indicator
+        IptcKey::AutoPtr key_;                  //!< Key
+        Value::AutoPtr   value_;                //!< Value
 
     }; // class Iptcdatum
 
@@ -316,7 +250,7 @@ namespace Exiv2 {
     class FindMetadatumById {
     public:
         //! Constructor, initializes the object with the record and dataset id
-        FindMetadatumById(uint16 dataset, uint16 record)
+        FindMetadatumById(uint16_t dataset, uint16_t record)
             : dataset_(dataset), record_(record) {}
         /*!
           @brief Returns true if the record and dataset id of the argument
@@ -326,8 +260,8 @@ namespace Exiv2 {
             { return dataset_ == iptcdatum.tag() && record_ == iptcdatum.record(); }
 
     private:
-        uint16 dataset_;
-        uint16 record_;
+        uint16_t dataset_;
+        uint16_t record_;
     
     }; // class FindMetadatumById
 
@@ -343,81 +277,41 @@ namespace Exiv2 {
       - extract Iptc metadata to files, insert from these files
     */
     class IptcData {
-        //! @name Not implemented
-        //@{
-        //! Copying not allowed
-        IptcData(const IptcData& rhs);
-        //! Assignment not allowed
-        IptcData& operator=(const IptcData& rhs);
-        //@}
     public:
         //! IptcMetadata iterator type
         typedef IptcMetadata::iterator iterator;
         //! IptcMetadata const iterator type
         typedef IptcMetadata::const_iterator const_iterator;
 
-        //! @name Creators
-        //@{
-        //! Default constructor
-        IptcData();
-        //! Destructor
-        ~IptcData();
-        //@}
+        // Use the compiler generated constructors and assignment operator
 
         //! @name Manipulators
         //@{
         /*!
-          @brief Read the Iptc data from file path.
-          @param path Path to the file
-          @return  0 if successful;<BR>
-                   3 if the file contains no Iptc data;<BR>
-                  the return code of Image::readMetadata()
-                    if the call to this function fails;<BR>
-                  the return code of read(const char* buf, long len)
-                    if the call to this function fails;<BR>
-         */
-        int read(const std::string& path);
-        /*!
-          @brief Read the Iptc data from a byte buffer. The format must follow
+          @brief Load the Iptc data from a byte buffer. The format must follow
                  the IPTC IIM4 standard.
           @param buf Pointer to the data buffer to read from
           @param len Number of bytes in the data buffer 
           @return 0 if successful;<BR>
                  5 if Iptc data is invalid or corrupt;<BR>
          */
-        int read(const byte* buf, long len);
+        int load(const byte* buf, long len);
         /*!
-          @brief Write the Iptc data to file path. If an Iptc data section
-                 already exists in the file, it is replaced.  If there is no
-                 metadata to write, the Iptc data section is
-                 deleted from the file.  Otherwise, an Iptc data section is
-                 created.
-          @return 0 if successful;<BR>
-                -2 if the file contains an unknown image type;<BR>
-                the return code of Image::writeMetadata()
-                    if the call to this function fails;<BR>
+          @brief Write the Iptc data to a data buffer and return the data buffer.
+                 Caller owns this buffer. The copied data follows the IPTC IIM4
+                 standard.
+          @return Data buffer containing the Iptc data.
          */
-        int write(const std::string& path);
+        DataBuf copy();
         /*!
-          @brief Write the Iptc data to a binary file. By convention, the
-                 filename extension should be ".exv". This file format contains
-                 the Iptc data as it is found in a JPEG file. Exv files can
-                 be read with int read(const std::string& path) just like
-                 normal image files.
-          @return 0 if successful;<BR>
-                 the return code of Image::writeMetadata()
-                    if the call to this function fails;<BR>
+          @brief Returns a reference to the %Iptcdatum that is associated with a
+                 particular \em key. If %IptcData does not already contain such
+                 an %Iptcdatum, operator[] adds object \em Iptcdatum(key).
+
+          @note  Since operator[] might insert a new element, it can't be a const
+                 member function.
          */
-        int writeIptcData(const std::string& path);
-        /*!
-          @brief Write the Iptc data to a data buffer, return number of bytes 
-                 written. The copied data follows the IPTC IIM4 standard.
-          @param buf The data buffer to write to.  The user must ensure that the
-                 buffer has enough memory using the %size method. Otherwise
-                 the call results in undefined behaviour.
-          @return Number of characters written to the buffer.
-         */
-        long copy(byte* buf);
+        Iptcdatum& operator[](const std::string& key);
         /*!
           @brief Add an %Iptcdatum from the supplied key and value pair. This
                  method copies (clones) the value. A check for non-repeatable
@@ -440,6 +334,10 @@ namespace Exiv2 {
                  by this call.
          */
         iterator erase(iterator pos);
+        /*!
+          @brief Delete all Iptcdatum instances resulting in an empty container.
+         */
+        void clear() { iptcMetadata_.clear(); }
         //! Sort metadata by key
         void sortByKey();
         //! Sort metadata by tag (aka dataset)
@@ -460,23 +358,12 @@ namespace Exiv2 {
                 same Ids exists, it is undefined which of the matching
                 metadata is found.
          */
-        iterator findId(uint16 dataset, 
-                        uint16 record = IptcDataSets::application2);
+        iterator findId(uint16_t dataset, 
+                        uint16_t record = IptcDataSets::application2);
         //@}
 
         //! @name Accessors
         //@{
-        /*!
-          @brief Erase the Iptc data from file path. 
-          @param path Path to the file.
-          @return 0 if successful;<BR>
-                -2 if the file contains an unknown image type;<BR>
-                the return code of Image::writeMetadata()
-                    if the call to this function fails;<BR>
-         */
-        int erase(const std::string& path) const;
-        //! Return true if any metadata was modified, added, or erased.
-        bool modified() const;
         //! Begin of the metadata
         const_iterator begin() const { return iptcMetadata_.begin(); }
         //! End of the metadata
@@ -493,8 +380,10 @@ namespace Exiv2 {
                 same Ids exist it is undefined which of the matching
                 metadata is found.
          */
-        const_iterator findId(uint16 dataset, 
-                              uint16 record = IptcDataSets::application2) const;
+        const_iterator findId(uint16_t dataset, 
+                              uint16_t record = IptcDataSets::application2) const;
+        //! Return true if there is no Iptc metadata
+        bool empty() const { return count() == 0; }
         //! Get the number of metadata entries
         long count() const { return static_cast<long>(iptcMetadata_.size()); }
         /*!
@@ -503,19 +392,6 @@ namespace Exiv2 {
         long size() const;
         //@}
 
-        /*!
-          @brief Convert the return code from 
-                 int read(const std::string& path),
-                 int read(const byte* buf, long len),
-                 int write(const std::string& path),
-                 int writeIptcData(const std::string& path), 
-                 int erase(const std::string& path) const 
-                 into an error message.
-
-                 Todo: Implement global handling of error messages
-         */
-        static std::string strError(int rc, const std::string& path);
-
     private:
         /*!
           @brief Read a single dataset payload and create a new metadata entry
@@ -525,30 +401,14 @@ namespace Exiv2 {
           @param sizeData Length in bytes of dataset payload
           @return 0 if successful.
          */
-        int readData(uint16 dataSet, uint16 record, 
-                     const byte* data, uint32 sizeData);
-
-        //! Resets modified flag
-        void clearModified();
-        
-        //! @name Manipulators
-        //@{
-        /*!
-          @brief Rebuilds the Iptc data buffer from scratch using the current
-                metadata. After this method is called, pData_ points to the
-                updated data.
-         */
-        void updateBuffer();
-        //@}
+        int readData(uint16_t dataSet, uint16_t record, 
+                     const byte* data, uint32_t sizeData);
 
         // Constant data
         static const byte marker_;          // Dataset marker
         
         // DATA
         IptcMetadata iptcMetadata_;
-        long size_;              //!< Size of the Iptc raw data in bytes
-        byte* pData_;            //!< Iptc raw data buffer
-        mutable bool modified_;
     }; // class IptcData
 
 }                                       // namespace Exiv2
