@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004, 2005 Andreas Huggel <ahuggel@gmx.net>
  * 
  * This program is part of the Exiv2 distribution.
  *
@@ -20,150 +20,61 @@
  */
 /*
   File:      makernote.cpp
-  Version:   Exiv2-0_5_040913 1.26
+  Version:   $Rev: 579 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
   History:   18-Feb-04, ahu: created
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) Exiv2-0_5_040913 1.26 makernote.cpp,v");
+EXIV2_RCSID("@(#) $Id: makernote.cpp 579 2005-06-11 04:11:23Z ahuggel $");
 
-// Define DEBUG_* to output debug information to std::cerr
-#undef DEBUG_MAKERNOTE
-#undef DEBUG_REGISTRY
+// Define DEBUG_* to output debug information to std::cerr, e.g, by calling
+// make like this: make DEFS=-DDEBUG_MAKERNOTE makernote.o 
+//#define DEBUG_MAKERNOTE
+//#define DEBUG_REGISTRY
 
 // *****************************************************************************
 // included header files
 #include "makernote.hpp"
-#include "tags.hpp"                         // for ExifTags::ifdItem
 #include "error.hpp"
 
 // + standard includes
 #include <string>
 #include <sstream>
 #include <iomanip>
-
-#if defined DEBUG_MAKERNOTE || defined DEBUG_REGISTRY
-#   include <iostream>
-#endif
+#include <iostream>
+#include <cassert>
 
 // *****************************************************************************
 // class member definitions
 namespace Exiv2 {
 
-    MakerNote::MakerNote(const MnTagInfo* pMnTagInfo, bool alloc) 
-        : pMnTagInfo_(pMnTagInfo), alloc_(alloc),
-          offset_(0), byteOrder_(invalidByteOrder)
+    MakerNote::MakerNote(bool alloc) 
+        : alloc_(alloc), offset_(0), byteOrder_(invalidByteOrder)
     {
     }
 
-    std::string MakerNote::makeKey(uint16 tag) const
+    MakerNote::AutoPtr MakerNote::create(bool alloc) const
     {
-        return std::string(ExifTags::familyName())
-            + "." + std::string(ifdItem())
-            + "." + tagName(tag);
-    } // MakerNote::makeKey
+        return AutoPtr(create_(alloc));
+    }
 
-    uint16 MakerNote::decomposeKey(const std::string& key) const
+    MakerNote::AutoPtr MakerNote::clone() const
     {
-        // Get the family, item and tag name parts of the key
-        std::string::size_type pos1 = key.find('.');
-        if (pos1 == std::string::npos) throw Error("Invalid key");
-        std::string familyName = key.substr(0, pos1);
-        std::string::size_type pos0 = pos1 + 1;
-        pos1 = key.find('.', pos0);
-        if (pos1 == std::string::npos) throw Error("Invalid key");
-        std::string ifdItem = key.substr(pos0, pos1 - pos0);
-        pos0 = pos1 + 1;
-        std::string tagName = key.substr(pos0);
-        if (tagName == "") throw Error("Invalid key");
+        return AutoPtr(clone_());
+    }
 
-        if (familyName != ExifTags::familyName()) return 0xffff;
-        uint16 tag = this->tag(tagName);
-        if (tag == 0xffff) return tag;
-        if (ifdItem != this->ifdItem()) return 0xffff;
-
-        return tag;
-    } // MakerNote::decomposeKey
-
-    std::string MakerNote::tagName(uint16 tag) const
+    IfdMakerNote::IfdMakerNote(IfdId ifdId, bool alloc, bool hasNext)
+        : MakerNote(alloc), 
+          absOffset_(true), adjOffset_(0), ifd_(ifdId, 0, alloc, hasNext)
     {
-        std::string tagName;
-        if (pMnTagInfo_) {
-            for (int i = 0; pMnTagInfo_[i].tag_ != 0xffff; ++i) {
-                if (pMnTagInfo_[i].tag_ == tag) {
-                    tagName = pMnTagInfo_[i].name_;
-                    break;
-                }
-            }
-        }
-        if (tagName.empty()) {
-            std::ostringstream os;
-            os << "0x" << std::setw(4) << std::setfill('0') << std::right
-               << std::hex << tag;
-            tagName = os.str();
-        }
-        return tagName;
-    } // MakerNote::tagName
+    }
 
-    uint16 MakerNote::tag(const std::string& tagName) const
-    {
-        uint16 tag = 0xffff;
-        if (pMnTagInfo_) {
-            for (int i = 0; pMnTagInfo_[i].tag_ != 0xffff; ++i) {
-                if (pMnTagInfo_[i].name_ == tagName) {
-                    tag = pMnTagInfo_[i].tag_;
-                    break;
-                }
-            }
-        }
-        if (tag == 0xffff) {
-            std::istringstream is(tagName);
-            is >> std::hex >> tag;
-        }
-        return tag;
-    } // MakerNote::tag
-
-    std::string MakerNote::tagDesc(uint16 tag) const
-    {
-        std::string tagDesc;
-        if (pMnTagInfo_) {
-            for (int i = 0; pMnTagInfo_[i].tag_ != 0xffff; ++i) {
-                if (pMnTagInfo_[i].tag_ == tag) {
-                    tagDesc = pMnTagInfo_[i].desc_;
-                    break;
-                }
-            }
-        }
-        return tagDesc;
-    } // MakerNote::tagDesc
-
-    void MakerNote::taglist(std::ostream& os) const
-    {
-        if (pMnTagInfo_) {
-            for (int i = 0; pMnTagInfo_[i].tag_ != 0xffff; ++i) {
-                writeMnTagInfo(os, pMnTagInfo_[i].tag_) << std::endl;
-            }
-        }
-    } // MakerNote::taglist
-
-    std::ostream& MakerNote::writeMnTagInfo(std::ostream& os, uint16 tag) const
-    {
-        return os << tagName(tag) << ", "
-                  << std::dec << tag << ", "
-                  << "0x" << std::setw(4) << std::setfill('0') 
-                  << std::right << std::hex << tag << ", "
-                  << ExifTags::ifdItem(makerIfdId) << ", "
-                  << makeKey(tag) << ", " 
-                  << tagDesc(tag);
-    } // MakerNote::writeMnTagInfo
-
-
-    IfdMakerNote::IfdMakerNote(const MakerNote::MnTagInfo* pMnTagInfo,
-                               bool alloc)
-        : MakerNote(pMnTagInfo, alloc),
-          absOffset_(true), adjOffset_(0), ifd_(makerIfdId, 0, alloc)
+    IfdMakerNote::IfdMakerNote(const IfdMakerNote& rhs)
+        : MakerNote(rhs), absOffset_(rhs.absOffset_), adjOffset_(rhs.adjOffset_),
+          header_(rhs.header_.size_), ifd_(rhs.ifd_)
     {
+        memcpy(header_.pData_, rhs.header_.pData_, header_.size_);
     }
 
     int IfdMakerNote::read(const byte* buf,
@@ -191,12 +102,10 @@ namespace Exiv2 {
         }
         if (rc == 0) {
             // IfdMakerNote currently does not support multiple IFDs
-            if (ifd_.next() != 0) rc = 3;
-        }
-        if (rc == 0) {
-            Entries::iterator end = ifd_.end();
-            for (Entries::iterator i = ifd_.begin(); i != end; ++i) {
-                i->setMakerNote(this);
+            if (ifd_.next() != 0) {
+                std::cerr << "Warning: Makernote IFD has a next pointer != 0 ("
+                          << ifd_.next()
+                          << "). Ignored.\n";
             }
         }
 #ifdef DEBUG_MAKERNOTE
@@ -231,6 +140,13 @@ namespace Exiv2 {
         return 0;
     }
 
+    void IfdMakerNote::updateBase(byte* pNewBase)
+    { 
+        if (absOffset_) {
+            ifd_.updateBase(pNewBase);
+        }
+    }
+
     int IfdMakerNote::checkHeader() const
     {
         // Default implementation does nothing, assuming there is no header
@@ -258,15 +174,46 @@ namespace Exiv2 {
         return headerSize() + ifd_.size() + ifd_.dataSize();
     }
 
-    MakerNoteFactory* MakerNoteFactory::pInstance_ = 0;
-
-    MakerNoteFactory& MakerNoteFactory::instance()
+    IfdMakerNote::AutoPtr IfdMakerNote::create(bool alloc) const
     {
-        if (0 == pInstance_) {
-            pInstance_ = new MakerNoteFactory;
+        return AutoPtr(create_(alloc));
+    }
+
+    IfdMakerNote::AutoPtr IfdMakerNote::clone() const
+    {
+        return AutoPtr(clone_());
+    }
+
+    MakerNoteFactory::Registry* MakerNoteFactory::pRegistry_ = 0;
+    MakerNoteFactory::IfdIdRegistry* MakerNoteFactory::pIfdIdRegistry_ = 0;
+
+    void MakerNoteFactory::init()
+    {
+        if (0 == pRegistry_) {
+            pRegistry_ = new Registry;
         }
-        return *pInstance_;
-    } // MakerNoteFactory::instance
+        if (0 == pIfdIdRegistry_) {
+            pIfdIdRegistry_ = new IfdIdRegistry;
+        }
+    } // MakerNoteFactory::init
+
+    void MakerNoteFactory::registerMakerNote(IfdId ifdId,
+                                             MakerNote::AutoPtr makerNote)
+    {
+        init();
+        MakerNote* pMakerNote = makerNote.release();
+        assert(pMakerNote);
+        (*pIfdIdRegistry_)[ifdId] = pMakerNote;
+    } // MakerNoteFactory::registerMakerNote
+
+    MakerNote::AutoPtr MakerNoteFactory::create(IfdId ifdId, bool alloc)
+    {
+        assert(pIfdIdRegistry_ != 0);
+        IfdIdRegistry::const_iterator i = pIfdIdRegistry_->find(ifdId);
+        if (i == pIfdIdRegistry_->end()) return MakerNote::AutoPtr(0);
+        assert(i->second);
+        return i->second->create(alloc);
+    } // MakerNoteFactory::create
 
     void MakerNoteFactory::registerMakerNote(const std::string& make, 
                                              const std::string& model, 
@@ -276,44 +223,45 @@ namespace Exiv2 {
         std::cerr << "Registering MakerNote create function for \"" 
                   << make << "\" and \"" << model << "\".\n";
 #endif
-
+        init();
         // Todo: use case insensitive make and model comparisons
 
         // Find or create a registry entry for make
-        ModelRegistry* modelRegistry = 0;
-        Registry::const_iterator end1 = registry_.end();
+        ModelRegistry* pModelRegistry = 0;
+        assert(pRegistry_ != 0);
+        Registry::const_iterator end1 = pRegistry_->end();
         Registry::const_iterator pos1;
-        for (pos1 = registry_.begin(); pos1 != end1; ++pos1) {
+        for (pos1 = pRegistry_->begin(); pos1 != end1; ++pos1) {
             if (pos1->first == make) break;
         }
         if (pos1 != end1) {
-            modelRegistry = pos1->second;
+            pModelRegistry = pos1->second;
         }
         else {
-            modelRegistry = new ModelRegistry;
-            registry_.push_back(std::make_pair(make, modelRegistry));
+            pModelRegistry = new ModelRegistry;
+            pRegistry_->push_back(std::make_pair(make, pModelRegistry));
         }
         // Find or create a registry entry for model
-        ModelRegistry::iterator end2 = modelRegistry->end();
+        ModelRegistry::iterator end2 = pModelRegistry->end();
         ModelRegistry::iterator pos2;
-        for (pos2 = modelRegistry->begin(); pos2 != end2; ++pos2) {
+        for (pos2 = pModelRegistry->begin(); pos2 != end2; ++pos2) {
             if (pos2->first == model) break;
         }
         if (pos2 != end2) {
             pos2->second = createMakerNote;
         }
         else {
-            modelRegistry->push_back(std::make_pair(model, createMakerNote));
+            pModelRegistry->push_back(std::make_pair(model, createMakerNote));
         }
     } // MakerNoteFactory::registerMakerNote
 
-    MakerNote* MakerNoteFactory::create(const std::string& make, 
-                                        const std::string& model,
-                                        bool alloc,    
-                                        const byte* buf, 
-                                        long len, 
-                                        ByteOrder byteOrder, 
-                                        long offset) const
+    MakerNote::AutoPtr MakerNoteFactory::create(const std::string& make, 
+                                                const std::string& model,
+                                                bool alloc,
+                                                const byte* buf, 
+                                                long len, 
+                                                ByteOrder byteOrder, 
+                                                long offset)
     {
 #ifdef DEBUG_REGISTRY
         std::cerr << "Entering MakerNoteFactory::create(\"" 
@@ -322,24 +270,25 @@ namespace Exiv2 {
 #endif
         // loop through each make of the registry to find the best matching make
         int score = 0;
-        ModelRegistry* modelRegistry = 0;
+        ModelRegistry* pModelRegistry = 0;
 #ifdef DEBUG_REGISTRY
         std::string makeMatch;
         std::cerr << "Searching make registry...\n"; 
 #endif
-        Registry::const_iterator end1 = registry_.end();
+        assert(pRegistry_ != 0);
+        Registry::const_iterator end1 = pRegistry_->end();
         Registry::const_iterator pos1;
-        for (pos1 = registry_.begin(); pos1 != end1; ++pos1) {
+        for (pos1 = pRegistry_->begin(); pos1 != end1; ++pos1) {
             int rc = match(pos1->first, make);
             if (rc > score) {
                 score = rc;
 #ifdef DEBUG_REGISTRY
                 makeMatch = pos1->first;
 #endif
-                modelRegistry = pos1->second;
+                pModelRegistry = pos1->second;
             }
         }
-        if (modelRegistry == 0) return 0;
+        if (pModelRegistry == 0) return MakerNote::AutoPtr(0);
 #ifdef DEBUG_REGISTRY
         std::cerr << "Best match is \"" << makeMatch << "\".\n";
 #endif
@@ -351,9 +300,9 @@ namespace Exiv2 {
         std::string modelMatch;
         std::cerr << "Searching model registry...\n";
 #endif
-        ModelRegistry::const_iterator end2 = modelRegistry->end();
+        ModelRegistry::const_iterator end2 = pModelRegistry->end();
         ModelRegistry::const_iterator pos2;
-        for (pos2 = modelRegistry->begin(); pos2 != end2; ++pos2) {
+        for (pos2 = pModelRegistry->begin(); pos2 != end2; ++pos2) {
             int rc = match(pos2->first, model);
             if (rc > score) {
                 score = rc;
@@ -363,7 +312,7 @@ namespace Exiv2 {
                 createMakerNote = pos2->second;
             }
         }
-        if (createMakerNote == 0) return 0;
+        if (createMakerNote == 0) return MakerNote::AutoPtr(0);
 #ifdef DEBUG_REGISTRY
         std::cerr << "Best match is \"" << modelMatch << "\".\n";
 #endif
@@ -376,8 +325,8 @@ namespace Exiv2 {
     {
 #ifdef DEBUG_REGISTRY
         std::cerr << "   Matching registry entry \"" << regEntry << "\" (" 
-                  << regEntry.size() << ") with key \"" << key << "\" ("
-                  << key.size() << "): ";
+                  << (int)regEntry.size() << ") with key \"" << key << "\" ("
+                  << (int)key.size() << "): ";
 #endif
         // Todo: make the comparisons case insensitive
 
@@ -385,7 +334,7 @@ namespace Exiv2 {
         // return value - the following algorithm also finds exact matches)
         if (regEntry == key) {
 #ifdef DEBUG_REGISTRY
-            std::cerr << "Exact match (score: " << key.size() + 2 << ")\n";
+            std::cerr << "Exact match (score: " << (int)key.size() + 2 << ")\n";
 #endif
             return static_cast<int>(key.size()) + 2;
         }
