@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004, 2005 Andreas Huggel <ahuggel@gmx.net>
  * 
  * This program is part of the Exiv2 distribution.
  *
@@ -22,7 +22,7 @@
   @file    makernote.hpp
   @brief   Contains the Exif %MakerNote interface, IFD %MakerNote and a 
            MakerNote factory
-  @version Exiv2-0_5_040913 1.22
+  @version $Rev: 569 $
   @author  Andreas Huggel (ahu)
            <a href="mailto:ahuggel@gmx.net">ahuggel@gmx.net</a>
   @date    18-Feb-04, ahu: created
@@ -40,6 +40,8 @@
 #include <iosfwd>
 #include <utility>
 #include <vector>
+#include <map>
+#include <memory>
 
 // *****************************************************************************
 // namespace extensions
@@ -49,12 +51,6 @@ namespace Exiv2 {
 // class declarations
     class Value;
 
-// *****************************************************************************
-// type definitions
-
-    //! Type for a pointer to a function creating a makernote
-    typedef MakerNote* (*CreateFct)(bool, const byte*, long, ByteOrder, long);
-
 // *****************************************************************************
 // class definitions
 
@@ -73,7 +69,6 @@ namespace Exiv2 {
       - read the makernote from a character buffer
       - copy the makernote to a character buffer
       - maintain a list of makernote entries (similar to IFD entries) 
-      - provide makernote specific tag names and keys
       - interpret (print) the values of makernote tags
 
       Makernotes can be added to the system by subclassing %MakerNote and
@@ -93,9 +88,9 @@ namespace Exiv2 {
       See existing makernote implementations for examples, e.g., CanonMakerNote
       or FujiMakerNote.
 
-      Finally, the implementation files should be named *mn.[ch]pp, so that the
-      magic, which ensures that the makernote is automatically registered 
-      in the factory, will pick it up (see mn.sh for details).
+      Finally, the header file which defines the static variable 
+      \em register*MakerNote needs to be included from mn.hpp, to ensure that 
+      the makernote is automatically registered in the factory.
      */
     class MakerNote {
         //! @name Not implemented
@@ -105,26 +100,16 @@ namespace Exiv2 {
         //@}
 
     public:
-
-        //! MakerNote Tag information
-        struct MnTagInfo {
-            //! Constructor
-            MnTagInfo(uint16 tag, const char* name, const char* desc) 
-                : tag_(tag), name_(name), desc_(desc) {}
-
-            uint16 tag_;                //!< Tag
-            const char* name_;          //!< One word tag label
-            const char* desc_;          //!< Short tag description
-        }; // struct MnTagInfo
+        //! Shortcut for a %MakerNote auto pointer.
+        typedef std::auto_ptr<MakerNote> AutoPtr;
 
         //! @name Creators
         //@{
         /*!
-          @brief Constructor. Takes an optional makernote info tag array and
-                 allows to choose whether or not memory management is required
-                 for the Entries.
+          @brief Constructor. Allows to choose whether or not memory management 
+                 is required for the Entries.
          */
-        explicit MakerNote(const MnTagInfo* pMnTagInfo =0, bool alloc =true);
+        explicit MakerNote(bool alloc =true);
         //! Virtual destructor.
         virtual ~MakerNote() {}
         //@}
@@ -160,60 +145,45 @@ namespace Exiv2 {
         virtual Entries::iterator begin() =0;
         //! End of the makernote entries
         virtual Entries::iterator end() =0;
+        /*!
+          @brief Update the base pointer of the %MakerNote and all its entries 
+                 to \em pNewBase.
+
+          Allows to re-locate the underlying data buffer to a new location
+          \em pNewBase. This method only has an effect in non-alloc mode.
+         */
+        virtual void updateBase(byte* pNewBase) =0;
         //@}
 
         //! @name Accessors
         //@{
-        //! Return the key for the tag.
-        std::string makeKey(uint16 tag) const;
-        //! Return the associated tag for a makernote key.
-        uint16 decomposeKey(const std::string& key) const;
         //! Return the byte order (little or big endian).
         ByteOrder byteOrder() const { return byteOrder_; }
         //! Return the offset of the makernote from the start of the TIFF header
         long offset() const  { return offset_; }
         /*!
-          @brief Return the name of a makernote tag. The default implementation
-                 looks up the makernote info tag array if one is set, else
-                 it translates the tag to a string with the hexadecimal value of
-                 the tag.
-         */
-        virtual std::string tagName(uint16 tag) const;
-        /*!
-          @brief Return the description of a makernote tag. The default
-                 implementation looks up the makernote info tag array if one is
-                 set, else it returns an empty string.
-         */
-        virtual uint16 tag(const std::string& tagName) const;
-        /*!
-          @brief Print a list of all tags known by this MakerNote to the output 
-                 stream os. The default implementation prints all tags in the 
-                 makernote info tag array if one is set.
-         */
-        virtual std::string tagDesc(uint16 tag) const;
-        /*!
-          @brief Return the tag associated with a makernote tag name. The
-                 default implementation looks up the makernote info tag array
-                 if one is set, else it expects tag names in the form \"0x01ff\"
-                 and converts them to unsigned integer.
-         */
-        virtual void taglist(std::ostream& os) const;
-        /*!
-          @brief Write the makernote tag info of tag to the output stream os.
-         */
-        virtual std::ostream& writeMnTagInfo(std::ostream& os, uint16 tag) const;
-        /*!
-          @brief Return a pointer to an newly created, empty instance of the 
-                 same type as this. The makernote entries are <B>not</B> copied.
-                 The caller owns the new object and is responsible to delete it!
+          @brief Return an auto-pointer to an newly created, empty instance of
+                 the same type as this. The makernote entries are <B>not</B>
+                 copied.  The caller owns the new object and the auto-pointer
+                 ensures that it will be deleted.
 
-          @param alloc Memory management model for the clone. Indicates if 
-                 memory required to store data should be allocated and deallocated
-                 (true) or not (false). If false, only pointers to the buffer
-                 provided to read() will be kept. See Ifd for more background on 
-                 this concept.
+          @param alloc Memory management model for the newly created object. 
+                 Indicates if memory required to store data should be allocated
+                 and deallocated (true) or not (false). If false, only pointers
+                 to the buffer provided to read() will be kept. See Ifd for more
+                 background on this concept.
          */
-        virtual MakerNote* clone(bool alloc =true) const =0;
+        AutoPtr create(bool alloc =true) const;
+        /*!
+          @brief Return an auto-pointer to a clone of this object. The caller
+                 owns the new object and the auto-pointer ensures that it will
+                 be deleted.
+
+          @note  In non-alloc mode the clone potentially contains pointers to 
+                 the same data buffer as the original. 
+                 Use updateBase(byte* pNewBase) to adjust them.
+         */
+        AutoPtr clone() const;
         //! The first makernote entry
         virtual Entries::const_iterator begin() const =0;
         //! End of the makernote entries
@@ -222,18 +192,10 @@ namespace Exiv2 {
         virtual Entries::const_iterator findIdx(int idx) const =0;
         //! Return the size of the makernote in bytes
         virtual long size() const =0;
-        //! Return the name of the makernote item
-        virtual std::string ifdItem() const =0; 
-        //! Interpret and print the value of a makernote tag
-        virtual std::ostream& printTag(std::ostream& os,
-                                       uint16 tag, 
-                                       const Value& value) const =0;
         //@}
 
     protected:
         // DATA
-        //! Pointer to an array of makernote tag infos
-        const MnTagInfo* pMnTagInfo_;   
         /*!
           @brief Flag to control the memory management: <BR>
                  True:  requires memory allocation and deallocation, <BR>
@@ -251,8 +213,17 @@ namespace Exiv2 {
          */
         ByteOrder byteOrder_;
 
+    private:
+        //! Internal virtual create function.
+        virtual MakerNote* create_(bool alloc =true) const =0;
+        //! Internal virtual copy constructor.
+        virtual MakerNote* clone_() const =0;
+
     }; // class MakerNote
 
+    //! Type for a pointer to a function creating a makernote
+    typedef MakerNote::AutoPtr (*CreateFct)(bool, const byte*, long, ByteOrder, long);
+
     /*!
       @brief Interface for MakerNotes in IFD format. See MakerNote.
      */
@@ -264,15 +235,19 @@ namespace Exiv2 {
         //@}
 
     public:
+        //! Shortcut for an %IfdMakerNote auto pointer.
+        typedef std::auto_ptr<IfdMakerNote> AutoPtr;
+
         //! @name Creators
         //@{        
         /*!
-          @brief Constructor. Takes an optional makernote info tag array and
-                 allows to choose whether or not memory management is required
-                 for the Entries.
-         */
-        explicit IfdMakerNote(const MakerNote::MnTagInfo* pMnTagInfo =0, 
-                              bool alloc =true);
+          @brief Constructor. Requires an %Ifd id and allows to choose whether 
+                 or not memory management is needed for the Entries and whether
+                 the IFD has a next pointer.
+        */
+        explicit IfdMakerNote(IfdId ifdId, bool alloc =true, bool hasNext =true);
+        //! Copy constructor
+        IfdMakerNote(const IfdMakerNote& rhs);
         //! Virtual destructor
         virtual ~IfdMakerNote() {}
         //@}
@@ -295,17 +270,20 @@ namespace Exiv2 {
                                long len,
                                ByteOrder byteOrder);
         virtual long copy(byte* buf, ByteOrder byteOrder, long offset);
-        void add(const Entry& entry) { ifd_.add(entry); }
-        Entries::iterator begin() { return ifd_.begin(); }
-        Entries::iterator end() { return ifd_.end(); }
+        virtual void add(const Entry& entry) { ifd_.add(entry); }
+        virtual Entries::iterator begin() { return ifd_.begin(); }
+        virtual Entries::iterator end() { return ifd_.end(); }
+        virtual void updateBase(byte* pNewBase);
         //@}
 
         //! @name Accessors
         //@{
-        Entries::const_iterator begin() const { return ifd_.begin(); }
-        Entries::const_iterator end() const { return ifd_.end(); }
-        Entries::const_iterator findIdx(int idx) const;
-        long size() const;
+        virtual Entries::const_iterator begin() const { return ifd_.begin(); }
+        virtual Entries::const_iterator end() const { return ifd_.end(); }
+        virtual Entries::const_iterator findIdx(int idx) const;
+        virtual long size() const;
+        AutoPtr create(bool alloc =true) const;
+        AutoPtr clone() const;
         /*!
           @brief Check the makernote header. This will typically check if a
                  required prefix string is present in the header. Return 0 if
@@ -326,18 +304,14 @@ namespace Exiv2 {
                  buffer.
          */
         virtual long headerSize() const;
-        virtual IfdMakerNote* clone(bool alloc =true) const =0;
-        virtual std::string ifdItem() const =0; 
-        virtual std::ostream& printTag(std::ostream& os,
-                                       uint16 tag, 
-                                       const Value& value) const =0;
         //@}
 
     protected:
         // DATA
         /*!
           @brief True:  Adjustment of the IFD offsets is to be added to the
-                        offset from the start of the TIFF header,
+                        offset from the start of the TIFF header (i.e., the
+                        start of the Exif data section),
                  False: Adjustment of the IFD offsets is a suitable absolute 
                         value. Ignore the offset from the start of the TIFF 
                         header.
@@ -354,6 +328,10 @@ namespace Exiv2 {
         //! The makernote IFD
         Ifd ifd_;
 
+    private:
+        virtual IfdMakerNote* create_(bool alloc =true) const =0;
+        virtual IfdMakerNote* clone_() const =0;
+
     }; // class IfdMakerNote
 
     /*!
@@ -362,21 +340,13 @@ namespace Exiv2 {
       Maintains an associative list (tree) of camera makes/models and
       corresponding %MakerNote create functions. Creates an instance of the
       %MakerNote for one camera make/model. The factory is implemented as a
-      singleton, which can be accessed only through the static member function
-      instance().
+      static class.
     */
     class MakerNoteFactory {
     public:
         /*!
-          @brief Access the MakerNote factory. Clients access the task factory
-                 exclusively through this method.
-        */
-        static MakerNoteFactory& instance();
-
-        //! @name Manipulators
-        //@{        
-        /*!
-          @brief Register a %MakerNote create function for a camera make and model.
+          @brief Register a %MakerNote create function for a camera make and
+                 model.
 
           Registers a create function for a %MakerNote for a given make and
           model combination with the factory. Both the make and model strings
@@ -392,18 +362,18 @@ namespace Exiv2 {
           @param createMakerNote Pointer to a function to create a new 
                  %MakerNote of a particular type.
         */
-        void registerMakerNote(const std::string& make, 
-                               const std::string& model, 
-                               CreateFct createMakerNote);
-        //@}
+        static void registerMakerNote(const std::string& make, 
+                                      const std::string& model, 
+                                      CreateFct createMakerNote);
+        
+        //! Register a %MakerNote prototype in the IFD id registry.
+        static void registerMakerNote(IfdId ifdId, MakerNote::AutoPtr makerNote);
 
-        //! @name Accessors
-        //@{
         /*!
           @brief Create the appropriate %MakerNote based on camera make and
                  model and possibly the contents of the makernote itself, return
-                 a pointer to the newly created MakerNote instance. Return 0 if
-                 no %MakerNote is defined for the camera model.
+                 an auto-pointer to the newly created MakerNote instance. Return
+                 0 if no %MakerNote is defined for the camera model.
 
           The method searches the make-model tree for a make and model
           combination in the registry that matches the search key. The search is
@@ -411,17 +381,17 @@ namespace Exiv2 {
           wildcards in the registry entries are supported. First the best
           matching make is searched, then the best matching model for this make
           is searched. If there is no matching make or no matching model within
-          the models registered for the best matching make, then no maker note
+          the models registered for the best matching make, then no makernote
           is created and the function returns 0. If a match is found, the
-          function invokes the registered create function and returns a pointer
-          to the newly created MakerNote. The makernote pointed to is owned by
-          the caller of the function, i.e., the caller is responsible to delete
-          the returned makernote when it is no longer needed.  The best match is
-          an exact match, then a match is rated according to the number of
-          matching characters. The makernote buffer is passed on to the create
-          function, which can based on its content, automatically determine the
-          correct version or flavour of the makernote required. This is used,
-          e.g., to determine which of the three Nikon makernotes to create.
+          function invokes the registered create function and returns an
+          auto-pointer to the newly created MakerNote. The makernote pointed to
+          is owned by the caller of the function and the auto-pointer ensures
+          that it is deleted.  The best match is an exact match, then a match is
+          rated according to the number of matching characters. The makernote
+          buffer is passed on to the create function, which can based on its
+          content, automatically determine the correct version or flavour of the
+          makernote required. This is used, e.g., to determine which of the
+          three Nikon makernotes to create.
 
           @param make Camera manufacturer. (Typically the string from the Exif
                  make tag.)
@@ -439,17 +409,19 @@ namespace Exiv2 {
           @param offset Offset from the start of the TIFF header of the makernote
                  buffer.
 
-          @return A pointer that owns a %MakerNote for the camera model.  If the
-                 camera is not supported, the pointer is 0.
+          @return An auto-pointer that owns a %MakerNote for the camera model.  
+                 If the camera is not supported, the pointer is 0.
          */
-        MakerNote* create(const std::string& make, 
-                          const std::string& model, 
-                          bool alloc, 
-                          const byte* buf, 
-                          long len, 
-                          ByteOrder byteOrder, 
-                          long offset) const; 
-        //@}
+        static MakerNote::AutoPtr create(const std::string& make, 
+                                         const std::string& model, 
+                                         bool alloc, 
+                                         const byte* buf, 
+                                         long len, 
+                                         ByteOrder byteOrder, 
+                                         long offset); 
+
+        //! Create a %MakerNote for an IFD id.
+        static MakerNote::AutoPtr create(IfdId ifdId, bool alloc =true);
 
         /*!
           @brief Match a registry entry with a key (used for make and model).
@@ -473,22 +445,27 @@ namespace Exiv2 {
     private:
         //! @name Creators
         //@{                
-        //! Prevent construction other than through instance().
+        //! Prevent construction: not implemented.
         MakerNoteFactory() {}
         //! Prevent copy construction: not implemented.
         MakerNoteFactory(const MakerNoteFactory& rhs);
         //@}
 
+        //! Creates the private static instance
+        static void init();
+
         //! Type used to store model labels and %MakerNote create functions
         typedef std::vector<std::pair<std::string, CreateFct> > ModelRegistry;
         //! Type used to store a list of make labels and model registries
         typedef std::vector<std::pair<std::string, ModelRegistry*> > Registry;
+        //! Type used to store a list of IFD ids and %MakerNote prototypes
+        typedef std::map<IfdId, MakerNote*> IfdIdRegistry;
 
         // DATA
-        //! Pointer to the one and only instance of this class.
-        static MakerNoteFactory* pInstance_;
         //! List of makernote types and corresponding makernote create functions.
-        Registry registry_;
+        static Registry* pRegistry_;
+        //! List of makernote IFD ids and corresponding create functions.
+        static IfdIdRegistry* pIfdIdRegistry_;
 
     }; // class MakerNoteFactory
    
