@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004, 2005 Andreas Huggel <ahuggel@gmx.net>
  * 
  * This program is part of the Exiv2 distribution.
  *
@@ -21,11 +21,11 @@
 /*!
   @file    metadatum.hpp
   @brief   Provides abstract base classes Metadatum and Key
-  @version Exiv2-0_5_040913 1.2
+  @version $Rev: 560 $
   @author  Andreas Huggel (ahu)
            <a href="mailto:ahuggel@gmx.net">ahuggel@gmx.net</a>
   @author  Brad Schick (brad) 
-           <a href="mailto:schick@robotbattle.com">schick@robotbattle.com</a>
+           <a href="mailto:brad@robotbattle.com">brad@robotbattle.com</a>
   @date    09-Jan-04, ahu: created<BR>
            31-Jul-04, brad: isolated as a component<BR>
            23-Aug-04, ahu: added Key
@@ -40,6 +40,7 @@
 
 // + standard includes
 #include <string>
+#include <memory>
 
 // *****************************************************************************
 // namespace extensions
@@ -54,6 +55,9 @@ namespace Exiv2 {
     */
     class Key {
     public:
+        //! Shortcut for a %Key auto pointer.
+        typedef std::auto_ptr<Key> AutoPtr;
+
         //! @name Creators
         //@{
         //! Destructor
@@ -76,12 +80,13 @@ namespace Exiv2 {
         //! Return the name of the tag (which is also the third part of the key)
         virtual std::string tagName() const =0;
         //! Return the tag number
-        virtual uint16 tag() const =0;
+        virtual uint16_t tag() const =0;
         /*!
-          @brief Return a pointer to a copy of itself (deep copy).
-                 The caller owns this copy and is responsible to delete it!
+          @brief Return an auto-pointer to a copy of itself (deep copy).
+                 The caller owns this copy and the auto-pointer ensures that it
+                 will be deleted.
          */
-        virtual Key* clone() const =0;
+        AutoPtr clone() const;
         /*! 
           @brief Write the key to an output stream. You do not usually have
                  to use this function; it is used for the implementation of 
@@ -101,6 +106,10 @@ namespace Exiv2 {
         Key& operator=(const Key& rhs) { return *this; }
         //@}
 
+    private:
+        //! Internal virtual copy constructor.
+        virtual Key* clone_() const =0;
+
     }; // class Key
 
     //! Output operator for Key types
@@ -164,7 +173,7 @@ namespace Exiv2 {
         //! Return the name of the tag (which is also the third part of the key)
         virtual std::string tagName() const =0;
         //! Return the tag
-        virtual uint16 tag() const =0;
+        virtual uint16_t tag() const =0;
         //! Return the type id of the value
         virtual TypeId typeId() const =0;
         //! Return the name of the type
@@ -199,18 +208,19 @@ namespace Exiv2 {
          */
         virtual Rational toRational(long n =0) const =0;
         /*!
-          @brief Return a pointer to a copy (clone) of the value. The caller
-                 is responsible to delete this copy when it's no longer needed.
+          @brief Return an auto-pointer to a copy (clone) of the value. The
+                 caller owns this copy and the auto-poiner ensures that it will
+                 be deleted.
 
           This method is provided for users who need full control over the 
           value. A caller may, e.g., downcast the pointer to the appropriate
           subclass of Value to make use of the interface of the subclass to set
           or modify its contents.
           
-          @return A pointer to a copy (clone) of the value, 0 if the value is 
-                  not set.
+          @return An auto-pointer containing a pointer to a copy (clone) of the 
+                  value, 0 if the value is not set.
          */
-        virtual Value* getValue() const =0;
+        virtual Value::AutoPtr getValue() const =0;
         /*!
           @brief Return a constant reference to the value. 
 
@@ -228,7 +238,7 @@ namespace Exiv2 {
           @endcode
 
           @return A constant reference to the value.
-          @throw Error ("Value not set") if the value is not set.
+          @throw Error if the value is not set.
          */
         virtual const Value& value() const =0;
         //@}
@@ -241,7 +251,7 @@ namespace Exiv2 {
                  by subclasses but not directly.
          */
         Metadatum& operator=(const Metadatum& rhs) { return *this; }
-        //@}        
+        //@}
 
     }; // class Metadatum
 
