@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004, 2005 Andreas Huggel <ahuggel@gmx.net>
  * 
  * This program is part of the Exiv2 distribution.
  *
@@ -20,7 +20,7 @@
  */
 /*
   File:      sigmamn.cpp
-  Version:   Exiv2-0_5_040913 1.9
+  Version:   $Rev: 569 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
   History:   02-Apr-04, ahu: created
   Credits:   Sigma and Foveon MakerNote implemented according to the specification
@@ -29,7 +29,7 @@
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) Exiv2-0_5_040913 1.9 sigmamn.cpp,v");
+EXIV2_RCSID("@(#) $Id: sigmamn.cpp 569 2005-05-28 05:48:43Z ahuggel $");
 
 // *****************************************************************************
 // included header files
@@ -51,39 +51,58 @@ EXIV2_RCSID("@(#) Exiv2-0_5_040913 1.9 sigmamn.cpp,v");
 // class member definitions
 namespace Exiv2 {
 
-    const SigmaMakerNote::RegisterMakerNote SigmaMakerNote::register_;
+    //! @cond IGNORE
+    SigmaMakerNote::RegisterMn::RegisterMn()
+    {
+        MakerNoteFactory::registerMakerNote("SIGMA", "*", createSigmaMakerNote); 
+        MakerNoteFactory::registerMakerNote("FOVEON", "*", createSigmaMakerNote);
+        MakerNoteFactory::registerMakerNote(
+            sigmaIfdId, MakerNote::AutoPtr(new SigmaMakerNote));
+
+        ExifTags::registerMakerTagInfo(sigmaIfdId, tagInfo_);
+    }
+    //! @endcond
 
     // Sigma (Foveon) MakerNote Tag Info
-    static const MakerNote::MnTagInfo sigmaMnTagInfo[] = {
-        MakerNote::MnTagInfo(0x0002, "SerialNumber", "Camera serial number"),
-        MakerNote::MnTagInfo(0x0003, "DriveMode", "Drive Mode"),
-        MakerNote::MnTagInfo(0x0004, "ResolutionMode", "Resolution Mode"),
-        MakerNote::MnTagInfo(0x0005, "AutofocusMode", "Autofocus mode"),
-        MakerNote::MnTagInfo(0x0006, "FocusSetting", "Focus setting"),
-        MakerNote::MnTagInfo(0x0007, "WhiteBalance", "White balance"),
-        MakerNote::MnTagInfo(0x0008, "ExposureMode", "Exposure mode"),
-        MakerNote::MnTagInfo(0x0009, "MeteringMode", "Metering mode"),
-        MakerNote::MnTagInfo(0x000a, "LensRange", "Lens focal length range"),
-        MakerNote::MnTagInfo(0x000b, "ColorSpace", "Color space"),
-        MakerNote::MnTagInfo(0x000c, "Exposure", "Exposure"),
-        MakerNote::MnTagInfo(0x000d, "Contrast", "Contrast"),
-        MakerNote::MnTagInfo(0x000e, "Shadow", "Shadow"),
-        MakerNote::MnTagInfo(0x000f, "Highlight", "Highlight"),
-        MakerNote::MnTagInfo(0x0010, "Saturation", "Saturation"),
-        MakerNote::MnTagInfo(0x0011, "Sharpness", "Sharpness"),
-        MakerNote::MnTagInfo(0x0012, "FillLight", "X3 Fill light"),
-        MakerNote::MnTagInfo(0x0014, "ColorAdjustment", "Color adjustment"),
-        MakerNote::MnTagInfo(0x0015, "AdjustmentMode", "Adjustment mode"),
-        MakerNote::MnTagInfo(0x0016, "Quality", "Quality"),
-        MakerNote::MnTagInfo(0x0017, "Firmware", "Firmware"),
-        MakerNote::MnTagInfo(0x0018, "Software", "Software"),
-        MakerNote::MnTagInfo(0x0019, "AutoBracket", "Auto bracket"),
+    const TagInfo SigmaMakerNote::tagInfo_[] = {
+        TagInfo(0x0002, "SerialNumber", "Camera serial number", sigmaIfdId, makerTags, asciiString, printValue),
+        TagInfo(0x0003, "DriveMode", "Drive Mode", sigmaIfdId, makerTags, asciiString, printValue),
+        TagInfo(0x0004, "ResolutionMode", "Resolution Mode", sigmaIfdId, makerTags, asciiString, printValue),
+        TagInfo(0x0005, "AutofocusMode", "Autofocus mode", sigmaIfdId, makerTags, asciiString, printValue),
+        TagInfo(0x0006, "FocusSetting", "Focus setting", sigmaIfdId, makerTags, asciiString, printValue),
+        TagInfo(0x0007, "WhiteBalance", "White balance", sigmaIfdId, makerTags, asciiString, printValue),
+        TagInfo(0x0008, "ExposureMode", "Exposure mode", sigmaIfdId, makerTags, asciiString, print0x0008),
+        TagInfo(0x0009, "MeteringMode", "Metering mode", sigmaIfdId, makerTags, asciiString, print0x0009),
+        TagInfo(0x000a, "LensRange", "Lens focal length range", sigmaIfdId, makerTags, asciiString, printValue),
+        TagInfo(0x000b, "ColorSpace", "Color space", sigmaIfdId, makerTags, asciiString, printValue),
+        TagInfo(0x000c, "Exposure", "Exposure", sigmaIfdId, makerTags, asciiString, printStripLabel),
+        TagInfo(0x000d, "Contrast", "Contrast", sigmaIfdId, makerTags, asciiString, printStripLabel),
+        TagInfo(0x000e, "Shadow", "Shadow", sigmaIfdId, makerTags, asciiString, printStripLabel),
+        TagInfo(0x000f, "Highlight", "Highlight", sigmaIfdId, makerTags, asciiString, printStripLabel),
+        TagInfo(0x0010, "Saturation", "Saturation", sigmaIfdId, makerTags, asciiString, printStripLabel),
+        TagInfo(0x0011, "Sharpness", "Sharpness", sigmaIfdId, makerTags, asciiString, printStripLabel),
+        TagInfo(0x0012, "FillLight", "X3 Fill light", sigmaIfdId, makerTags, asciiString, printStripLabel),
+        TagInfo(0x0014, "ColorAdjustment", "Color adjustment", sigmaIfdId, makerTags, asciiString, printStripLabel),
+        TagInfo(0x0015, "AdjustmentMode", "Adjustment mode", sigmaIfdId, makerTags, asciiString, printValue),
+        TagInfo(0x0016, "Quality", "Quality", sigmaIfdId, makerTags, asciiString, printStripLabel),
+        TagInfo(0x0017, "Firmware", "Firmware", sigmaIfdId, makerTags, asciiString, printValue),
+        TagInfo(0x0018, "Software", "Software", sigmaIfdId, makerTags, asciiString, printValue),
+        TagInfo(0x0019, "AutoBracket", "Auto bracket", sigmaIfdId, makerTags, asciiString, printValue),
         // End of list marker
-        MakerNote::MnTagInfo(0xffff, "(UnknownSigmaMakerNoteTag)", "Unknown SigmaMakerNote tag")
+        TagInfo(0xffff, "(UnknownSigmaMakerNoteTag)", "Unknown SigmaMakerNote tag", sigmaIfdId, makerTags, invalidTypeId, printValue)
     };
 
     SigmaMakerNote::SigmaMakerNote(bool alloc)
-        : IfdMakerNote(sigmaMnTagInfo, alloc), ifdItem_("Sigma")
+        : IfdMakerNote(sigmaIfdId, alloc)
+    {
+        byte buf[] = {
+            'S', 'I', 'G', 'M', 'A', '\0', '\0', '\0', 0x01, 0x00
+        };
+        readHeader(buf, 10, byteOrder_);
+    }
+
+    SigmaMakerNote::SigmaMakerNote(const SigmaMakerNote& rhs)
+        : IfdMakerNote(rhs)
     {
     }
 
@@ -117,36 +136,27 @@ namespace Exiv2 {
         return rc;
     }
 
-    SigmaMakerNote* SigmaMakerNote::clone(bool alloc) const 
+    SigmaMakerNote::AutoPtr SigmaMakerNote::create(bool alloc) const
     {
-        SigmaMakerNote* pMakerNote = new SigmaMakerNote(alloc);
-        assert(pMakerNote);
-        pMakerNote->readHeader(header_.pData_, header_.size_, byteOrder_);
-        return pMakerNote;
+        return AutoPtr(create_(alloc));
     }
 
-    std::ostream& SigmaMakerNote::printTag(std::ostream& os, 
-                                          uint16 tag, 
-                                          const Value& value) const
+    SigmaMakerNote* SigmaMakerNote::create_(bool alloc) const
     {
-        switch (tag) {
-        case 0x000c: // fallthrough
-        case 0x000d: // fallthrough
-        case 0x000e: // fallthrough
-        case 0x000f: // fallthrough
-        case 0x0010: // fallthrough
-        case 0x0011: // fallthrough
-        case 0x0012: // fallthrough
-        case 0x0014: // fallthrough
-        case 0x0016: printStripLabel(os, value); break;
-        case 0x0008: print0x0008(os, value); break;
-        case 0x0009: print0x0009(os, value); break;
-        default:
-            // All other tags (known or unknown) go here
-            os << value;
-            break;
-        }
-        return os;
+        AutoPtr makerNote = AutoPtr(new SigmaMakerNote(alloc));
+        assert(makerNote.get() != 0);
+        makerNote->readHeader(header_.pData_, header_.size_, byteOrder_);
+        return makerNote.release();
+    }
+
+    SigmaMakerNote::AutoPtr SigmaMakerNote::clone() const
+    {
+        return AutoPtr(clone_());
+    }
+
+    SigmaMakerNote* SigmaMakerNote::clone_() const
+    {
+        return new SigmaMakerNote(*this);
     }
 
     std::ostream& SigmaMakerNote::printStripLabel(std::ostream& os,
@@ -189,13 +199,13 @@ namespace Exiv2 {
 // *****************************************************************************
 // free functions
 
-    MakerNote* createSigmaMakerNote(bool alloc,
-                                    const byte* buf, 
-                                    long len, 
-                                    ByteOrder byteOrder, 
-                                    long offset)
+    MakerNote::AutoPtr createSigmaMakerNote(bool alloc,
+                                            const byte* buf, 
+                                            long len, 
+                                            ByteOrder byteOrder, 
+                                            long offset)
     {
-        return new SigmaMakerNote(alloc);
+        return MakerNote::AutoPtr(new SigmaMakerNote(alloc));
     }
 
 }                                       // namespace Exiv2
