@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004, 2005 Andreas Huggel <ahuggel@gmx.net>
  * 
  * This program is part of the Exiv2 distribution.
  *
@@ -21,7 +21,7 @@
 /*!
   @file    tags.hpp
   @brief   Exif tag and type information
-  @version Exiv2-0_5_040913 1.25
+  @version $Rev: 580 $
   @author  Andreas Huggel (ahu)
            <a href="mailto:ahuggel@gmx.net">ahuggel@gmx.net</a>
   @date    15-Jan-04, ahu: created<BR>
@@ -32,12 +32,14 @@
 
 // *****************************************************************************
 // included header files
+#include "metadatum.hpp"
 #include "types.hpp"
 
 // + standard includes
 #include <string>
 #include <utility>                              // for std::pair
 #include <iosfwd>
+#include <memory>
 
 // *****************************************************************************
 // namespace extensions
@@ -46,6 +48,7 @@ namespace Exiv2 {
 // *****************************************************************************
 // class declarations
     class Value;
+    class Entry;
 
 // *****************************************************************************
 // type definitions
@@ -60,7 +63,7 @@ namespace Exiv2 {
     enum SectionId { sectionIdNotSet, 
                      imgStruct, recOffset, imgCharacter, otherTags, exifFormat, 
                      exifVersion, imgConfig, userInfo, relatedFile, dateTime,
-                     captureCond, gpsTags, iopTags, 
+                     captureCond, gpsTags, iopTags, makerTags,
                      lastSectionId };
 
 // *****************************************************************************
@@ -89,21 +92,55 @@ namespace Exiv2 {
     struct TagInfo {
         //! Constructor
         TagInfo(
-            uint16 tag, 
+            uint16_t tag, 
             const char* name,
             const char* desc, 
             IfdId ifdId,
             SectionId sectionId,
+            TypeId typeId,
             PrintFct printFct
         );
-        uint16 tag_;                            //!< Tag
+        uint16_t tag_;                          //!< Tag
         const char* name_;                      //!< One word tag label
         const char* desc_;                      //!< Short tag description
         IfdId ifdId_;                           //!< Link to the (prefered) IFD
         SectionId sectionId_;                   //!< Section id
+        TypeId typeId_;                         //!< Type id
         PrintFct printFct_;                     //!< Pointer to tag print function
     }; // struct TagInfo
 
+    /*!
+      @brief Helper structure for lookup tables for translations of numeric 
+             tag values to human readable labels.
+     */
+    struct TagDetails {
+        long val_;                              //!< Tag value
+        char* label_;                           //!< Translation of the tag value
+    }; // struct TagDetails
+
+    /*!
+      @brief Translation from numeric values from a lookup list to human
+             readable labels
+     */
+    class TagTranslator {
+    public:
+        //! @name Creators
+        //@{
+        //! Default constructor.
+        explicit TagTranslator(const TagDetails* pTagDetails)
+            : pTagDetails_(pTagDetails) {}
+        // No d'tor: Do not delete the list.
+        //@}
+
+        //! @name Accessors
+        //@{
+        //! Translate the tag value and write it out to the provided stream
+        std::ostream& print(std::ostream& os, const Value& value) const;
+        //@}
+    private:
+        const TagDetails* pTagDetails_;
+    }; // class TagTranslator
+
     //! Container for Exif tag information. Implemented as a static class.
     class ExifTags {
         //! Prevent construction: not implemented.
@@ -114,31 +151,36 @@ namespace Exiv2 {
         ExifTags& operator=(const ExifTags& rhs);
 
     public:
-        //! Return an identifier for Exif metadata
-        static const char* familyName() { return familyName_; }
-
         /*!
-          @brief Return the name of the tag.
+          @brief Return the name of the tag or a string with the hexadecimal 
+                 value of the tag in the form "0x01ff", if the tag is not 
+                 a known Exif tag. 
+
           @param tag The tag
           @param ifdId IFD id
-          @return The name of the tag or a string indicating that the 
-                  tag is unknown. 
-          @throw Error ("No taginfo for IFD") if there is no tag info
-                 data for the given IFD id in the lookup tables.
+          @return The name of the tag or a string containing the hexadecimal
+                  value of the tag in the form "0x01ff", if this is an unknown 
+                  tag.
          */
-        static const char* tagName(uint16 tag, IfdId ifdId);
+        static std::string tagName(uint16_t tag, IfdId ifdId);
         /*!
           @brief Return the description of the tag.
           @param tag The tag
           @param ifdId IFD id
           @return The description of the tag or a string indicating that
                  the tag is unknown. 
-          @throw Error ("No taginfo for IFD") if there is no tag info
-                 data for the given IFD id in the lookup tables.
          */
-        static const char* tagDesc(uint16 tag, IfdId ifdId);
-        //! Return the tag for one combination of IFD id and tagName
-        static uint16 tag(const std::string& tagName, IfdId ifdId);
+        static const char* tagDesc(uint16_t tag, IfdId ifdId);
+        /*!
+          @brief Return the tag for one combination of IFD id and tagName. 
+                 If the tagName is not known, it expects tag names in the 
+                 form "0x01ff" and converts them to unsigned integer.
+
+          @throw Error if the tagname or ifdId is invalid
+         */
+        static uint16_t tag(const std::string& tagName, IfdId ifdId);
+        //! Return the IFD id for an IFD item
+        static IfdId ifdIdByIfdItem(const std::string& ifdItem);
         //! Return the name of the IFD
         static const char* ifdName(IfdId ifdId);
         //! Return the related image item (image or thumbnail)
@@ -152,10 +194,8 @@ namespace Exiv2 {
           @param ifdId IFD id
           @return The name of the section or a string indicating that the 
                   section or the tag is unknown. 
-          @throw Error ("No taginfo for IFD") if there is no tag info
-                 data for the given IFD id in the lookup tables.
          */
-        static const char* sectionName(uint16 tag, IfdId ifdId);
+        static const char* sectionName(uint16_t tag, IfdId ifdId);
         /*!
           @brief Return the description of the section for a combination of 
                  tag and IFD id.
@@ -163,47 +203,165 @@ namespace Exiv2 {
           @param ifdId IFD id
           @return The description of the section or a string indicating that
                  the section or the tag is unknown. 
-          @throw Error ("No taginfo for IFD") if there is no tag info
-                 data for the given IFD id in the lookup tables.
          */
-        static const char* sectionDesc(uint16 tag, IfdId ifdId);
+        static const char* sectionDesc(uint16_t tag, IfdId ifdId);
         //! Return the section id for a section name
         static SectionId sectionId(const std::string& sectionName);
-        /*!
-          @brief Return the key for the tag and IFD id.  The key is of the form
-                 '<b>Exif</b>.ifdItem.tagName'.
-         */
-        static std::string makeKey(uint16 tag, IfdId ifdId);
-        /*!
-          @brief Return tag and IFD id pair for the key.
-          @return A pair consisting of the tag and IFD id.
-          @throw Error ("Invalid key") if the key cannot be parsed into
-                 item item, section name and tag name parts.
-         */
-        static std::pair<uint16, IfdId> decomposeKey(const std::string& key);
+        //! Return the type for tag and IFD id
+        static TypeId tagType(uint16_t tag, IfdId ifdId);
         //! Interpret and print the value of an Exif tag
         static std::ostream& printTag(std::ostream& os,
-                                      uint16 tag, 
+                                      uint16_t tag, 
                                       IfdId ifdId,
                                       const Value& value);
-        //! Print a list of all tags to output stream
+        //! Print a list of all standard Exif tags to output stream
         static void taglist(std::ostream& os);
+        //! Print a list of all tags related to one makernote %IfdId
+        static void makerTaglist(std::ostream& os, IfdId ifdId);
+        //! Register an %IfdId with the base IFD %TagInfo list for a makernote
+        static void registerBaseTagInfo(IfdId ifdId);
+        /*!
+          @brief Register an %IfdId and %TagInfo list for a makernote
+
+          @throw Error if the MakerTagInfo registry is full
+         */
+        static void registerMakerTagInfo(IfdId ifdId, const TagInfo* tagInfo);
+        /*!
+          @brief Return true if \em ifdId is an %Ifd Id which is registered
+                 as a makernote %Ifd id. Note: Calling this function with 
+                 makerIfd returns false.
+        */
+        static bool isMakerIfd(IfdId ifdId);
 
     private:
-        static int tagInfoIdx(uint16 tag, IfdId ifdId);
-
-        static const char* familyName_;
+        static int tagInfoIdx(uint16_t tag, IfdId ifdId);
+        static const TagInfo* makerTagInfo(uint16_t tag, IfdId ifdId);
+        static const TagInfo* makerTagInfo(const std::string& tagName, 
+                                           IfdId ifdId);
 
         static const IfdInfo     ifdInfo_[];
         static const SectionInfo sectionInfo_[];
 
         static const TagInfo*    tagInfos_[];
 
+        static const int         MAX_MAKER_TAG_INFOS = 64;
+        static const TagInfo*    makerTagInfos_[MAX_MAKER_TAG_INFOS];
+        static IfdId             makerIfdIds_[MAX_MAKER_TAG_INFOS];
+
     }; // class ExifTags
 
+    /*!
+      @brief Concrete keys for Exif metadata.
+     */
+    class ExifKey : public Key {
+    public:
+        //! Shortcut for an %ExifKey auto pointer.
+        typedef std::auto_ptr<ExifKey> AutoPtr;
+
+        //! @name Creators
+        //@{
+        /*!
+          @brief Constructor to create an Exif key from a key string.
+
+          @param key The key string.
+          @throw Error if the first part of the key is not '<b>Exif</b>' or
+                 the remainin parts of the key cannot be parsed and
+                 converted to an ifd-item and tag name. 
+        */
+        explicit ExifKey(const std::string& key);
+        /*!
+          @brief Constructor to create an Exif key from a tag and IFD item 
+                 string.
+          @param tag The tag value
+          @param ifdItem The IFD string. For MakerNote tags, this must be the 
+                 IFD item of the specific MakerNote. "MakerNote" is not allowed.
+          @throw Error if the key cannot be constructed from the tag and IFD
+                 item parameters.
+         */
+        ExifKey(uint16_t tag, const std::string& ifdItem);
+        //! Constructor to build an ExifKey from an IFD entry.
+        explicit ExifKey(const Entry& e);
+        //! Copy constructor
+        ExifKey(const ExifKey& rhs);
+        virtual ~ExifKey();
+        //@}
+
+        //! @name Manipulators
+        //@{
+        /*!
+          @brief Assignment operator.
+         */
+        ExifKey& operator=(const ExifKey& rhs);
+        //@}
+
+        //! @name Accessors
+        //@{
+        virtual std::string key() const { return key_; }
+        virtual const char* familyName() const { return familyName_; }
+        /*!
+          @brief Return the name of the group (the second part of the key).
+                 For Exif keys, the group name is the IFD item.
+        */ 
+        virtual std::string groupName() const { return ifdItem(); }
+        virtual std::string tagName() const;
+        virtual uint16_t tag() const { return tag_; }
+
+        AutoPtr clone() const;
+        //! Return the IFD id
+        IfdId ifdId() const { return ifdId_; }
+        //! Return the name of the IFD
+        const char* ifdName() const { return ExifTags::ifdName(ifdId()); }
+        //! Return the related image item
+        std::string ifdItem() const { return ifdItem_; }
+        //! Return the name of the Exif section (deprecated) 
+        std::string sectionName() const;
+        //! Return the index (unique id of this key within the original IFD)
+        int idx() const { return idx_; }
+        //@}
+
+    protected:
+        //! @name Manipulators
+        //@{
+        /*!
+          @brief Set the key corresponding to the tag and IFD id. 
+                 The key is of the form '<b>Exif</b>.ifdItem.tagName'.
+         */
+        void makeKey();
+        /*!
+          @brief Parse and convert the key string into tag and IFD Id. 
+                 Updates data members if the string can be decomposed,
+                 or throws \em Error .
+
+          @throw Error if the key cannot be decomposed.
+         */
+        void decomposeKey();
+        //@}
+
+    private:
+        //! Internal virtual copy constructor.        
+        virtual ExifKey* clone_() const;
+
+        // DATA
+        static const char* familyName_;
+
+        uint16_t tag_;                  //!< Tag value
+        IfdId ifdId_;                   //!< The IFD associated with this tag
+        std::string ifdItem_;           //!< The IFD item 
+        int idx_;                       //!< Unique id of an entry within one IFD
+        std::string key_;               //!< Key
+    }; // class ExifKey
+
 // *****************************************************************************
 // free functions
 
+    /*!
+      @brief Return true if \em ifdId is an Exif %Ifd Id, i.e., one of
+             ifd0Id, exifIfdId, gpsIfdId, iopIfdId or ifd1Id, else false.
+             This is used to differentiate between standard Exif %Ifds
+             and %Ifds associated with the makernote.
+     */
+    bool isExifIfd(IfdId ifdId);
+
     //! Output operator for TagInfo
     std::ostream& operator<<(std::ostream& os, const TagInfo& ti);
 
