@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004, 2005 Andreas Huggel <ahuggel@gmx.net>
  * 
  * This program is part of the Exiv2 distribution.
  *
@@ -20,14 +20,14 @@
  */
 /*
   File:      types.cpp
-  Version:   Exiv2-0_5_040913 1.12
+  Version:   $Rev: 578 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
   History:   26-Jan-04, ahu: created
              11-Feb-04, ahu: isolated as a component
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) Exiv2-0_5_040913 1.12 types.cpp,v");
+EXIV2_RCSID("@(#) $Id: types.cpp 578 2005-06-07 15:01:11Z ahuggel $");
 
 // *****************************************************************************
 // included header files
@@ -39,6 +39,7 @@ EXIV2_RCSID("@(#) Exiv2-0_5_040913 1.12 types.cpp,v");
 #include <iomanip>
 #include <sstream>
 #include <utility>
+#include <cctype>
 
 // *****************************************************************************
 // class member definitions
@@ -57,14 +58,17 @@ namespace Exiv2 {
         TypeInfoTable(unsignedShort,    "Short",       2),
         TypeInfoTable(unsignedLong,     "Long",        4),
         TypeInfoTable(unsignedRational, "Rational",    8),
-        TypeInfoTable(invalid6,         "Invalid (6)", 1),
+        TypeInfoTable(invalid6,         "Invalid(6)",  1),
         TypeInfoTable(undefined,        "Undefined",   1),
         TypeInfoTable(signedShort,      "SShort",      2),
         TypeInfoTable(signedLong,       "SLong",       4),
         TypeInfoTable(signedRational,   "SRational",   8),
         TypeInfoTable(string,           "String",      1),
         TypeInfoTable(date,             "Date",        8),
-        TypeInfoTable(time,             "Time",        11)
+        TypeInfoTable(time,             "Time",        11),
+        TypeInfoTable(comment,          "Comment",     1),
+        // End of list marker
+        TypeInfoTable(lastTypeId,       "(Unknown)",   0)
     };
 
     const char* TypeInfo::typeName(TypeId typeId)
@@ -72,15 +76,73 @@ namespace Exiv2 {
         return typeInfoTable_[ typeId < lastTypeId ? typeId : 0 ].name_;
     }
 
+    TypeId TypeInfo::typeId(const std::string& typeName)
+    {
+        int i = 0;
+        for (;    typeInfoTable_[i].typeId_ != lastTypeId
+               && typeInfoTable_[i].name_ != typeName; ++i) {}
+        return typeInfoTable_[i].typeId_ == lastTypeId ?
+               invalidTypeId : typeInfoTable_[i].typeId_;
+    }
+
     long TypeInfo::typeSize(TypeId typeId)
     {
         return typeInfoTable_[ typeId < lastTypeId ? typeId : 0 ].size_;
     }
+    
+    DataBuf::DataBuf(DataBuf& rhs)
+        : pData_(rhs.pData_), size_(rhs.size_)
+    {
+        rhs.release();
+    }
+
+    DataBuf::DataBuf(byte* pData, long size) 
+        : pData_(0), size_(0)
+    {
+        if (size > 0) {
+            pData_ = new byte[size];
+            memcpy(pData_, pData, size);
+            size_ = size;
+        }
+    }
+
+    DataBuf& DataBuf::operator=(DataBuf& rhs)
+    {
+        if (this == &rhs) return *this;
+        reset(rhs.release());
+        return *this;
+    }
+
+    void DataBuf::alloc(long size)
+    { 
+        if (size > size_) {
+            delete[] pData_; 
+            size_ = size; 
+            pData_ = new byte[size];
+        } 
+    }
+
+    std::pair<byte*, long> DataBuf::release()
+    {
+        std::pair<byte*, long> p = std::make_pair(pData_, size_);
+        pData_ = 0;
+        size_ = 0;
+        return p;
+    }
+
+    void DataBuf::reset(std::pair<byte*, long> p)
+    {
+        if (pData_ != p.first) {
+            delete[] pData_;
+            pData_ = p.first;
+        }
+        size_ = p.second;
+    }
 
     // *************************************************************************
     // free functions
 
-    uint16 getUShort(const byte* buf, ByteOrder byteOrder)
+    uint16_t getUShort(const byte* buf, ByteOrder byteOrder)
     {
         if (byteOrder == littleEndian) {
             return (byte)buf[1] << 8 | (byte)buf[0];
@@ -90,7 +152,7 @@ namespace Exiv2 {
         }
     }
 
-    uint32 getULong(const byte* buf, ByteOrder byteOrder)
+    uint32_t getULong(const byte* buf, ByteOrder byteOrder)
     {
         if (byteOrder == littleEndian) {
             return   (byte)buf[3] << 24 | (byte)buf[2] << 16 
@@ -104,12 +166,12 @@ namespace Exiv2 {
 
     URational getURational(const byte* buf, ByteOrder byteOrder)
     {
-        uint32 nominator = getULong(buf, byteOrder);
-        uint32 denominator = getULong(buf + 4, byteOrder);
+        uint32_t nominator = getULong(buf, byteOrder);
+        uint32_t denominator = getULong(buf + 4, byteOrder);
         return std::make_pair(nominator, denominator);
     }
 
-    int16 getShort(const byte* buf, ByteOrder byteOrder)
+    int16_t getShort(const byte* buf, ByteOrder byteOrder)
     {
         if (byteOrder == littleEndian) {
             return (byte)buf[1] << 8 | (byte)buf[0];
@@ -119,7 +181,7 @@ namespace Exiv2 {
         }
     }
 
-    int32 getLong(const byte* buf, ByteOrder byteOrder)
+    int32_t getLong(const byte* buf, ByteOrder byteOrder)
     {
         if (byteOrder == littleEndian) {
             return   (byte)buf[3] << 24 | (byte)buf[2] << 16 
@@ -133,12 +195,12 @@ namespace Exiv2 {
 
     Rational getRational(const byte* buf, ByteOrder byteOrder)
     {
-        int32 nominator = getLong(buf, byteOrder);
-        int32 denominator = getLong(buf + 4, byteOrder);
+        int32_t nominator = getLong(buf, byteOrder);
+        int32_t denominator = getLong(buf + 4, byteOrder);
         return std::make_pair(nominator, denominator);
     }
 
-    long us2Data(byte* buf, uint16 s, ByteOrder byteOrder)
+    long us2Data(byte* buf, uint16_t s, ByteOrder byteOrder)
     {
         if (byteOrder == littleEndian) {
             buf[0] =  (byte)(s & 0x00ff);
@@ -151,7 +213,7 @@ namespace Exiv2 {
         return 2;
     }
 
-    long ul2Data(byte* buf, uint32 l, ByteOrder byteOrder)
+    long ul2Data(byte* buf, uint32_t l, ByteOrder byteOrder)
     {
         if (byteOrder == littleEndian) {
             buf[0] =  (byte)(l & 0x000000ff);
@@ -175,7 +237,7 @@ namespace Exiv2 {
         return o;
     }
 
-    long s2Data(byte* buf, int16 s, ByteOrder byteOrder)
+    long s2Data(byte* buf, int16_t s, ByteOrder byteOrder)
     {
         if (byteOrder == littleEndian) {
             buf[0] =  (byte)(s & 0x00ff);
@@ -188,7 +250,7 @@ namespace Exiv2 {
         return 2;
     }
 
-    long l2Data(byte* buf, int32 l, ByteOrder byteOrder)
+    long l2Data(byte* buf, int32_t l, ByteOrder byteOrder)
     {
         if (byteOrder == littleEndian) {
             buf[0] =  (byte)(l & 0x000000ff);
@@ -225,7 +287,7 @@ namespace Exiv2 {
             std::ostringstream ss;
             do {
                 byte c = buf[i];
-                os << std::setw(2) << std::setfill('0') 
+                os << std::setw(2) << std::setfill('0') << std::right
                    << std::hex << (int)c << " ";
                 ss << ((int)c >= 31 && (int)c < 127 ? char(buf[i]) : '.');
             } while (++i < len && i%16 != 0);
@@ -265,4 +327,17 @@ namespace Exiv2 {
         return b;
     } // lgcd
 
+    bool isHex(const std::string& str, size_t size, const std::string& prefix)
+    {
+        if (   str.size() <= prefix.size() 
+            || str.substr(0, prefix.size()) != prefix) return false;
+        if (   size > 0
+            && str.size() != size + prefix.size()) return false;
+        
+        for (size_t i = prefix.size(); i < str.size(); ++i) {
+            if (!isxdigit(str[i])) return false;
+        }
+        return true;
+    } // isHex
+
 }                                       // namespace Exiv2
