@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004, 2005 Andreas Huggel <ahuggel@gmx.net>
  * 
  * This program is part of the Exiv2 distribution.
  *
@@ -21,7 +21,7 @@
 /*!
   @file    types.hpp
   @brief   Type definitions for %Exiv2 and related functionality
-  @version Exiv2-0_5_040913 1.20
+  @version $Rev: 581 $
   @author  Andreas Huggel (ahu)
            <a href="mailto:ahuggel@gmx.net">ahuggel@gmx.net</a>
   @date    09-Jan-04, ahu: created<BR>
@@ -33,19 +33,29 @@
 
 // *****************************************************************************
 // included header files
+#ifdef _MSC_VER
+# include "exv_msvc.h"
+#else
+# include "exv_conf.h"
+#endif
 
 // + standard includes
 #include <string>
 #include <iosfwd>
 #include <utility>
 #include <sstream>
+#include <cstdio>
+#ifdef EXV_HAVE_STDINT_H
+# include <stdint.h>
+#endif
 
-
+// MSVC doesn't provide C99 types, but it has MS specific variants
 #ifdef _MSC_VER
-// disable unreferenced formal parameter warning C4100
-// disable truncation of constant value warning C4309
-// disable warning conversion of int to bool C4800
-#pragma warning (disable: 4100 4309 4800)
+typedef unsigned __int8  uint8_t;
+typedef unsigned __int16 uint16_t;
+typedef unsigned __int32 uint32_t;
+typedef __int16          int16_t;
+typedef __int32          int32_t;
 #endif
 
 // *****************************************************************************
@@ -56,22 +66,12 @@ namespace Exiv2 {
 // type definitions
 
     //! 1 byte unsigned integer type.
-    typedef unsigned char byte;
-    //! 2 byte unsigned integer type.
-    typedef unsigned short uint16;
-    //! 4 byte unsigned integer type.
-    typedef unsigned long  uint32;
-    //! 2 byte signed integer type.
-    typedef short          int16;
-    //! 4 byte signed integer type.
-    typedef long           int32;
-
-
+    typedef uint8_t byte;
 
     //! 8 byte unsigned rational type.
-    typedef std::pair<uint32, uint32> URational;
+    typedef std::pair<uint32_t, uint32_t> URational;
     //! 8 byte signed rational type.
-    typedef std::pair<int32, int32> Rational;
+    typedef std::pair<int32_t, int32_t> Rational;
 
     //! Type to express the byte order (little or big endian)
     enum ByteOrder { invalidByteOrder, littleEndian, bigEndian };
@@ -80,12 +80,17 @@ namespace Exiv2 {
     enum TypeId { invalidTypeId, unsignedByte, asciiString, unsignedShort, 
                   unsignedLong, unsignedRational, invalid6, undefined, 
                   signedShort, signedLong, signedRational, 
-                  string, date, time,
+                  string, date, time, 
+                  comment,
                   lastTypeId };
 
+    // Todo: decentralize IfdId, so that new ids can be defined elsewhere
     //! Type to specify the IFD to which a metadata belongs
     enum IfdId { ifdIdNotSet, 
-                 ifd0Id, exifIfdId, gpsIfdId, makerIfdId, iopIfdId, ifd1Id, 
+                 ifd0Id, exifIfdId, gpsIfdId, iopIfdId, ifd1Id, 
+                 canonIfdId, canonCs1IfdId, canonCs2IfdId, canonCfIfdId,
+                 fujiIfdId, nikon1IfdId, nikon2IfdId, nikon3IfdId, 
+                 olympusIfdId, panasonicIfdId, sigmaIfdId, sonyIfdId,
                  lastIfdId };
 
 // *****************************************************************************
@@ -112,6 +117,8 @@ namespace Exiv2 {
     public:
         //! Return the name of the type
         static const char* typeName(TypeId typeId);
+        //! Return the type id for a type name
+        static TypeId typeId(const std::string& typeName);
         //! Return the size in bytes of one element of this type
         static long typeSize(TypeId typeId);
 
@@ -119,48 +126,99 @@ namespace Exiv2 {
         static const TypeInfoTable typeInfoTable_[];
     };
 
+    /*!
+      @brief Auxiliary type to enable copies and assignments, similar to
+             std::auto_ptr_ref. See http://www.josuttis.com/libbook/auto_ptr.html
+             for a discussion.
+     */
+    struct DataBufRef {
+        //! Constructor
+        DataBufRef(std::pair<byte*, long> rhs) : p(rhs) {}
+        //! Pointer to a byte array and its size
+        std::pair<byte*, long> p;
+    };
+
     /*!
       @brief Utility class containing a character array. All it does is to take
              care of memory allocation and deletion. Its primary use is meant to
              be as a stack variable in functions that need a temporary data
              buffer. Todo: this should be some sort of smart pointer,
-             essentially an std:auto_ptr for a character array. But it isn't.
+             essentially an std::auto_ptr for a character array. But it isn't...
      */
     class DataBuf {
-        // Not implemented
-        //! Copy constructor
-        DataBuf(const DataBuf&);
-        //! Assignment operator
-        DataBuf& operator=(const DataBuf&);
     public:
+        //! @name Creators
+        //@{
         //! Default constructor
-        DataBuf() : size_(0), pData_(0) {}
+        DataBuf() : pData_(0), size_(0) {}
         //! Constructor with an initial buffer size 
-        DataBuf(long size) : size_(size), pData_(new byte[size]) {}
+        explicit DataBuf(long size) : pData_(new byte[size]), size_(size) {}
+        //! Constructor, copies an existing buffer
+        DataBuf(byte* pData, long size);
+        /*! 
+          @brief Copy constructor. Transfers the buffer to the newly created 
+                 object similar to std::auto_ptr, i.e., the original object is
+                 modified.
+         */
+        DataBuf(DataBuf& rhs);
         //! Destructor, deletes the allocated buffer
         ~DataBuf() { delete[] pData_; }
+        //@}
+
+        //! @name Manipulators
+        //@{
+        /*!
+          @brief Assignment operator. Transfers the buffer and releases the
+                 buffer at the original object similar to std::auto_ptr, i.e., 
+                 the original object is modified.
+         */
+        DataBuf& operator=(DataBuf& rhs);
         //! Allocate a data buffer of the given size
-        void alloc(long size)
-            { if( size > size_ ){delete[] pData_; size_ = size; pData_ = new byte[size];} }
-        //! The current size of the buffer
-        long size_; 
+        void alloc(long size);
+        /*!
+          @brief Release ownership of the buffer to the caller. Returns the 
+                 buffer as a data pointer and size pair, resets the internal
+                 buffer.
+         */
+        std::pair<byte*, long> release();
+        //! Reset value
+        void reset(std::pair<byte*, long> =std::make_pair(0,0));
+        //@}
+
+        /*!
+          @name Conversions
+
+          Special conversions with auxiliary type to enable copies 
+          and assignments, similar to those used for std::auto_ptr.
+          See http://www.josuttis.com/libbook/auto_ptr.html for a discussion.
+         */
+        //@{
+        DataBuf(DataBufRef rhs) : pData_(rhs.p.first), size_(rhs.p.second) {}
+        DataBuf& operator=(DataBufRef rhs) { reset(rhs.p); return *this; }
+        operator DataBufRef() { return DataBufRef(release()); }
+        //@}
+
+        // DATA
         //! Pointer to the buffer, 0 if none has been allocated
         byte* pData_;
+        //! The current size of the buffer
+        long size_; 
     }; // class DataBuf
 
+
 // *****************************************************************************
 // free functions
 
     //! Read a 2 byte unsigned short value from the data buffer
-    uint16 getUShort(const byte* buf, ByteOrder byteOrder);
+    uint16_t getUShort(const byte* buf, ByteOrder byteOrder);
     //! Read a 4 byte unsigned long value from the data buffer
-    uint32 getULong(const byte* buf, ByteOrder byteOrder);
+    uint32_t getULong(const byte* buf, ByteOrder byteOrder);
     //! Read an 8 byte unsigned rational value from the data buffer
     URational getURational(const byte* buf, ByteOrder byteOrder);
     //! Read a 2 byte signed short value from the data buffer
-    int16 getShort(const byte* buf, ByteOrder byteOrder);
+    int16_t getShort(const byte* buf, ByteOrder byteOrder);
     //! Read a 4 byte signed long value from the data buffer
-    int32 getLong(const byte* buf, ByteOrder byteOrder);
+    int32_t getLong(const byte* buf, ByteOrder byteOrder);
     //! Read an 8 byte signed rational value from the data buffer
     Rational getRational(const byte* buf, ByteOrder byteOrder);
 
@@ -177,12 +235,12 @@ namespace Exiv2 {
       @brief Convert an unsigned short to data, write the data to the buffer, 
              return number of bytes written.
      */
-    long us2Data(byte* buf, uint16 s, ByteOrder byteOrder);
+    long us2Data(byte* buf, uint16_t s, ByteOrder byteOrder);
     /*!
       @brief Convert an unsigned long to data, write the data to the buffer,
              return number of bytes written.
      */
-    long ul2Data(byte* buf, uint32 l, ByteOrder byteOrder);
+    long ul2Data(byte* buf, uint32_t l, ByteOrder byteOrder);
     /*!
       @brief Convert an unsigned rational to data, write the data to the buffer,
              return number of bytes written.
@@ -192,12 +250,12 @@ namespace Exiv2 {
       @brief Convert a signed short to data, write the data to the buffer, 
              return number of bytes written.
      */
-    long s2Data(byte* buf, int16 s, ByteOrder byteOrder);
+    long s2Data(byte* buf, int16_t s, ByteOrder byteOrder);
     /*!
       @brief Convert a signed long to data, write the data to the buffer,
              return number of bytes written.
      */
-    long l2Data(byte* buf, int32 l, ByteOrder byteOrder);
+    long l2Data(byte* buf, int32_t l, ByteOrder byteOrder);
     /*!
       @brief Convert a signed rational to data, write the data to the buffer,
              return number of bytes written.
@@ -222,7 +280,16 @@ namespace Exiv2 {
              Both parameters must be greater than 0.
      */
     long lgcd(long a, long b);
-   
+
+    /*!
+      @brief Return true if str is a hex number starting with prefix followed
+             by size hex digits, false otherwise. If size is 0, any number of 
+             digits is allowed and all are checked.
+     */
+    bool isHex(const std::string& str, 
+               size_t size =0,
+               const std::string& prefix ="");
+
 // *****************************************************************************
 // template and inline definitions
 
