@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004, 2005 Andreas Huggel <ahuggel@gmx.net>
  * 
  * This program is part of the Exiv2 distribution.
  *
@@ -21,7 +21,7 @@
 /*!
   @file    value.hpp
   @brief   Value interface and concrete subclasses
-  @version Exiv2-0_5_040913 1.15
+  @version $Rev: 560 $
   @author  Andreas Huggel (ahu)
            <a href="mailto:ahuggel@gmx.net">ahuggel@gmx.net</a>
   @date    09-Jan-04, ahu: created
@@ -40,6 +40,7 @@
 #include <vector>
 #include <iostream>
 #include <sstream>
+#include <memory>
 
 // *****************************************************************************
 // namespace extensions
@@ -59,6 +60,9 @@ namespace Exiv2 {
      */
     class Value {
     public:
+        //! Shortcut for a %Value auto pointer.
+        typedef std::auto_ptr<Value> AutoPtr;
+
         //! @name Creators
         //@{
         //! Constructor, taking a type id to initialize the base class with
@@ -90,6 +94,19 @@ namespace Exiv2 {
           @param buf The string to read from.
          */
         virtual void read(const std::string& buf) =0;
+        /*!
+          @brief Set the data area, if the value has one by copying (cloning)
+                 the buffer pointed to by buf.
+
+          Values may have a data area, which can contain additional
+          information besides the actual value. This method is used to set such
+          a data area.
+
+          @param buf Pointer to the source data area
+          @param len Size of the data area
+          @return Return -1 if the value has no data area, else 0.
+         */
+        virtual int setDataArea(const byte* buf, long len) { return -1; }
         //@}
 
         //! @name Accessors
@@ -101,6 +118,12 @@ namespace Exiv2 {
                  write(std::ostream& os) const of the concrete class. 
          */
         std::string toString() const;
+        /*!
+          @brief Return an auto-pointer to a copy of itself (deep copy).
+                 The caller owns this copy and the auto-pointer ensures that 
+                 it will be deleted.
+         */
+        AutoPtr clone() const { return AutoPtr(clone_()); }
         /*!
           @brief Write value to a data buffer.
 
@@ -116,11 +139,6 @@ namespace Exiv2 {
         virtual long count() const =0;
         //! Return the size of the value in bytes
         virtual long size() const =0;
-        /*!
-          @brief Return a pointer to a copy of itself (deep copy).
-                 The caller owns this copy and is responsible to delete it!
-         */
-        virtual Value* clone() const =0;
         /*! 
           @brief Write the value to an output stream. You do not usually have
                  to use this function; it is used for the implementation of 
@@ -152,6 +170,21 @@ namespace Exiv2 {
           @return The converted value. 
          */
         virtual Rational toRational(long n =0) const =0;
+        //! Return the size of the data area, 0 if there is none.
+        virtual long sizeDataArea() const { return 0; }
+        /*!
+          @brief Return a copy of the data area if the value has one. The
+                 caller owns this copy and DataBuf ensures that it will be
+                 deleted. 
+
+          Values may have a data area, which can contain additional
+          information besides the actual value. This method is used to access
+          such a data area.
+
+          @return A DataBuf containing a copy of the data area or an empty
+                  DataBuf if the value does not have a data area assigned.
+         */
+        virtual DataBuf dataArea() const { return DataBuf(0, 0); };
         //@}
 
         /*!
@@ -159,29 +192,30 @@ namespace Exiv2 {
 
           The following Value subclasses are created depending on typeId:<BR><BR>
           <TABLE>
-          <TR><TD><B>typeId</B></TD><TD><B>%Value subclass</B></TD></TR>
-          <TR><TD>invalidTypeId</TD><TD>%DataValue(invalidTypeId)</TD></TR>
-          <TR><TD>unsignedByte</TD><TD>%DataValue(unsignedByte)</TD></TR>
-          <TR><TD>asciiString</TD><TD>%AsciiValue</TD></TR>
-          <TR><TD>string</TD><TD>%StringValue</TD></TR>
-          <TR><TD>unsignedShort</TD><TD>%ValueType &lt; uint16 &gt;</TD></TR>
-          <TR><TD>unsignedLong</TD><TD>%ValueType &lt; uint32 &gt;</TD></TR>
-          <TR><TD>unsignedRational</TD><TD>%ValueType &lt; URational &gt;</TD></TR>
-          <TR><TD>invalid6</TD><TD>%DataValue(invalid6)</TD></TR>
-          <TR><TD>undefined</TD><TD>%DataValue</TD></TR>
-          <TR><TD>signedShort</TD><TD>%ValueType &lt; int16 &gt;</TD></TR>
-          <TR><TD>signedLong</TD><TD>%ValueType &lt; int32 &gt;</TD></TR>
-          <TR><TD>signedRational</TD><TD>%ValueType &lt; Rational &gt;</TD></TR>
-          <TR><TD>date</TD><TD>%DateValue</TD></TR>
-          <TR><TD>time</TD><TD>%TimeValue</TD></TR>
-          <TR><TD><EM>default:</EM></TD><TD>%DataValue(typeId)</TD></TR>
+          <TR><TD class="indexkey"><B>typeId</B></TD><TD class="indexvalue"><B>%Value subclass</B></TD></TR>
+          <TR><TD class="indexkey">invalidTypeId</TD><TD class="indexvalue">%DataValue(invalidTypeId)</TD></TR>
+          <TR><TD class="indexkey">unsignedByte</TD><TD class="indexvalue">%DataValue(unsignedByte)</TD></TR>
+          <TR><TD class="indexkey">asciiString</TD><TD class="indexvalue">%AsciiValue</TD></TR>
+          <TR><TD class="indexkey">string</TD><TD class="indexvalue">%StringValue</TD></TR>
+          <TR><TD class="indexkey">unsignedShort</TD><TD class="indexvalue">%ValueType &lt; uint16_t &gt;</TD></TR>
+          <TR><TD class="indexkey">unsignedLong</TD><TD class="indexvalue">%ValueType &lt; uint32_t &gt;</TD></TR>
+          <TR><TD class="indexkey">unsignedRational</TD><TD class="indexvalue">%ValueType &lt; URational &gt;</TD></TR>
+          <TR><TD class="indexkey">invalid6</TD><TD class="indexvalue">%DataValue(invalid6)</TD></TR>
+          <TR><TD class="indexkey">undefined</TD><TD class="indexvalue">%DataValue</TD></TR>
+          <TR><TD class="indexkey">signedShort</TD><TD class="indexvalue">%ValueType &lt; int16_t &gt;</TD></TR>
+          <TR><TD class="indexkey">signedLong</TD><TD class="indexvalue">%ValueType &lt; int32_t &gt;</TD></TR>
+          <TR><TD class="indexkey">signedRational</TD><TD class="indexvalue">%ValueType &lt; Rational &gt;</TD></TR>
+          <TR><TD class="indexkey">date</TD><TD class="indexvalue">%DateValue</TD></TR>
+          <TR><TD class="indexkey">time</TD><TD class="indexvalue">%TimeValue</TD></TR>
+          <TR><TD class="indexkey">comment</TD><TD class="indexvalue">%CommentValue</TD></TR>
+          <TR><TD class="indexkey"><EM>default:</EM></TD><TD class="indexvalue">%DataValue(typeId)</TD></TR>
           </TABLE>
 
           @param typeId Type of the value.
-          @return Pointer to the newly created Value.
-                  The caller owns this copy and is responsible to delete it!
+          @return Auto-pointer to the newly created Value. The caller owns this
+                  copy and the auto-pointer ensures that it will be deleted.
          */
-        static Value* create(TypeId typeId);
+        static AutoPtr create(TypeId typeId);
 
     protected:
         /*!
@@ -191,6 +225,9 @@ namespace Exiv2 {
         Value& operator=(const Value& rhs);
 
     private:
+        //! Internal virtual copy constructor.
+        virtual Value* clone_() const =0;
+        // DATA
         TypeId type_;                           //!< Type of the data
 
     }; // class Value
@@ -204,6 +241,9 @@ namespace Exiv2 {
     //! %Value for an undefined data type.
     class DataValue : public Value {
     public:
+        //! Shortcut for a %DataValue auto pointer.
+        typedef std::auto_ptr<DataValue> AutoPtr;
+
         //! @name Creators
         //@{
         //! Default constructor.
@@ -239,7 +279,8 @@ namespace Exiv2 {
         //@}
 
         //! @name Accessors
-        //@{        
+        //@{
+        AutoPtr clone() const { return AutoPtr(clone_()); }
         /*!
           @brief Write value to a character data buffer. 
 
@@ -256,7 +297,6 @@ namespace Exiv2 {
         virtual long copy(byte* buf, ByteOrder byteOrder =invalidByteOrder) const;
         virtual long count() const { return size(); }
         virtual long size() const;
-        virtual DataValue* clone() const;
         virtual std::ostream& write(std::ostream& os) const;
         virtual long toLong(long n =0) const { return value_[n]; }
         virtual float toFloat(long n =0) const { return value_[n]; }
@@ -265,6 +305,9 @@ namespace Exiv2 {
         //@}
 
     private:
+        //! Internal virtual copy constructor.
+        virtual DataValue* clone_() const;
+        // DATA
         std::vector<byte> value_;
 
     }; // class DataValue
@@ -277,6 +320,9 @@ namespace Exiv2 {
      */
     class StringValueBase : public Value {
     public:
+        //! Shortcut for a %StringValueBase auto pointer.
+        typedef std::auto_ptr<StringValueBase> AutoPtr;
+
         //! @name Creators
         //@{
         //! Constructor for subclasses
@@ -316,6 +362,7 @@ namespace Exiv2 {
 
         //! @name Accessors
         //@{
+        AutoPtr clone() const { return AutoPtr(clone_()); }
         /*!
           @brief Write value to a character data buffer.
 
@@ -337,11 +384,12 @@ namespace Exiv2 {
         virtual Rational toRational(long n =0) const
             { return Rational(value_[n], 1); }
         virtual std::ostream& write(std::ostream& os) const;
-
-        virtual StringValueBase* clone() const =0;
         //@}
 
     protected:
+        //! Internal virtual copy constructor.
+        virtual StringValueBase* clone_() const =0;
+        // DATA
         std::string value_;                     //!< Stores the string value. 
 
     }; // class StringValueBase
@@ -355,6 +403,9 @@ namespace Exiv2 {
     */
     class StringValue : public StringValueBase {
     public:
+        //! Shortcut for a %StringValue auto pointer.
+        typedef std::auto_ptr<StringValue> AutoPtr;
+
         //! @name Creators
         //@{
         //! Default constructor.
@@ -377,9 +428,13 @@ namespace Exiv2 {
 
         //! @name Accessors
         //@{
-        virtual StringValue* clone() const;
+        AutoPtr clone() const { return AutoPtr(clone_()); }
         //@}
 
+    private:
+        //! Internal virtual copy constructor.
+        virtual StringValue* clone_() const;
+
     }; // class StringValue
 
     /*!
@@ -390,6 +445,9 @@ namespace Exiv2 {
      */
     class AsciiValue : public StringValueBase {
     public:
+        //! Shortcut for a %AsciiValue auto pointer.
+        typedef std::auto_ptr<AsciiValue> AutoPtr;
+
         //! @name Creators
         //@{
         //! Default constructor.
@@ -409,20 +467,6 @@ namespace Exiv2 {
         //@{
         //! Assignment operator
         AsciiValue& operator=(const AsciiValue& rhs);
-        /*!
-          @brief Read the value from a character buffer. Appends a terminating
-                 '\\0' character if buf doesn't end with 0.
-
-          @note The byte order is required by the interface but not used by this
-                method, so just use the default.
-
-          @param buf Pointer to the data buffer to read from
-          @param len Number of bytes in the data buffer 
-          @param byteOrder Byte order. Not needed.
-         */
-        virtual void read(const byte* buf, 
-                          long len, 
-                          ByteOrder byteOrder =invalidByteOrder);
         /*!
           @brief Set the value to that of the string buf. Overrides base class
                  to append a terminating '\\0' character if buf doesn't end
@@ -433,7 +477,7 @@ namespace Exiv2 {
 
         //! @name Accessors
         //@{
-        virtual AsciiValue* clone() const;
+        AutoPtr clone() const { return AutoPtr(clone_()); }
         /*! 
           @brief Write the value to an output stream. Any trailing '\\0'
                  characters of the ASCII value are stripped and not written to
@@ -441,8 +485,114 @@ namespace Exiv2 {
         */
         virtual std::ostream& write(std::ostream& os) const;
         //@}
+
+    private:
+        //! Internal virtual copy constructor.
+        virtual AsciiValue* clone_() const;
+
     }; // class AsciiValue
 
+    /*!
+      @brief %Value for an Exif comment.
+
+      This can be a plain Ascii string or a multipe byte encoded string. The
+      comment is expected to be encoded in the character set indicated (default
+      undefined), but this is not checked. It is left to caller to decode and
+      encode the string to and from readable text if that is required.
+    */
+    class CommentValue : public StringValueBase {
+    public:
+        //! Character set identifiers for the character sets defined by %Exif
+        enum CharsetId { ascii, jis, unicode, undefined,
+                         invalidCharsetId, lastCharsetId };
+        //! Information pertaining to the defined character sets
+        struct CharsetTable {
+            //! Constructor
+            CharsetTable(CharsetId charsetId, 
+                         const char* name, 
+                         const char* code);
+            CharsetId charsetId_;                   //!< Charset id
+            const char* name_;                      //!< Name of the charset
+            const char* code_;                      //!< Code of the charset 
+        }; // struct CharsetTable
+        //! Charset information lookup functions. Implemented as a static class.
+        class CharsetInfo {
+            //! Prevent construction: not implemented.
+            CharsetInfo() {}
+            //! Prevent copy-construction: not implemented.
+            CharsetInfo(const CharsetInfo&);
+            //! Prevent assignment: not implemented.
+            CharsetInfo& operator=(const CharsetInfo&);
+            
+        public:
+            //! Return the name for a charset id
+            static const char* name(CharsetId charsetId);
+            //! Return the code for a charset id
+            static const char* code(CharsetId charsetId);
+            //! Return the charset id for a name
+            static CharsetId charsetIdByName(const std::string& name);
+            //! Return the charset id for a code
+            static CharsetId charsetIdByCode(const std::string& code);
+            
+        private:
+            static const CharsetTable charsetTable_[];
+        }; // class CharsetInfo
+
+        //! Shortcut for a %CommentValue auto pointer.
+        typedef std::auto_ptr<CommentValue> AutoPtr;
+
+        //! @name Creators
+        //@{
+        //! Default constructor.
+        CommentValue()
+            : StringValueBase(Exiv2::undefined) {}
+        //! Constructor, uses read(const std::string& comment)
+        CommentValue(const std::string& comment);
+        //! Copy constructor 
+        CommentValue(const CommentValue& rhs)
+            : StringValueBase(rhs) {}
+        //! Virtual destructor.
+        virtual ~CommentValue() {}
+        //@}
+
+        //! @name Manipulators
+        //@{
+        //! Assignment operator.
+        CommentValue& operator=(const CommentValue& rhs);
+        /*!
+          @brief Read the value from a comment
+ 
+          The format of \em comment is:
+          <BR>
+          <CODE>[charset=["]Ascii|Jis|Unicode|Undefined["] ]comment</CODE> 
+          <BR>
+          The default charset is Undefined.
+
+          @throw Error if an invalid character set is encountered
+        */
+        void read(const std::string& comment);
+        //@}
+
+        //! @name Accessors
+        //@{
+        AutoPtr clone() const { return AutoPtr(clone_()); }
+        /*!
+          @brief Write the comment in a format which can be read by 
+          read(const std::string& comment).
+         */
+        std::ostream& write(std::ostream& os) const;
+        //! Return the comment (without a charset="..." prefix)
+        std::string comment() const;
+        //! Return the charset id of the comment
+        CharsetId charsetId() const;
+        //@}
+
+    private:
+        //! Internal virtual copy constructor.
+        virtual CommentValue* clone_() const;
+
+    }; // class CommentValue
+
     /*! 
       @brief %Value for simple ISO 8601 dates
 
@@ -451,6 +601,9 @@ namespace Exiv2 {
      */
     class DateValue : public Value {
     public:
+        //! Shortcut for a %DateValue auto pointer.
+        typedef std::auto_ptr<DateValue> AutoPtr;
+
         //! @name Creators
         //@{
         //! Default constructor.
@@ -482,12 +635,18 @@ namespace Exiv2 {
           @param buf Pointer to the data buffer to read from
           @param len Number of bytes in the data buffer 
           @param byteOrder Byte order. Not needed.
+
+          @throw Error in case of an unsupported date format
          */
         virtual void read(const byte* buf, 
                           long len, 
                           ByteOrder byteOrder =invalidByteOrder);
         /*!
           @brief Set the value to that of the string buf. 
+
+          @param buf String containing the date
+
+          @throw Error in case of an unsupported date format
          */
         virtual void read(const std::string& buf);
         //! Set the date
@@ -496,6 +655,7 @@ namespace Exiv2 {
 
         //! @name Accessors
         //@{
+        AutoPtr clone() const { return AutoPtr(clone_()); }
         /*!
           @brief Write value to a character data buffer.
 
@@ -514,7 +674,6 @@ namespace Exiv2 {
         virtual const Date& getDate() const { return date_; }
         virtual long count() const { return size(); }
         virtual long size() const;
-        virtual DateValue* clone() const;
         /*! 
           @brief Write the value to an output stream. .
         */
@@ -527,6 +686,9 @@ namespace Exiv2 {
         //@}
 
     private:
+        //! Internal virtual copy constructor.
+        virtual DateValue* clone_() const;
+        // DATA
         Date date_;
 
     }; // class DateValue    
@@ -541,6 +703,9 @@ namespace Exiv2 {
      */
     class TimeValue : public Value {
     public:
+        //! Shortcut for a %TimeValue auto pointer.
+        typedef std::auto_ptr<TimeValue> AutoPtr;
+
         //! @name Creators
         //@{
         //! Default constructor.
@@ -576,12 +741,18 @@ namespace Exiv2 {
           @param buf Pointer to the data buffer to read from
           @param len Number of bytes in the data buffer 
           @param byteOrder Byte order. Not needed.
+
+          @throw Error in case of an unsupported time format
          */
         virtual void read(const byte* buf, 
                           long len, 
                           ByteOrder byteOrder =invalidByteOrder);
         /*!
           @brief Set the value to that of the string buf. 
+
+          @param buf String containing the time.
+
+          @throw Error in case of an unsupported time format
          */
         virtual void read(const std::string& buf);
         //! Set the time
@@ -590,6 +761,7 @@ namespace Exiv2 {
 
         //! @name Accessors
         //@{
+        AutoPtr clone() const { return AutoPtr(clone_()); }
         /*!
           @brief Write value to a character data buffer.
 
@@ -608,7 +780,6 @@ namespace Exiv2 {
         virtual const Time& getTime() const { return time_; }
         virtual long count() const { return size(); }
         virtual long size() const;
-        virtual TimeValue* clone() const;
         /*! 
           @brief Write the value to an output stream. .
         */
@@ -621,6 +792,9 @@ namespace Exiv2 {
         //@}
 
     private:
+        //! Internal virtual copy constructor.
+        virtual TimeValue* clone_() const;
+        // DATA
         Time time_;
 
     }; // class TimeValue        
@@ -628,15 +802,15 @@ namespace Exiv2 {
     template<typename T> TypeId getType();
 
     //! Specialization for an unsigned short
-    template<> inline TypeId getType<uint16>() { return unsignedShort; }
+    template<> inline TypeId getType<uint16_t>() { return unsignedShort; }
     //! Specialization for an unsigned long
-    template<> inline TypeId getType<uint32>() { return unsignedLong; }
+    template<> inline TypeId getType<uint32_t>() { return unsignedLong; }
     //! Specialization for an unsigned rational
     template<> inline TypeId getType<URational>() { return unsignedRational; }
     //! Specialization for a signed short
-    template<> inline TypeId getType<int16>() { return signedShort; }
+    template<> inline TypeId getType<int16_t>() { return signedShort; }
     //! Specialization for a signed long
-    template<> inline TypeId getType<int32>() { return signedLong; }
+    template<> inline TypeId getType<int32_t>() { return signedLong; }
     //! Specialization for a signed rational
     template<> inline TypeId getType<Rational>() { return signedRational; }
 
@@ -650,17 +824,21 @@ namespace Exiv2 {
     template<typename T>
     class ValueType : public Value {
     public:
+        //! Shortcut for a %ValueType\<T\> auto pointer.
+        typedef std::auto_ptr<ValueType<T> > AutoPtr;
+
         //! @name Creators
         //@{
         //! Default constructor.
-        ValueType() : Value(getType<T>()) {}
+        ValueType() : Value(getType<T>()), pDataArea_(0), sizeDataArea_(0) {}
         //! Constructor
-        ValueType(const byte* buf, long len, ByteOrder byteOrder) 
-            : Value(getType<T>()) { read(buf, len, byteOrder); }
+        ValueType(const byte* buf, long len, ByteOrder byteOrder);
         //! Constructor
-        ValueType( const T& val, ByteOrder byteOrder =littleEndian);
+        ValueType(const T& val, ByteOrder byteOrder =littleEndian);
+        //! Copy constructor
+        ValueType(const ValueType<T>& rhs);
         //! Virtual destructor.
-        virtual ~ValueType() {}
+        virtual ~ValueType();
         //@}
 
         //! @name Manipulators
@@ -675,18 +853,30 @@ namespace Exiv2 {
                  produced by the write() method.
          */
         virtual void read(const std::string& buf);
+        /*!
+          @brief Set the data area. This method copies (clones) the buffer
+                 pointed to by buf.
+         */
+        virtual int setDataArea(const byte* buf, long len);
         //@}
 
         //! @name Accessors
         //@{
+        AutoPtr clone() const { return AutoPtr(clone_()); }
         virtual long copy(byte* buf, ByteOrder byteOrder) const;
         virtual long count() const { return static_cast<long>(value_.size()); }
         virtual long size() const;
-        virtual ValueType<T>* clone() const;
         virtual std::ostream& write(std::ostream& os) const;
         virtual long toLong(long n =0) const;
         virtual float toFloat(long n =0) const;
         virtual Rational toRational(long n =0) const;
+        //! Return the size of the data area.
+        virtual long sizeDataArea() const { return sizeDataArea_; }
+        /*!
+          @brief Return a copy of the data area in a DataBuf. The caller owns
+                 this copy and DataBuf ensures that it will be deleted.
+         */
+        virtual DataBuf dataArea() const;
         //@}
 
         //! Container for values 
@@ -696,6 +886,7 @@ namespace Exiv2 {
         //! Const iterator type defined for convenience.
         typedef typename std::vector<T>::const_iterator const_iterator;
 
+        // DATA
         /*!
           @brief The container for all values. In your application, if you know 
                  what subclass of Value you're dealing with (and possibly the T)
@@ -704,18 +895,27 @@ namespace Exiv2 {
          */
         ValueList value_;
 
+    private:
+        //! Internal virtual copy constructor.
+        virtual ValueType<T>* clone_() const;
+
+        // DATA
+        //! Pointer to the buffer, 0 if none has been allocated
+        byte* pDataArea_;
+        //! The current size of the buffer
+        long sizeDataArea_; 
     }; // class ValueType
 
     //! Unsigned short value type
-    typedef ValueType<uint16> UShortValue;
+    typedef ValueType<uint16_t> UShortValue;
     //! Unsigned long value type
-    typedef ValueType<uint32> ULongValue;
+    typedef ValueType<uint32_t> ULongValue;
     //! Unsigned rational value type
     typedef ValueType<URational> URationalValue;
     //! Signed short value type
-    typedef ValueType<int16> ShortValue;
+    typedef ValueType<int16_t> ShortValue;
     //! Signed long value type
-    typedef ValueType<int32> LongValue;
+    typedef ValueType<int32_t> LongValue;
     //! Signed rational value type
     typedef ValueType<Rational> RationalValue;
 
@@ -736,13 +936,13 @@ namespace Exiv2 {
     template<typename T> T getValue(const byte* buf, ByteOrder byteOrder);
     // Specialization for a 2 byte unsigned short value.
     template<> 
-    inline uint16 getValue(const byte* buf, ByteOrder byteOrder)
+    inline uint16_t getValue(const byte* buf, ByteOrder byteOrder)
     {
         return getUShort(buf, byteOrder);
     }
     // Specialization for a 4 byte unsigned long value.
     template<> 
-    inline uint32 getValue(const byte* buf, ByteOrder byteOrder)
+    inline uint32_t getValue(const byte* buf, ByteOrder byteOrder)
     {
         return getULong(buf, byteOrder);
     }
@@ -754,13 +954,13 @@ namespace Exiv2 {
     }
     // Specialization for a 2 byte signed short value.
     template<> 
-    inline int16 getValue(const byte* buf, ByteOrder byteOrder)
+    inline int16_t getValue(const byte* buf, ByteOrder byteOrder)
     {
         return getShort(buf, byteOrder);
     }
     // Specialization for a 4 byte signed long value.
     template<> 
-    inline int32 getValue(const byte* buf, ByteOrder byteOrder)
+    inline int32_t getValue(const byte* buf, ByteOrder byteOrder)
     {
         return getLong(buf, byteOrder);
     }
@@ -789,7 +989,7 @@ namespace Exiv2 {
              Return the number of bytes written.
      */
     template<> 
-    inline long toData(byte* buf, uint16 t, ByteOrder byteOrder)
+    inline long toData(byte* buf, uint16_t t, ByteOrder byteOrder)
     {
         return us2Data(buf, t, byteOrder);
     }
@@ -798,7 +998,7 @@ namespace Exiv2 {
              Return the number of bytes written.
      */
     template<> 
-    inline long toData(byte* buf, uint32 t, ByteOrder byteOrder)
+    inline long toData(byte* buf, uint32_t t, ByteOrder byteOrder)
     {
         return ul2Data(buf, t, byteOrder);
     }
@@ -816,7 +1016,7 @@ namespace Exiv2 {
              Return the number of bytes written.
      */
     template<> 
-    inline long toData(byte* buf, int16 t, ByteOrder byteOrder)
+    inline long toData(byte* buf, int16_t t, ByteOrder byteOrder)
     {
         return s2Data(buf, t, byteOrder);
     }
@@ -825,7 +1025,7 @@ namespace Exiv2 {
              Return the number of bytes written.
      */
     template<> 
-    inline long toData(byte* buf, int32 t, ByteOrder byteOrder)
+    inline long toData(byte* buf, int32_t t, ByteOrder byteOrder)
     {
         return l2Data(buf, t, byteOrder);
     }
@@ -839,21 +1039,55 @@ namespace Exiv2 {
         return r2Data(buf, t, byteOrder);
     }
 
+    template<typename T>
+    ValueType<T>::ValueType(const byte* buf, long len, ByteOrder byteOrder) 
+        : Value(getType<T>()), pDataArea_(0), sizeDataArea_(0)
+    {
+        read(buf, len, byteOrder);
+    }
+
     template<typename T>
     ValueType<T>::ValueType(const T& val, ByteOrder byteOrder)
-        : Value(getType<T>()) 
+        : Value(getType<T>()), pDataArea_(0), sizeDataArea_(0) 
     {
         read(reinterpret_cast<const byte*>(&val), 
              TypeInfo::typeSize(typeId()), 
              byteOrder); 
     }
 
+    template<typename T>
+    ValueType<T>::ValueType(const ValueType<T>& rhs)
+        : Value(rhs), value_(rhs.value_), pDataArea_(0), sizeDataArea_(0)
+    {
+        if (rhs.sizeDataArea_ > 0) {
+            pDataArea_ = new byte[rhs.sizeDataArea_];
+            memcpy(pDataArea_, rhs.pDataArea_, rhs.sizeDataArea_); 
+            sizeDataArea_ = rhs.sizeDataArea_;        
+        }
+    }
+
+    template<typename T>
+    ValueType<T>::~ValueType()
+    {
+        delete[] pDataArea_;
+    }
+
     template<typename T>
     ValueType<T>& ValueType<T>::operator=(const ValueType<T>& rhs)
     {
         if (this == &rhs) return *this;
         Value::operator=(rhs);
         value_ = rhs.value_;
+
+        byte* tmp = 0;
+        if (rhs.sizeDataArea_ > 0) {
+            tmp = new byte[rhs.sizeDataArea_];
+            memcpy(tmp, rhs.pDataArea_, rhs.sizeDataArea_); 
+        }
+        delete[] pDataArea_;
+        pDataArea_ = tmp;
+        sizeDataArea_ = rhs.sizeDataArea_;
+
         return *this;
     }
 
@@ -895,7 +1129,7 @@ namespace Exiv2 {
     }
 
     template<typename T>
-    ValueType<T>* ValueType<T>::clone() const
+    ValueType<T>* ValueType<T>::clone_() const
     {
         return new ValueType<T>(*this);
     }
@@ -966,6 +1200,26 @@ namespace Exiv2 {
         return Rational(value_[n].first, value_[n].second);
     }
 
+    template<typename T>
+    inline DataBuf ValueType<T>::dataArea() const
+    {
+        return DataBuf(pDataArea_, sizeDataArea_);
+    }
+
+    template<typename T>
+    inline int ValueType<T>::setDataArea(const byte* buf, long len)
+    {
+        byte* tmp = 0;
+        if (len > 0) {
+            tmp = new byte[len];
+            memcpy(tmp, buf, len);
+        }
+        delete[] pDataArea_;
+        pDataArea_ = tmp;
+        sizeDataArea_ = len;
+        return 0;
+    }
+
 }                                       // namespace Exiv2
 
 #endif                                  // #ifndef VALUE_HPP_
